%% MSH_Plot_EQ_3D_allfaults.m
%
% AUTHOR:           Michael W. Swyer
% DATE CREATED:     2015/03/26
% DATE MODIFIED:    
% VERSION:          v.1
%
% PURPOSE:  To construct 3D fault geometry around Mount Saint Helens for
% permeability potential modeling for play fairway analysis
%
% PURPOSE OF MODIFICATION:
% 
% INPUT: Seismic catalogs selected from Pacific Northwest Seismic Network:
%           1) pnsn_relocations_faults_north.csv
%           2) pnsn_relocations_faults_east1.csv
%           3) FocalMech_east2.csv
%           4) FocalMech_south.csv
%           5) pnsn_relocations_faults_west1.csv
%           6) FocalMech_west2.csv
%        Points along fault trace of Goat Mountain spaced ~300m.
%           1) Goat_Mountain_Fault.csv
%
% OUTPUT: Position of fault nodes in UTM Zone 10N meters (XYZ-),
% triangulation of fault nodes (tn-), and unit normal vectors of fault element
% triangles (nvec-).
%
% NOTE:
%
% KEY functions written specifically for this modeling:
%
% Other related functions:
%       deg2utm.m
%       fitNormal.m
%       lowess.m
%       smooth.m
%       R3RSR.m
%       R3R.m
%       SR.m
%       H.m
%       Delaunay2_5D.m
%       PlotTriNormals.m

clear all;close all

show_graph = 0;% Show graph of fit normal (0=no 1=yes)

%% Import Data

%%% north fault %%%
data = importdata('pnsn_relocations_faults_north.csv',',');
for i=2:numel(data.textdata(:,1))
    datetime = strsplit(data.textdata{i,3},' ');
    date = strsplit(datetime{1},'/');
    Date = [str2num(date{1}) str2num(date{2}) str2num(date{3})];
    time = strsplit(datetime{2},':');
    Time = [str2num(time{1}) str2num(time{2}) str2num(time{3})];
    dnum(i-1,1) = datenum(Date(1),Date(2),Date(3),Time(1),Time(2),Time(3));
    lat(i-1,1) = str2num(data.textdata{i,7});
    long(i-1,1) = str2num(data.textdata{i,8});
    z(i-1,1) = (-str2num(data.textdata{i,5})*1000)-(rand*(10^-7));
    m(i-1,1) = str2num(data.textdata{i,6});
end
S = 3.^m;
[x,y,utmzone] = deg2utm(lat,long);
% UTM origin
xo = 555500;
yo = 5123500;
x = x - xo;
y = y - yo;



%%% east fault 1 %%%
dataEF1 = importdata('pnsn_relocations_faults_east1.csv',',');
for i=2:numel(dataEF1.textdata(:,1))
    datetime = strsplit(dataEF1.textdata{i,3},' ');
    date = strsplit(datetime{1},'/');
    Date = [str2num(date{1}) str2num(date{2}) str2num(date{3})];
    time = strsplit(datetime{2},':');
    Time = [str2num(time{1}) str2num(time{2}) str2num(time{3})];
    dnumEF1(i-1,1) = datenum(Date(1),Date(2),Date(3),Time(1),Time(2),Time(3));
    latEF1(i-1,1) = str2num(dataEF1.textdata{i,7});
    longEF1(i-1,1) = str2num(dataEF1.textdata{i,8});
    zEF1(i-1,1) = (-str2num(dataEF1.textdata{i,5})*1000);%-(rand*(10^-7));
    mEF1(i-1,1) = str2num(dataEF1.textdata{i,6});
end
SEF1 = 3.^mEF1;
[xEF1,yEF1,utmzone] = deg2utm(latEF1,longEF1);
% UTM origin
xoEF1 = 573000;
yoEF1 = 5122000;
xEF1 = xEF1 - xoEF1;
yEF1 = yEF1 - yoEF1;
% fit normal vector
if show_graph==1
    figure
end
clear c;clear n
c(:,1) = xEF1;c(:,2) = yEF1;c(:,3) = zEF1;
n = fitNormal(c, show_graph);
thetaEF1 = atan(n(2)/n(1));
% rotate to j=0
Rcw = [cos(thetaEF1) sin(thetaEF1);-sin(thetaEF1) cos(thetaEF1)];
for i=1:numel(xEF1)
    b = Rcw*[xEF1(i);yEF1(i)];
    xEF1(i) = b(1)+400;yEF1(i) = b(2);
end
% rotate about y-axis
zEF1 = zEF1 + 12000;xEF1 = xEF1 - 500;
Rcw = [cos(7*(pi/180)) sin(7*(pi/180));-sin(7*(pi/180)) cos(7*(pi/180))];
for i=1:numel(xEF1)
    b = Rcw*[xEF1(i);zEF1(i)];
    xEF1(i) = b(1);zEF1(i) = b(2);
end
zEF1 = zEF1 - 12000;xEF1 = xEF1 + 500;



%%% east fault 2 %%%
dataEF2 = importdata('FocalMech_east2.csv',',');
for i=2:numel(dataEF2)
    d = strsplit(dataEF2{i},',');
    datetime = strsplit(d{5});
    date = strsplit(datetime{1},'/');
    Date = [str2num(date{3}) str2num(date{1}) str2num(date{2})];
    dnumEF2(i-1,1) = datenum(Date(1),Date(2),Date(3));
    latEF2(i-1,1) = str2num(d{6}) + (rand*(10^-7));
    longEF2(i-1,1) = str2num(d{7}) + (rand*(10^-7));
    mEF2(i-1,1) = str2num(d{8});
    zEF2(i-1,1) = -str2num(d{9})*1000 - rand;
end
SEF2 = 3.^mEF2;
[xEF2,yEF2,utmzone] = deg2utm(latEF2,longEF2);
% UTM origin
xoEF2 = 567000;
yoEF2 = 5119500;
xEF2 = xEF2 - xoEF2;
yEF2 = yEF2 - yoEF2;
% fit normal vector
if show_graph==1
    figure
end
clear c;clear n
c(:,1) = xEF2;c(:,2) = yEF2;c(:,3) = zEF2;
n = fitNormal(c, show_graph);
thetaEF2 = atan(n(2)/n(1));
% rotate to j=0
Rcw = [cos(thetaEF2) sin(thetaEF2);-sin(thetaEF2) cos(thetaEF2)];
for i=1:numel(xEF2)
    b = Rcw*[xEF2(i);yEF2(i)];
    xEF2(i) = b(1);yEF2(i) = b(2);
end



%%% south fault %%%
dataSF = importdata('FocalMech_south.csv',',');
for i=2:numel(dataSF)
    d = strsplit(dataSF{i},',');
    datetime = strsplit(d{5});
    date = strsplit(datetime{1},'/');
    Date = [str2num(date{3}) str2num(date{1}) str2num(date{2})];
    dnumSF(i-1,1) = datenum(Date(1),Date(2),Date(3));
    latSF(i-1,1) = str2num(d{6}) + (rand*(10^-7));
    longSF(i-1,1) = str2num(d{7}) + (rand*(10^-7));
    mSF(i-1,1) = str2num(d{8});
    if mSF(i-1,1)<1
        mSF(i-1,1)=1;
    end
    zSF(i-1,1) = -str2num(d{9})*1000 - rand;
end
SSF = 3.^mSF;
[xSF,ySF,utmzone] = deg2utm(latSF,longSF);
% UTM origin
xoSF = 564000;
yoSF = 5097500;
xSF = xSF - xoSF;
ySF = ySF - yoSF;
% fit normal vector
if show_graph==1
    figure
end
clear c;clear n
c(:,1) = xSF;c(:,2) = ySF;c(:,3) = zSF;
n = fitNormal(c, show_graph);
thetaSF = atan(n(2)/n(1));
% rotate to j=0
Rcw = [cos(thetaSF) sin(thetaSF);-sin(thetaSF) cos(thetaSF)];
for i=1:numel(xSF)
    b = Rcw*[xSF(i);ySF(i)];
    xSF(i) = b(1)-4000;ySF(i) = b(2)+3000;
end
% rotate about y-axis
zSF = zSF + 8000;xSF = xSF - 1750;
Rcw = [cos(17*(pi/180)) sin(17*(pi/180));-sin(17*(pi/180)) cos(17*(pi/180))];
for i=1:numel(xSF)
    b = Rcw*[xSF(i);zSF(i)];
    xSF(i) = b(1);zSF(i) = b(2);
end
zSF = zSF - 8000;xSF = xSF + 1750;



%%% west fault 1 %%%
dataWF1 = importdata('pnsn_relocations_faults_west1.csv',',');
for i=2:numel(dataWF1.textdata(:,1))
    datetime = strsplit(dataWF1.textdata{i,3},' ');
    date = strsplit(datetime{1},'/');
    Date = [str2num(date{1}) str2num(date{2}) str2num(date{3})];
    time = strsplit(datetime{2},':');
    Time = [str2num(time{1}) str2num(time{2}) str2num(time{3})];
    dnumWF1(i-1,1) = datenum(Date(1),Date(2),Date(3),Time(1),Time(2),Time(3));
    latWF1(i-1,1) = str2num(dataWF1.textdata{i,7});
    longWF1(i-1,1) = str2num(dataWF1.textdata{i,8});
    zWF1(i-1,1) = (-str2num(dataWF1.textdata{i,5})*1000);%-(rand*(10^-7));
    mWF1(i-1,1) = str2num(dataWF1.textdata{i,6});
end
SWF1 = 3.^mWF1;
[xWF1,yWF1,utmzone] = deg2utm(latWF1,longWF1);
% UTM origin
xoWF1 = 548500;
yoWF1 = 5113000;
xWF1 = xWF1 - xoWF1;
yWF1 = yWF1 - yoWF1;
% fit normal vector
if show_graph==1
    figure
end
clear c;clear n
c(:,1) = xWF1;c(:,2) = yWF1;c(:,3) = zWF1;
n = fitNormal(c, show_graph);
thetaWF1 = atan(n(2)/n(1));
% rotate to j=0
Rcw = [cos(thetaWF1) sin(thetaWF1);-sin(thetaWF1) cos(thetaWF1)];
for i=1:numel(xWF1)
    b = Rcw*[xWF1(i);yWF1(i)];
    xWF1(i) = b(1);yWF1(i) = b(2);
end
if show_graph==1
    figure
end
clear c;clear n
c(:,1) = xWF1;c(:,2) = yWF1;c(:,3) = zWF1;
n = fitNormal(c, show_graph);
theta2WF1 = atan(n(3)/n(1));



%%% west fault 2 %%%
dataWF2 = importdata('FocalMech_west2.csv',',');
for i=2:numel(dataWF2)
    d = strsplit(dataWF2{i},',');
    datetime = strsplit(d{5});
    date = strsplit(datetime{1},'/');
    Date = [str2num(date{3}) str2num(date{1}) str2num(date{2})];
    dnumWF2(i-1,1) = datenum(Date(1),Date(2),Date(3));
    latWF2(i-1,1) = str2num(d{6}) + (rand*(10^-7));
    longWF2(i-1,1) = str2num(d{7}) + (rand*(10^-7));
    mWF2(i-1,1) = str2num(d{8});
    if mWF2(i-1,1)<1
        mWF2(i-1,1)=1;
    end
    zWF2(i-1,1) = -str2num(d{9})*1000 - rand;
end
SWF2 = 3.^mWF2;
[xWF2,yWF2,utmzone] = deg2utm(latWF2,longWF2);
% UTM origin
xoWF2 = 545500;
yoWF2 = 5128000;
xWF2 = xWF2 - xoWF2;
yWF2 = yWF2 - yoWF2;
% fit normal vector
if show_graph==1
    figure
end
clear c;clear n
c(:,1) = xWF2;c(:,2) = yWF2;c(:,3) = zWF2;
n = fitNormal(c, show_graph);
thetaWF2 = atan(n(2)/n(1));
% rotate to j=0
Rcw = [cos(thetaWF2) sin(thetaWF2);-sin(thetaWF2) cos(thetaWF2)];
for i=1:numel(xWF2)
    b = Rcw*[xWF2(i);yWF2(i)];
    xWF2(i) = b(1)-500;yWF2(i) = b(2)+1000;
end


%% filter data
% by time
% north fault
J = find(dnum>datenum(1980,1,1)&dnum<datenum(1990,1,1));% 1980s
K = find(dnum>datenum(1990,1,1));% 1990s & 2000s

% east fault 1
JEF1 = find(dnumEF1>datenum(1980,1,1)&dnumEF1<datenum(1990,1,1));% 1980s
KEF1 = find(dnumEF1>datenum(1990,1,1));% 1990s & 2000s

% east fault 2
JEF2 = find(dnumEF2<datenum(1990,1,1));% 1980s
KEF2 = find(dnumEF2>datenum(1990,1,1));% 1990s & 2000s

% south fault
JSF = find(dnumSF<datenum(1990,1,1));% 1980s
KSF = find(dnumSF>datenum(1990,1,1));% 1990s & 2000s

% west fault 1
JWF1 = find(dnumWF1<datenum(1990,1,1));% 1980s
KWF1 = find(dnumWF1>datenum(1990,1,1));% 1990s & 2000s

% west fault 1
JWF2 = find(dnumWF2<datenum(1990,1,1));% 1980s
KWF2 = find(dnumWF2>datenum(1990,1,1));% 1990s & 2000s

%% Plot 3D EQs
% north fault
figure
h1 = scatter3(x(J),y(J),z(J),S(J),'k');
hold on
h2 = scatter3(x(K),y(K),z(K),S(K),'r');
legend([h1 h2],'1980s','1990s & 2000s');
axis equal;grid on;
xlim([0 8000])
ylim([0 18000])
zlim([-14000 0])
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title('North Fault','fontsize',14)

% east fault 1
figure
h1 = scatter3(xEF1(JEF1),yEF1(JEF1),zEF1(JEF1),SEF1(JEF1),'k');
hold on
h2 = scatter3(xEF1(KEF1),yEF1(KEF1),zEF1(KEF1),SEF1(KEF1),'r');
legend([h1 h2],'1980s','1990s & 2000s');
axis equal;grid on;
xlim([0 1000])
ylim([0 2500])
zlim([-14000 -10000])
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title(strcat('East Fault 1',' Strike=',num2str(round(180-thetaEF1*(180/pi)))),'fontsize',14)

% east fault 2
figure
h1 = scatter3(xEF2(JEF2),yEF2(JEF2),zEF2(JEF2),SEF2(JEF2),'k');
hold on
h2 = scatter3(xEF2(KEF2),yEF2(KEF2),zEF2(KEF2),SEF2(KEF2),'r');
legend([h1 h2],'1980s','1990s & 2000s');
axis equal;grid on;
xlim([0 2000])
ylim([0 2500])
zlim([-11000 -8000])
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title(strcat('East Fault 2',' Strike=',num2str(round(180-thetaEF2*(180/pi)))),'fontsize',14)

% south fault
figure
h1 = scatter3(xSF(JSF),ySF(JSF),zSF(JSF),SSF(JSF),'k');
hold on
h2 = scatter3(xSF(KSF),ySF(KSF),zSF(KSF),SSF(KSF),'r');
legend([h1 h2],'1980s','1990s & 2000s');
axis equal;grid on;
xlim([0 5000])
ylim([0 17500])
zlim([-13000 -3000])
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title(strcat('South Fault 1 ',' Strike=',num2str(round(180-thetaSF*(180/pi)))),'fontsize',14)

% west fault 1
figure
h1 = scatter3(xWF1(JWF1),yWF1(JWF1),zWF1(JWF1),SWF1(JWF1),'k');
hold on
h2 = scatter3(xWF1(KWF1),yWF1(KWF1),zWF1(KWF1),SWF1(KWF1),'r');
legend([h1 h2],'1980s','1990s & 2000s');
axis equal;grid on;
xlim([0 5500])
ylim([0 5500])
zlim([-13000 -9000])
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title(strcat('West Fault 1 ',' Strike=',num2str(round(-thetaWF1*(180/pi)+180))),'fontsize',14)
% rotate about y-axis
zWF1 = zWF1 + 11000;xWF1 = xWF1 - 2750;
Rcw = [cos(theta2WF1) sin(theta2WF1);-sin(theta2WF1) cos(theta2WF1)];
for i=1:numel(xWF1)
    b = Rcw*[xWF1(i);zWF1(i)];
    xWF1(i) = b(1);zWF1(i) = b(2);
end
zWF1 = zWF1 - 11000;xWF1 = xWF1 + 2750;

% west fault 2
figure
h1 = scatter3(xWF2(JWF2),yWF2(JWF2),zWF2(JWF2),SWF2(JWF2),'k');
hold on
h2 = scatter3(xWF2(KWF2),yWF2(KWF2),zWF2(KWF2),SWF2(KWF2),'r');
legend([h1 h2],'1980s','1990s & 2000s');
axis equal;grid on;
xlim([0 4000])
ylim([0 4000])
zlim([-16500 -12000])
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title(strcat('West Fault 2 ',' Strike=',num2str(round(thetaWF2*(180/pi)+270))),'fontsize',14)

%% North Fault

% cross sections

% remove events below 14km depth
Q = find(z>-14000);
x = x(Q);y = y(Q);z = z(Q);

% depth slices for node locations
depths = [-13900:300:-1000];depths = depths';
% northing coordinates for cross-sections/node locations
norths = [900:300:13800];% 300m spacing

q = 2;% moving window, # of cells on each side

for i=1:numel(norths)
    if norths(i)<4500
        I = find(y>(norths(i)-3500)&y<=(norths(i)+2500));
        J = find(dnum(I)>datenum(1980,1,1)&dnum(I)<datenum(1990,1,1));% 1980s
        K = find(dnum(I)>datenum(1990,1,1));% 1990s & 2000s
        figure
        scatter(x(I(J)),z(I(J)),S(I(J)),'k');
        hold on
        scatter(x(I(K)),z(I(K)),S(I(K)),'r');
        axis equal;grid on;
        xlim([0 8000])
        ylim([-14000 0])
        xlabel('E-W(m)','fontsize',12)
        ylabel('depth(m)','fontsize',12)
        title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
        X{i}.pre1990 = x(I(J));X{i}.post1990 = x(I(K));X{i}.north = norths(i);
        Y{i}.pre1990 = y(I(J));Y{i}.post1990 = y(I(K));Y{i}.north = norths(i);
        Z{i}.pre1990 = z(I(J));Z{i}.post1990 = z(I(K));Z{i}.north = norths(i);
    else
        if norths(i)<8100
            q=4;
            I = find(y>(norths(i-q))&y<=(norths(i+q)));
            J = find(dnum(I)>datenum(1980,1,1)&dnum(I)<datenum(1990,1,1));% 1980s
            K = find(dnum(I)>datenum(1990,1,1));% 1990s & 2000s
            figure
            scatter(x(I(J)),z(I(J)),S(I(J)),'k');
            hold on
            scatter(x(I(K)),z(I(K)),S(I(K)),'r');
            axis equal;grid on;
            xlim([0 8000])
            ylim([-14000 0])
            xlabel('E-W(m)','fontsize',12)
            ylabel('depth(m)','fontsize',12)
            title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
            X{i}.pre1990 = x(I(J));X{i}.post1990 = x(I(K));X{i}.north = norths(i);
            Y{i}.pre1990 = y(I(J));Y{i}.post1990 = y(I(K));Y{i}.north = norths(i);
            Z{i}.pre1990 = z(I(J));Z{i}.post1990 = z(I(K));Z{i}.north = norths(i);
        elseif norths(i)>=8100&&norths(i)<=11100
            q=3;
            Is = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z>-6000);% shallow fault
            Id = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z<-6000);% deep fault
            Js = find(dnum(Is)>datenum(1980,1,1)&dnum(Is)<datenum(1990,1,1));% 1980s
            Jd = find(dnum(Id)>datenum(1980,1,1)&dnum(Id)<datenum(1990,1,1));% 1980s
            Ks = find(dnum(Is)>datenum(1990,1,1));% 1990s & 2000s
            Kd = find(dnum(Id)>datenum(1990,1,1));% 1990s & 2000s
            figure
            scatter(x(Is(Js)),z(Is(Js)),S(Is(Js)),'k');
            hold on
            scatter(x(Id(Jd)),z(Id(Jd)),S(Id(Jd)),'k');
            scatter(x(Is(Ks)),z(Is(Ks)),S(Is(Ks)),'r');
            scatter(x(Id(Kd)),z(Id(Kd)),S(Id(Kd)),'r');
            axis equal;grid on;
            xlim([0 8000])
            ylim([-14000 0])
            xlabel('E-W(m)','fontsize',12)
            ylabel('depth(m)','fontsize',12)
            title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
            X{i}.pre1990s = x(Is(Js));X{i}.post1990s = x(Is(Ks));X{i}.north = norths(i);
            X{i}.pre1990d = x(Id(Jd));X{i}.post1990d = x(Id(Kd));
            Y{i}.pre1990s = y(Is(Js));Y{i}.post1990s = y(Is(Ks));Y{i}.north = norths(i);
            Y{i}.pre1990d = y(Id(Jd));Y{i}.post1990d = y(Id(Kd));
            Z{i}.pre1990s = z(Is(Js));Z{i}.post1990s = z(Is(Ks));Z{i}.north = norths(i);
            Z{i}.pre1990d = z(Id(Jd));Z{i}.post1990d = z(Id(Kd));
        else
            q=3;
            Is = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z>-7000);% shallow fault
            if norths(i)<12600
                Id = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z<-7000);% deep fault
            else
                Id = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z<-7000&z>-12000);% deep fault
            end
            Js = find(dnum(Is)>datenum(1980,1,1)&dnum(Is)<datenum(1990,1,1));% 1980s
            Jd = find(dnum(Id)>datenum(1980,1,1)&dnum(Id)<datenum(1990,1,1));% 1980s
            Ks = find(dnum(Is)>datenum(1990,1,1));% 1990s & 2000s
            Kd = find(dnum(Id)>datenum(1990,1,1));% 1990s & 2000s
            figure
            scatter(x(Is(Js)),z(Is(Js)),S(Is(Js)),'k');
            hold on
            scatter(x(Id(Jd)),z(Id(Jd)),S(Id(Jd)),'k');
            scatter(x(Is(Ks)),z(Is(Ks)),S(Is(Ks)),'r');
            scatter(x(Id(Kd)),z(Id(Kd)),S(Id(Kd)),'r');
            axis equal;grid on;
            xlim([0 8000])
            ylim([-14000 0])
            xlabel('E-W(m)','fontsize',12)
            ylabel('depth(m)','fontsize',12)
            title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
            X{i}.pre1990d = x(Id(Jd));X{i}.post1990d = x(Id(Kd));X{i}.north = norths(i);
            Y{i}.pre1990d = y(Id(Jd));Y{i}.post1990d = y(Id(Kd));Y{i}.north = norths(i);
            Z{i}.pre1990d = z(Id(Jd));Z{i}.post1990d = z(Id(Kd));Z{i}.north = norths(i);
        end
    end
end
    
% Regression on cross sections
f = 1;% smoothing factor (0 to 1)
wantplot = 0;% zero = no plot
imagefile = 'lowess_plot\';% directory where plot is stored

Xc = [];Yc = [];Zc = [];% nodes south of 8100m north (bifercation point)
Xs = [];Ys = [];Zs = [];% nodes of shallow fault
Xd = [];Yd = [];Zd = [];% nodes of deep fault

for i=1:numel(X)
    clear datain
    clear datains
    clear dataind
    clear xy
    clear xys
    clear xyd
    clear xdata
    if X{1,i}.north<8100
        % pre 1990 events
        datain(:,1) = Z{1,i}.pre1990;
        datain(:,2) = X{1,i}.pre1990;
        xdata = depths;
    
        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);
    
        % vertical extent of fault 
        I = find(xdata>=min(Z{1,i}.pre1990)&xdata<=max(Z{1,i}.pre1990));
        
        % plot regression on cross section
        figure(i+7)
        plot(xy(I,2),xy(I,1),'k-');
                
        clear datain
        % post 1990 events
        datain(:,1) = [xy(I,1);Z{1,i}.post1990];
        datain(:,2) = [xy(I,2);X{1,i}.post1990];
        clear xy

        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);
        
        % vertical extent of fault
        if X{1,i}.north==900
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);
        elseif X{1,i}.north==1200
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);
            K = find(xdata==-11800);
            I = [K;I];
        elseif X{1,i}.north==1500
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);
            K = find(xdata<=-11800&xdata>=-12100);
            I = [K;I];
        elseif X{1,i}.north==1800
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);
            K = find(xdata<=-11800&xdata>=-12400);
            I = [K;I];
        elseif X{1,i}.north==2100
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);
            K = find(xdata<=-11800&xdata>=-12700);
            I = [K;I];
        elseif X{1,i}.north==2400
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            K = find(xdata<=-11800&xdata>=-13000);
            I = [K;I];
        elseif X{1,i}.north==2700
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            K = find(xdata<=-11800&xdata>=-13000);
            I = [K;I];
        elseif X{1,i}.north==3000
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            K = find(xdata<=-11800&xdata>=-13000);
            I = [K;I];
        elseif X{1,i}.north==4500
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            J = find(xdata>=-3100&xdata<=-1600);
            I = [I;J];
        elseif X{1,i}.north==4800
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            J = find(xdata>=-1900&xdata<=-1600);
            I = [I;J];
        elseif X{1,i}.north==5100
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            J = find(xdata>=-1900&xdata<=-1600);
            I = [I;J];
        else
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
        end
        
        % plot regression on cross section
        plot(xy(I,2),xy(I,1),'k--');
        
        % store xyz points for fault nodes
        Xc(numel(Xc)+1:numel(Xc)+numel(I),1) = xy(I,2);
        Yc(numel(Yc)+1:numel(Yc)+numel(I),1) = Y{i}.north;
        Zc(numel(Zc)+1:numel(Zc)+numel(I),1) = xy(I,1);
    else
        % bifercation point
        if X{1,i}.north>=8100&&X{1,i}.north<=11100
            % pre 1990 events
            datains(:,1) = Z{1,i}.pre1990s;
            datains(:,2) = X{1,i}.pre1990s;
            dataind(:,1) = Z{1,i}.pre1990d;
            dataind(:,2) = X{1,i}.pre1990d;
            xdata = depths;
            
            % regression on cross section using Lowess function
            [dataout lowerLimit upperLimit xys] = lowess(datains,f,wantplot,imagefile,xdata);
            [dataout lowerLimit upperLimit xyd] = lowess(dataind,f,wantplot,imagefile,xdata);
            
            % vertical extent of fault
            if X{1,i}.north==9300
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=max(Z{1,i}.pre1990s));
                Js = find(xdata>=-1900&xdata<=-1300);
                Is = [Is;Js];
            elseif X{1,i}.north==9600
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=max(Z{1,i}.pre1990s));
                Js = find(xdata>=-1900&xdata<=-1600);
                Ks = find(xdata==-5500);
                Is = [Ks;Is;Js];
            elseif X{1,i}.north==10200
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=max(Z{1,i}.pre1990s));
                Js = find(xdata==-2500);
                Is = [Is;Js];
            else
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=max(Z{1,i}.pre1990s));
            end
            
            % plot regression on cross sections
            Id = find(xdata>=min(Z{1,i}.pre1990d)&xdata<=max(Z{1,i}.pre1990d));
            figure(i+7)
            plot(xys(Is,2),xys(Is,1),'k-');
            plot(xyd(Id,2),xyd(Id,1),'k-');
            
            clear datains
            clear dataind
            % post 1990 events
            datains(:,1) = [xys(Is,1);Z{1,i}.post1990s];
            datains(:,2) = [xys(Is,2);X{1,i}.post1990s];
            dataind(:,1) = [xyd(Id,1);Z{1,i}.post1990d];
            dataind(:,2) = [xyd(Id,2);X{1,i}.post1990d];
            clear xys
            clear xyd
            
            % regression on cross section using Lowess function
            [dataout lowerLimit upperLimit xys] = lowess(datains,f,wantplot,imagefile,xdata);
            [dataout lowerLimit upperLimit xyd] = lowess(dataind,f,wantplot,imagefile,xdata);
            
            % plot regression on cross sections
            Is = find(xdata>=min(datains(:,1))&xdata<=max(datains(:,1)));
            Id = find(xdata>=min(dataind(:,1))&xdata<=max(dataind(:,1)));
            plot(xys(Is,2),xys(Is,1),'k--');
            plot(xyd(Id,2),xyd(Id,1),'k--');
            
            % store xyz points for fault nodes on shallow fault
            Xs(numel(Xs)+1:numel(Xs)+numel(Is),1) = xys(Is,2);
            Ys(numel(Ys)+1:numel(Ys)+numel(Is),1) = Y{i}.north;
            Zs(numel(Zs)+1:numel(Zs)+numel(Is),1) = xys(Is,1);
            
            % store xyz points for fault nodes on deep fault
            Xd(numel(Xd)+1:numel(Xd)+numel(Id),1) = xyd(Id,2);
            Yd(numel(Yd)+1:numel(Yd)+numel(Id),1) = Y{i}.north;
            Zd(numel(Zd)+1:numel(Zd)+numel(Id),1) = xyd(Id,1);
        else% shallow fault termination
            % pre 1990 events
            dataind(:,1) = Z{1,i}.pre1990d;
            dataind(:,2) = X{1,i}.pre1990d;
            xdata = depths;
            
            % regression on cross section using Lowess function
            [dataout lowerLimit upperLimit xyd] = lowess(dataind,f,wantplot,imagefile,xdata);
            
            % vertical extent of fault
            if X{1,i}.north==12600
                Id = find(xdata>=min(Z{1,i}.pre1990d)&xdata<=max(Z{1,i}.pre1990d));
                Jd = find(xdata>=-8500&xdata<=-8200);
                Kd = find(xdata>=-12700&xdata<=-12100);
                Id = [Kd;Id;Jd];
            elseif X{1,i}.north==12900
                Id = find(xdata>=min(Z{1,i}.pre1990d)&xdata<=max(Z{1,i}.pre1990d));
                Kd = find(xdata==-12100);
                Id = [Kd;Id];
            else
                Id = find(xdata>=min(Z{1,i}.pre1990d)&xdata<=max(Z{1,i}.pre1990d));
            end
            
            % plot regression on cross sections
            figure(i+7)
            plot(xyd(Id,2),xyd(Id,1),'k-');
            
            clear dataind
            % post 1990 events
            dataind(:,1) = [xyd(Id,1);Z{1,i}.post1990d];
            dataind(:,2) = [xyd(Id,2);X{1,i}.post1990d];
            clear xyd
            
            % regression on cross section using Lowess function
            [dataout lowerLimit upperLimit xyd] = lowess(dataind,f,wantplot,imagefile,xdata);
            
            % vertical extent of fault
            Id = find(xdata>=min(dataind(:,1))&xdata<=max(dataind(:,1)));
            
            % plot regression on cross section
            plot(xyd(Id,2),xyd(Id,1),'k--');
            
            % store xyz points for fault nodes on deep fault
            Xd(numel(Xd)+1:numel(Xd)+numel(Id),1) = xyd(Id,2);
            Yd(numel(Yd)+1:numel(Yd)+numel(Id),1) = Y{i}.north;
            Zd(numel(Zd)+1:numel(Zd)+numel(Id),1) = xyd(Id,1);
        end
    end
end

% Smooth data in y-direction
Cc = unique(Zc);
Cs = unique(Zs);
Cd = unique(Zd);

for i=1:numel(Cc)
    I = find(Zc==Cc(i));
    [Xc(I),r]=smooth(Xc(I),1);
end
for i=1:numel(Cs)
    I = find(Zs==Cs(i));
    [Xs(I),r]=smooth(Xs(I),1);
end
for i=1:numel(Cd)
    I = find(Zd==Cd(i));
    [Xd(I),r]=smooth(Xd(I),1);
end

% put nodes in 3 column matrix
XYZc(:,1) = Xc;XYZc(:,2) = Yc;XYZc(:,3) = Zc;
XYZs(:,1) = Xs;XYZs(:,2) = Ys;XYZs(:,3) = Zs;
XYZd(:,1) = Xd;XYZd(:,2) = Yd;XYZd(:,3) = Zd;

% connect fault sections
Is = find(XYZc(:,2)==7800&XYZc(:,3)>=-6100);
Id = find(XYZc(:,2)==7800&XYZc(:,3)<=-6100);
XYZs = [XYZc(Is,:);XYZs];
XYZd = [XYZc(Id,:);XYZd];

% triangulate fault nodes
CONc=Delaunay2_5D(XYZc);
CONs=Delaunay2_5D(XYZs);
CONd=Delaunay2_5D(XYZd);

% plot fault mesh on 3D scatterplot
figure(1)
for i=1:numel(CONc(:,1))
    plot3(XYZc([CONc(i,:) CONc(i,1)],1),XYZc([CONc(i,:) CONc(i,1)],2),XYZc([CONc(i,:) CONc(i,1)],3),'b-')
end
for i=1:numel(CONs(:,1))
    plot3(XYZs([CONs(i,:) CONs(i,1)],1),XYZs([CONs(i,:) CONs(i,1)],2),XYZs([CONs(i,:) CONs(i,1)],3),'b-')
end
for i=1:numel(CONd(:,1))
    plot3(XYZd([CONd(i,:) CONd(i,1)],1),XYZd([CONd(i,:) CONd(i,1)],2),XYZd([CONd(i,:) CONd(i,1)],3),'b-')
end

% correct polarity of triangulation
convention = 'z-';
[tnc,nvecc] = PlotTriNormals(CONc,XYZc,convention);
[tns,nvecs] = PlotTriNormals(CONs,XYZs,convention);
[tnd,nvecd] = PlotTriNormals(CONd,XYZd,convention);

%% East Fault 1

% depth slices for node locations
depthsEF1 = [-13000:300:-10000];depthsEF1 = depthsEF1';
% northing coordinates for cross-sections/node locations
northsEF1 = [450:300:2250];% 300m spacing

% cross sections
for i=1:numel(northsEF1)
        I = find(yEF1>(northsEF1(i)-600)&yEF1<=(northsEF1(i)+600));
        J = find(dnumEF1(I)>datenum(1980,1,1)&dnumEF1(I)<datenum(1990,1,1));% 1980s
        K = find(dnumEF1(I)>datenum(1990,1,1));% 1990s & 2000s
        figure
        scatter(xEF1(I(J)),zEF1(I(J)),SEF1(I(J)),'k');
        hold on
        scatter(xEF1(I(K)),zEF1(I(K)),SEF1(I(K)),'r');
        axis equal;grid on;
        xlim([0 1000])
        ylim([-13000 -10000])
        xlabel('E-W(m)','fontsize',12)
        ylabel('depth(m)','fontsize',12)
        title(strcat(num2str(northsEF1(i)),'m north'),'fontsize',12)
        XEF1{i}.pre1990 = xEF1(I(J));XEF1{i}.post1990 = xEF1(I(K));XEF1{i}.north = northsEF1(i);%XEF1{i}.extent = xEF1(Ie);
        YEF1{i}.pre1990 = yEF1(I(J));YEF1{i}.post1990 = yEF1(I(K));YEF1{i}.north = northsEF1(i);%YEF1{i}.extent = yEF1(Ie);
        ZEF1{i}.pre1990 = zEF1(I(J));ZEF1{i}.post1990 = zEF1(I(K));ZEF1{i}.north = northsEF1(i);%ZEF1{i}.extent = zEF1(Ie);
end

% regression

Xef1=[];
Yef1=[];
Zef1=[];

for i=1:numel(XEF1)
    clear datain
    clear datains
    clear dataind
    clear xy
    clear xys
    clear xyd
    clear xdata
        % pre/post 1990 events
        datain(:,1) = [ZEF1{1,i}.pre1990;ZEF1{1,i}.post1990];
        datain(:,2) = [XEF1{1,i}.pre1990;XEF1{1,i}.post1990];
        xdata = depthsEF1;
        
        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);
        
        % vertical extent of fault
        if XEF1{1,i}.north==450
                I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
                K = find(xdata==-13000);
                I = [K;I];
        elseif XEF1{1,i}.north==750
                I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
                K = find(xdata==-13000);
                I = [K;I];
        elseif XEF1{1,i}.north==1350
                I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
                J = find(xdata==-11200);
                K = find(xdata==-12400);
                I = [K;I;J];
        elseif XEF1{1,i}.north==1650
                I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
                J = find(xdata==-11200);
                K = find(xdata==-12400);
                I = [K;I;J];
        elseif XEF1{1,i}.north==2250
                I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
                K = find(xdata==-11800);
                I = [K;I];
        else
                I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
        end

        % plot regression on cross sections
        figure(i+54)
        plot(xy(I,2),xy(I,1),'k-');
                      
        % store xyz points for fault nodes
        Xef1(numel(Xef1)+1:numel(Xef1)+numel(I),1) = xy(I,2);
        Yef1(numel(Yef1)+1:numel(Yef1)+numel(I),1) = YEF1{i}.north;
        Zef1(numel(Zef1)+1:numel(Zef1)+numel(I),1) = xy(I,1);
end

% put nodes in 3 column matrix
XYZef1(:,1) = Xef1;XYZef1(:,2) = Yef1;XYZef1(:,3) = Zef1;

% triangulate fault nodes
CONef1=Delaunay2_5D(XYZef1);

% plot fault mesh on 3D scatterplot
figure(2)
for i=1:numel(CONef1(:,1))
    plot3(XYZef1([CONef1(i,:) CONef1(i,1)],1),XYZef1([CONef1(i,:) CONef1(i,1)],2),XYZef1([CONef1(i,:) CONef1(i,1)],3),'b-')
end

% rotate back
Rccw = [cos(thetaEF1) -sin(thetaEF1);sin(thetaEF1) cos(thetaEF1)];
for i=1:numel(Xef1)
    b = Rccw*[XYZef1(i,1)-400;(XYZef1(i,2))];
    XYZef1(i,1) = b(1);XYZef1(i,2) = b(2);
end

% correct polarity of triangulation
[tnef1,nvecef1] = PlotTriNormals(CONef1,XYZef1,convention);

%% East Fault 2

% depth slices for node locations
depthsEF2 = [-11000:300:-8000];depthsEF2 = depthsEF2';
% northing coordinates for cross-sections/node locations
northsEF2 = [450:300:2050];% 300m spacing

% cross sections
for i=1:numel(northsEF2)
        I = find(yEF2>(northsEF2(i)-1700)&yEF2<=(northsEF2(i)+1000));
        J = find(dnumEF2(I)<datenum(1990,1,1));% 1980s
        K = find(dnumEF2(I)>datenum(1990,1,1));% 1990s & 2000s
        figure
        scatter(xEF2(I(J)),zEF2(I(J)),SEF2(I(J)),'k');
        hold on
        scatter(xEF2(I(K)),zEF2(I(K)),SEF2(I(K)),'r');
        axis equal;grid on;
        xlim([0 2000])
        ylim([-11000 -8000])
        xlabel('E-W(m)','fontsize',12)
        ylabel('depth(m)','fontsize',12)
        title(strcat(num2str(northsEF2(i)),'m north'),'fontsize',12)
        XEF2{i}.pre1990 = xEF2(I(J));XEF2{i}.post1990 = xEF2(I(K));XEF2{i}.north = northsEF2(i);
        YEF2{i}.pre1990 = yEF2(I(J));YEF2{i}.post1990 = yEF2(I(K));YEF2{i}.north = northsEF2(i);
        ZEF2{i}.pre1990 = zEF2(I(J));ZEF2{i}.post1990 = zEF2(I(K));ZEF2{i}.north = northsEF2(i);
end

% regression

Xef2=[];
Yef2=[];
Zef2=[];

for i=1:numel(XEF2)
    clear datain
    clear datains
    clear dataind
    clear xy
    clear xys
    clear xyd
    clear xdata
        % pre 1990 events
        datain(:,1) = ZEF2{1,i}.pre1990;
        datain(:,2) = XEF2{1,i}.pre1990;
        xdata = depthsEF2;
    
        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);

        % vertical extent of fault
        I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
        
        % plot regression on cross sections
        figure(i+62)
        plot(xy(I,2),xy(I,1),'k-');
                
        clear datain
        % post 1990 events
        datain(:,1) = [xy(I,1);ZEF2{1,i}.post1990];
        datain(:,2) = [xy(I,2);XEF2{1,i}.post1990];
        clear xy
    
        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);
        
        % vertical extent of fault
        I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
        
        % plot regression on cross sections
        plot(xy(I,2),xy(I,1),'k--');
        
        % store xyz points for fault nodes
        Xef2(numel(Xef2)+1:numel(Xef2)+numel(I),1) = xy(I,2);
        Yef2(numel(Yef2)+1:numel(Yef2)+numel(I),1) = YEF2{i}.north;
        Zef2(numel(Zef2)+1:numel(Zef2)+numel(I),1) = xy(I,1);
end

% put nodes in 3 column matrix
XYZef2(:,1) = Xef2;XYZef2(:,2) = Yef2;XYZef2(:,3) = Zef2;

% triangulate fault nodes
CONef2=Delaunay2_5D(XYZef2);

% plot fault mesh on 3D scatterplot
figure(3)
for i=1:numel(CONef2(:,1))
    plot3(XYZef2([CONef2(i,:) CONef2(i,1)],1),XYZef2([CONef2(i,:) CONef2(i,1)],2),XYZef2([CONef2(i,:) CONef2(i,1)],3),'b-')
end

% Rotate back
Rccw = [cos(thetaEF2) -sin(thetaEF2);sin(thetaEF2) cos(thetaEF2)];
for i=1:numel(Xef2)
    b = Rccw*[XYZef2(i,1);(XYZef2(i,2))];
    XYZef2(i,1) = b(1);XYZef2(i,2) = b(2);
end

% correct polarity of fault triangulation
[tnef2,nvecef2] = PlotTriNormals(CONef2,XYZef2,convention);

%% South Fault

% depth slices for node locations
depthsSF = [-13000:300:-3000];depthsSF = depthsSF';
% northing coordinates for cross-sections/node locations
northsSF = [520:300:16420];% 300m spacing

% cross sections
for i=1:numel(northsSF)
        I = find(ySF>(northsSF(i)-7000)&ySF<=(northsSF(i)+11000));
        J = find(dnumSF(I)<datenum(1990,1,1));% 1980s
        K = find(dnumSF(I)>datenum(1990,1,1));% 1990s & 2000s
        figure
        scatter(xSF(I(J)),zSF(I(J)),SSF(I(J)),'k');
        hold on
        scatter(xSF(I(K)),zSF(I(K)),SSF(I(K)),'r');
        axis equal;grid on;
        xlim([0 5000])
        ylim([-13000 -3000])
        xlabel('E-W(m)','fontsize',12)
        ylabel('depth(m)','fontsize',12)
        title(strcat(num2str(northsSF(i)),'m north'),'fontsize',12)
        XSF{i}.pre1990 = xSF(I(J));XSF{i}.post1990 = xSF(I(K));XSF{i}.north = northsSF(i);%XEF1{i}.extent = xEF1(Ie);
        YSF{i}.pre1990 = ySF(I(J));YSF{i}.post1990 = ySF(I(K));YSF{i}.north = northsSF(i);%YEF1{i}.extent = yEF1(Ie);
        ZSF{i}.pre1990 = zSF(I(J));ZSF{i}.post1990 = zSF(I(K));ZSF{i}.north = northsSF(i);%ZEF1{i}.extent = zEF1(Ie);
end

% regression

Xsf=[];
Ysf=[];
Zsf=[];

for i=1:numel(XSF)
    clear datain
    clear datains
    clear dataind
    clear xy
    clear xys
    clear xyd
    clear xdata
        % pre/post 1990 events
        datain(:,1) = [ZSF{1,i}.pre1990;ZSF{1,i}.post1990];
        datain(:,2) = [XSF{1,i}.pre1990;XSF{1,i}.post1990];
        xdata = depthsSF;
    
        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);
    
        % vertical extent of fault
        if XSF{1,i}.north==520
                I = find(xdata>=min(datain(:,1))&xdata<=-5800&xdata>=-9700);
        elseif XSF{1,i}.north==820
                I = find(xdata>=min(datain(:,1))&xdata<=-4900&xdata>=-10600);
        elseif XSF{1,i}.north==1120
                I = find(xdata>=min(datain(:,1))&xdata<=-4300&xdata>=-11200);
        elseif XSF{1,i}.north==1420
                I = find(xdata>=min(datain(:,1))&xdata<=-4000&xdata>=-11500);
        elseif XSF{1,i}.north==1720
                I = find(xdata>=min(datain(:,1))&xdata<=-3700&xdata>=-11800);
        elseif XSF{1,i}.north==2020
                I = find(xdata>=min(datain(:,1))&xdata<=-3700&xdata>=-11800);
        elseif XSF{1,i}.north==2320
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-12100);
        elseif XSF{1,i}.north==2620
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-12100);
        elseif XSF{1,i}.north==2920
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-12100);
        elseif XSF{1,i}.north==14020
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-11500);
        elseif XSF{1,i}.north==14320
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-11500);
        elseif XSF{1,i}.north==14620
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-11500);
        elseif XSF{1,i}.north==14920
                I = find(xdata>=min(datain(:,1))&xdata<=-3700&xdata>=-11200);
        elseif XSF{1,i}.north==15220
                I = find(xdata>=min(datain(:,1))&xdata<=-3700&xdata>=-11200);
        elseif XSF{1,i}.north==15520
                I = find(xdata>=min(datain(:,1))&xdata<=-4000&xdata>=-10900);
        elseif XSF{1,i}.north==15820
                I = find(xdata>=min(datain(:,1))&xdata<=-4300&xdata>=-10600);
        elseif XSF{1,i}.north==16120
                I = find(xdata>=min(datain(:,1))&xdata<=-4900&xdata>=-10000);
        elseif XSF{1,i}.north==16420
                I = find(xdata>=min(datain(:,1))&xdata<=-5800&xdata>=-9400);
        else
                I = find(xdata>=min(datain(:,1)));%&xdata<=max(datain(:,1)));
        end

        % plot regression on cross-sections
        figure(i+69)
        plot(xy(I,2),xy(I,1),'k-');

        % store xyz points for fault nodes
        Xsf(numel(Xsf)+1:numel(Xsf)+numel(I),1) = xy(I,2);
        Ysf(numel(Ysf)+1:numel(Ysf)+numel(I),1) = YSF{i}.north;
        Zsf(numel(Zsf)+1:numel(Zsf)+numel(I),1) = xy(I,1);
end

% smooth fault in y-direction
Csf = unique(Zsf);
for i=1:numel(Csf)
    I = find(Zsf==Csf(i));
    [Xsf(I),r]=smooth(Xsf(I),1);
end

% put nodes in 3 column matrox
XYZsf(:,1) = Xsf;XYZsf(:,2) = Ysf;XYZsf(:,3) = Zsf;

% triangulate fault nodes
CONsf=Delaunay2_5D(XYZsf);

% plot fault mesh on 3Dscatterplot
figure(4)
for i=1:numel(CONsf(:,1))
    plot3(XYZsf([CONsf(i,:) CONsf(i,1)],1),XYZsf([CONsf(i,:) CONsf(i,1)],2),XYZsf([CONsf(i,:) CONsf(i,1)],3),'b-')
end

% Rotate back
Rccw = [cos(thetaSF) -sin(thetaSF);sin(thetaSF) cos(thetaSF)];
for i=1:numel(Xsf)
    b = Rccw*[(XYZsf(i,1)+4000);XYZsf(i,2)-3000];
    XYZsf(i,1) = b(1);XYZsf(i,2) = b(2);
end

% correct polarity of triangulation
[tnsf,nvecsf] = PlotTriNormals(CONsf,XYZsf,convention);

%% West Fault 1

% depth slices for node locations
depthsWF1 = [-14000:300:-8000];depthsWF1 = depthsWF1';
% northing coordinates for cross-sections/node locations
northsWF1 = [900:300:5400];% 200m spacing

% cross sections
for i=1:numel(northsWF1)
        I = find(yWF1>(northsWF1(i)-2500)&yWF1<=(northsWF1(i)+2500));
        J = find(dnumWF1(I)<datenum(1990,1,1));% 1980s
        K = find(dnumWF1(I)>datenum(1990,1,1));% 1990s & 2000s
        figure
        scatter(xWF1(I(J)),zWF1(I(J)),SWF1(I(J)),'k');
        hold on
        scatter(xWF1(I(K)),zWF1(I(K)),SWF1(I(K)),'r');
        axis equal;grid on;
        xlim([2000 3500])
        ylim([-14000 -8000])
        xlabel('E-W(m)','fontsize',12)
        ylabel('depth(m)','fontsize',12)
        title(strcat(num2str(northsWF1(i)),'m north'),'fontsize',12)
        XWF1{i}.pre1990 = xWF1(I(J));XWF1{i}.post1990 = xWF1(I(K));XWF1{i}.north = northsWF1(i);
        YWF1{i}.pre1990 = yWF1(I(J));YWF1{i}.post1990 = yWF1(I(K));YWF1{i}.north = northsWF1(i);
        ZWF1{i}.pre1990 = zWF1(I(J));ZWF1{i}.post1990 = zWF1(I(K));ZWF1{i}.north = northsWF1(i);
end

% regression

Xwf1=[];
Ywf1=[];
Zwf1=[];

for i=1:numel(XWF1)
    clear datain
    clear datains
    clear dataind
    clear xy
    clear xys
    clear xyd
    clear xdata
        % pre/post 1990 events
        datain(:,1) = [ZWF1{1,i}.pre1990;ZWF1{1,i}.post1990];
        datain(:,2) = [XWF1{1,i}.pre1990;XWF1{1,i}.post1990];
        xdata = depthsWF1;
    
        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);
    
        % vertical extent of fault
        I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));

        % plot regression on cross sections
        figure(i+124)
        plot(xy(I,2),xy(I,1),'k-');

        % store xyz points for fault nodes
        Xwf1(numel(Xwf1)+1:numel(Xwf1)+numel(I),1) = xy(I,2);
        Ywf1(numel(Ywf1)+1:numel(Ywf1)+numel(I),1) = YWF1{i}.north;
        Zwf1(numel(Zwf1)+1:numel(Zwf1)+numel(I),1) = xy(I,1);
end

% put nodes in 3 column matrix
XYZwf1(:,1) = Xwf1;XYZwf1(:,2) = Ywf1;XYZwf1(:,3) = Zwf1;

% triangulate fault nodes
CONwf1=Delaunay2_5D(XYZwf1);

% Rotate back around y-axis
Rccw = [cos(theta2WF1) -sin(theta2WF1);sin(theta2WF1) cos(theta2WF1)];
XYZwf1(:,3) = XYZwf1(:,3) + 11000;XYZwf1(:,1) = XYZwf1(:,1) - 2750;
for i=1:numel(Xwf1)
    b = Rccw*[(XYZwf1(i,1));XYZwf1(i,3)];
    XYZwf1(i,1) = b(1);XYZwf1(i,3) = b(2);
end
XYZwf1(:,3) = XYZwf1(:,3) - 11000;XYZwf1(:,1) = XYZwf1(:,1) + 2750;

% plot fault mexh on 3D scatterplot
figure(5)
for i=1:numel(CONwf1(:,1))
    plot3(XYZwf1([CONwf1(i,:) CONwf1(i,1)],1),XYZwf1([CONwf1(i,:) CONwf1(i,1)],2),XYZwf1([CONwf1(i,:) CONwf1(i,1)],3),'b-')
end

% Rotate back
Rccw = [cos(thetaWF1) -sin(thetaWF1);sin(thetaWF1) cos(thetaWF1)];
for i=1:numel(Xwf1)
    b = Rccw*[(XYZwf1(i,1));XYZwf1(i,2)];
    XYZwf1(i,1) = b(1);XYZwf1(i,2) = b(2);
end

% correct polarity of fault triangulation
[tnwf1,nvecwf1] = PlotTriNormals(CONwf1,XYZwf1,convention);

%% West Fault 2

% depth slices for node locations
depthsWF2 = [-16500:300:-12500];depthsWF2 = depthsWF2';
% northing coordinates for cross-sections/node locations
northsWF2 = [650:300:3500];% 300m spacing

% cross sections
for i=1:numel(northsWF2)
        I = find(yWF2>(northsWF2(i)-1000)&yWF2<=(northsWF2(i)+1000));
        J = find(dnumWF2(I)<datenum(1990,1,1));% 1980s
        K = find(dnumWF2(I)>datenum(1990,1,1));% 1990s & 2000s
        figure
        scatter(xWF2(I(J)),zWF2(I(J)),SWF2(I(J)),'k');
        hold on
        scatter(xWF2(I(K)),zWF2(I(K)),SWF2(I(K)),'r');
        axis equal;grid on;
        xlim([0 4000])
        ylim([-16500 -12500])
        xlabel('E-W(m)','fontsize',12)
        ylabel('depth(m)','fontsize',12)
        title(strcat(num2str(northsWF2(i)),'m north'),'fontsize',12)
        XWF2{i}.pre1990 = xWF2(I(J));XWF2{i}.post1990 = xWF2(I(K));XWF2{i}.north = northsWF2(i);
        YWF2{i}.pre1990 = yWF2(I(J));YWF2{i}.post1990 = yWF2(I(K));YWF2{i}.north = northsWF2(i);
        ZWF2{i}.pre1990 = zWF2(I(J));ZWF2{i}.post1990 = zWF2(I(K));ZWF2{i}.north = northsWF2(i);
end

% regression

Xwf2=[];
Ywf2=[];
Zwf2=[];

for i=1:numel(XWF2)
    clear datain
    clear datains
    clear dataind
    clear xy
    clear xys
    clear xyd
    clear xdata
        % pre/post 1990 events
        datain(:,1) = [ZWF2{1,i}.pre1990;ZWF2{1,i}.post1990];
        datain(:,2) = [XWF2{1,i}.pre1990;XWF2{1,i}.post1990];
        xdata = depthsWF2;
    
        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);
        
        % vertical extent of fault
        I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));

        % plot regression on cross sections
        figure(i+141)
        plot(xy(I,2),xy(I,1),'k-');

        % store xyz points for fault nodels
        Xwf2(numel(Xwf2)+1:numel(Xwf2)+numel(I),1) = xy(I,2);
        Ywf2(numel(Ywf2)+1:numel(Ywf2)+numel(I),1) = YWF2{i}.north;
        Zwf2(numel(Zwf2)+1:numel(Zwf2)+numel(I),1) = xy(I,1);
end

% put fault nodes in 3 column matrix
XYZwf2(:,1) = Xwf2;XYZwf2(:,2) = Ywf2;XYZwf2(:,3) = Zwf2;

% triangulate fault nodes
CONwf2=Delaunay2_5D(XYZwf2);

% plot fault mesh on 3D scatterplot
figure(6)
for i=1:numel(CONwf2(:,1))
    plot3(XYZwf2([CONwf2(i,:) CONwf2(i,1)],1),XYZwf2([CONwf2(i,:) CONwf2(i,1)],2),XYZwf2([CONwf2(i,:) CONwf2(i,1)],3),'b-')
end

% Rotate back
Rccw = [cos(thetaWF2) -sin(thetaWF2);sin(thetaWF2) cos(thetaWF2)];
for i=1:numel(Xwf2)
    b = Rccw*[(XYZwf2(i,1))+500;XYZwf2(i,2)-1000];
    XYZwf2(i,1) = b(1);XYZwf2(i,2) = b(2);
end

% correct polarity of fault triangulation
[tnwf2,nvecwf2] = PlotTriNormals(CONwf2,XYZwf2,convention);

%% Goat Mountain Fault
% import points along fault trace
dataGM = importdata('Goat_Mountain_Fault.csv',',');
latGM = dataGM.data(:,2);
longGM = dataGM.data(:,1);
[xGM,yGM,utmzone] = deg2utm(latGM,longGM);
xoGM = 553500;
yoGM = 5112000;
xGM = xGM - xoGM;
yGM = yGM - yoGM;
zGM = zeros(numel(xGM),1);

% row 2
xGM = [xGM;xGM(1);xGM(1);xGM;xGM(numel(xGM));xGM(numel(xGM))];
yGM = [yGM;yGM(1)+600;yGM(1)+300;yGM;yGM(numel(yGM))-300;yGM(numel(yGM))-600];
zGM = [zGM;ones(numel(zGM)+4,1)*-300];
% row 3
xGM = [xGM;xGM(1);xGM(12:26);xGM(numel(xGM))];
yGM = [yGM;yGM(1)+900;yGM(12:26);yGM(numel(yGM))-300];
zGM = [zGM;ones(17,1)*-600];
% row 4
xGM = [xGM;xGM(1);xGM(27:43);xGM(numel(xGM))];
yGM = [yGM;yGM(1)+1200;yGM(27:43);yGM(numel(yGM))-300];
zGM = [zGM;ones(19,1)*-900];
% row 5
xGM = [xGM;xGM(44:62)];
yGM = [yGM;yGM(44:62)];
zGM = [zGM;ones(19,1)*-1200];
% row 6
xGM = [xGM;xGM(1);xGM(63:81);xGM(numel(xGM))];
yGM = [yGM;yGM(1)+1500;yGM(63:81);yGM(numel(yGM))-300];
zGM = [zGM;ones(21,1)*-1500];
% row 7
xGM = [xGM;xGM(82:102)];
yGM = [yGM;yGM(82:102)];
zGM = [zGM;ones(21,1)*-1800];
% row 8
xGM = [xGM;xGM(104:122)];
yGM = [yGM;yGM(104:122)];
zGM = [zGM;ones(19,1)*-2100];
% row 9
xGM = [xGM;xGM(124:142)];
yGM = [yGM;yGM(124:142)];
zGM = [zGM;ones(19,1)*-2400];
% row 10
xGM = [xGM;xGM(144:160)];
yGM = [yGM;yGM(144:160)];
zGM = [zGM;ones(17,1)*-2700];
% row 11
xGM = [xGM;xGM(163:177)];
yGM = [yGM;yGM(163:177)];
zGM = [zGM;ones(15,1)*-3000];
% row 12
xGM = [xGM;xGM(181:191)];
yGM = [yGM;yGM(181:191)];
zGM = [zGM;ones(11,1)*-3300];

% move fault trace to x=0 for dip rotation
xs = min(xGM);
xGM = xGM - xs;

clear c
c(:,1) = xGM;c(:,2) = yGM;c(:,3) = zGM;
n = fitNormal(c, 0);
thetaGM = atan(n(2)/n(1));
% rotate to j=0
Rcw = [cos(thetaGM) sin(thetaGM);-sin(thetaGM) cos(thetaGM)];
for i=1:numel(xGM)
    b = Rcw*[xGM(i);yGM(i)];
    xGM(i) = b(1);yGM(i) = b(2);
end

% put fault nodes in 3 column matrix
XYZgm(:,1) = xGM;XYZgm(:,2) = yGM;XYZgm(:,3) = zGM;

% triangulate fault mesh
CONgm=Delaunay2_5D(XYZgm);

% rotate about y-axis to 80 degree dip
Rcw = [cos(-10*(pi/180)) sin(-10*(pi/180));-sin(-10*(pi/180)) cos(-10*(pi/180))];
for i=1:numel(XYZgm(:,1))
    b = Rcw*[XYZgm(i,1);XYZgm(i,3)];
    XYZgm(i,1) = b(1);XYZgm(i,3) = b(2);
end

% Rotate back
Rccw = [cos(thetaGM) -sin(thetaGM);sin(thetaGM) cos(thetaGM)];
for i=1:numel(XYZgm(:,1))
    b = Rccw*[XYZgm(i,1);XYZgm(i,2)];
    XYZgm(i,1) = b(1);XYZgm(i,2) = b(2);
end

% move fault trace back to to x=xs
XYZgm(:,1) = XYZgm(:,1) + xs;

% plot fault nodes
figure
plot3(XYZgm(:,1),XYZgm(:,2),XYZgm(:,3),'ko');
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
axis equal;grid on
xlim([0 1000])
ylim([0 6500])
zlim([-3600 0])
title('Goat Mountain Fault','fontsize',12)
hold on

% plot fault mesh on 3D scatterplot
for i=1:numel(CONgm(:,1))
    plot3(XYZgm([CONgm(i,:) CONgm(i,1)],1),XYZgm([CONgm(i,:) CONgm(i,1)],2),XYZgm([CONgm(i,:) CONgm(i,1)],3),'b-')
end

% correct polarity of fault triangulation
[tngm,nvecgm] = PlotTriNormals(CONgm,XYZgm,convention);

%% Translate Faults back to UTM

% north fault
XYZc(:,1) = XYZc(:,1) + xo;
XYZc(:,2) = XYZc(:,2) + yo;
XYZs(:,1) = XYZs(:,1) + xo;
XYZs(:,2) = XYZs(:,2) + yo;
XYZd(:,1) = XYZd(:,1) + xo;
XYZd(:,2) = XYZd(:,2) + yo;

% east fault 1
XYZef1(:,1) = XYZef1(:,1) + xoEF1;
XYZef1(:,2) = XYZef1(:,2) + yoEF1;

% east fault 2
XYZef2(:,1) = XYZef2(:,1) + xoEF2;
XYZef2(:,2) = XYZef2(:,2) + yoEF2;

% south fault
XYZsf(:,1) = XYZsf(:,1) + xoSF;
XYZsf(:,2) = XYZsf(:,2) + yoSF;

% west fault 1
XYZwf1(:,1) = XYZwf1(:,1) + xoWF1;
XYZwf1(:,2) = XYZwf1(:,2) + yoWF1;

% west fault 2
XYZwf2(:,1) = XYZwf2(:,1) + xoWF2;
XYZwf2(:,2) = XYZwf2(:,2) + yoWF2;

% Goat Mountain fault
XYZgm(:,1) = XYZgm(:,1) + xoGM;
XYZgm(:,2) = XYZgm(:,2) + yoGM;