%% MSH_Plot_EQ_3D_goatmountain_uncertainty.m
%
% AUTHOR:           Michael W. Swyer
% DATE CREATED:     2015/03/26
% DATE MODIFIED:    2015/08/26
% VERSION:          v.2
%
% PURPOSE:  To construct 3D fault geometry around Mount Saint Helens for
% permeability potential modeling for play fairway analysis
%
% PURPOSE OF MODIFICATION: To model fault geometry uncertainty on goat mountain
% fault which is +/- 10 degrees dip from an assumed 80 degree dip
% 
% INPUT: Points along mapped goat mountain fault trace.
%           1) Goat_Mountain_Fault.csv
%
% OUTPUT: Position of fault nodes in UTM Zone 10N meters (XYZ-) and for upper,
% (XYZ-_ul) and lower (XYZ-_ll) residuals, triangulation of fault nodes (tn-),
% and for upper (tn-_ul) and lower (tn-_ll) residuals, and unit normal vectors
% of fault element triangles (nvec-), and upper (nvec-_ul) and lower (nvec-_ll)
% residuals.
%
% NOTE:
%
% KEY functions written specifically for this modeling:
%
% Other related functions:
%       deg2utm.m
%       fitNormal.m
%       lowess.m
%       smooth.m
%       R3RSR.m
%       R3R.m
%       SR.m
%       H.m
%       Delaunay2_5D.m
%       PlotTriNormals.m

clear all;close all

show_graph = 0;% Show graph of fit normal (0=no 1=yes)

%% Goat Mountain Fault
dataGM = importdata('Goat_Mountain_Fault.csv',',');
latGM = dataGM.data(:,2);
longGM = dataGM.data(:,1);
[xGM,yGM,utmzone] = deg2utm(latGM,longGM);
% UTM origin
xoGM = 553500;
yoGM = 5112000;

% row 1
xGM = xGM - xoGM;
yGM = yGM - yoGM;
zGM = zeros(numel(xGM),1);
% row 2
xGM = [xGM;xGM(1);xGM(1);xGM;xGM(numel(xGM));xGM(numel(xGM))];
yGM = [yGM;yGM(1)+600;yGM(1)+300;yGM;yGM(numel(yGM))-300;yGM(numel(yGM))-600];
zGM = [zGM;ones(numel(zGM)+4,1)*-300];
% row 3
xGM = [xGM;xGM(1);xGM(12:26);xGM(numel(xGM))];
yGM = [yGM;yGM(1)+900;yGM(12:26);yGM(numel(yGM))-300];
zGM = [zGM;ones(17,1)*-600];
% row 4
xGM = [xGM;xGM(1);xGM(27:43);xGM(numel(xGM))];
yGM = [yGM;yGM(1)+1200;yGM(27:43);yGM(numel(yGM))-300];
zGM = [zGM;ones(19,1)*-900];
% row 5
xGM = [xGM;xGM(44:62)];
yGM = [yGM;yGM(44:62)];
zGM = [zGM;ones(19,1)*-1200];
% row 6
xGM = [xGM;xGM(1);xGM(63:81);xGM(numel(xGM))];
yGM = [yGM;yGM(1)+1500;yGM(63:81);yGM(numel(yGM))-300];
zGM = [zGM;ones(21,1)*-1500];
% row 7
xGM = [xGM;xGM(82:102)];
yGM = [yGM;yGM(82:102)];
zGM = [zGM;ones(21,1)*-1800];
% row 8
xGM = [xGM;xGM(104:122)];
yGM = [yGM;yGM(104:122)];
zGM = [zGM;ones(19,1)*-2100];
% row 9
xGM = [xGM;xGM(124:142)];
yGM = [yGM;yGM(124:142)];
zGM = [zGM;ones(19,1)*-2400];
% row 10
xGM = [xGM;xGM(144:160)];
yGM = [yGM;yGM(144:160)];
zGM = [zGM;ones(17,1)*-2700];
% row 11
xGM = [xGM;xGM(163:177)];
yGM = [yGM;yGM(163:177)];
zGM = [zGM;ones(15,1)*-3000];
% row 12
xGM = [xGM;xGM(181:191)];
yGM = [yGM;yGM(181:191)];
zGM = [zGM;ones(11,1)*-3300];

% translate fault trace to origin
xs = min(xGM);
xGM = xGM - xs;

% fit normal vector to nodes
clear c
c(:,1) = xGM;c(:,2) = yGM;c(:,3) = zGM;
n = fitNormal(c, 0);
thetaGM = atan(n(2)/n(1));
% rotate to j=0
Rcw = [cos(thetaGM) sin(thetaGM);-sin(thetaGM) cos(thetaGM)];
for i=1:numel(xGM)
    b = Rcw*[xGM(i);yGM(i)];
    xGM(i) = b(1);yGM(i) = b(2);
end

% put fault nodes in 3 column matrix and triangulate
XYZgm(:,1) = xGM;XYZgm(:,2) = yGM;XYZgm(:,3) = zGM;
CONgm=Delaunay2_5D(XYZgm);
XYZgm_ul(:,1) = xGM;XYZgm_ul(:,2) = yGM;XYZgm_ul(:,3) = zGM;
CONgm_ul=Delaunay2_5D(XYZgm_ul);
XYZgm_ll(:,1) = xGM;XYZgm_ll(:,2) = yGM;XYZgm_ll(:,3) = zGM;
CONgm_ll=Delaunay2_5D(XYZgm_ll);

% rotate about y-axis for dip
Rcw = [cos(-10*(pi/180)) sin(-10*(pi/180));-sin(-10*(pi/180)) cos(-10*(pi/180))];
Rcw_ul = [cos(-20*(pi/180)) sin(-20*(pi/180));-sin(-20*(pi/180)) cos(-20*(pi/180))];
Rcw_ll = [cos(-1*(pi/180)) sin(-1*(pi/180));-sin(-1*(pi/180)) cos(-1*(pi/180))];
for i=1:numel(XYZgm(:,1))
    b = Rcw*[XYZgm(i,1);XYZgm(i,3)];
    XYZgm(i,1) = b(1);XYZgm(i,3) = b(2);
    b = Rcw_ul*[XYZgm_ul(i,1);XYZgm_ul(i,3)];
    XYZgm_ul(i,1) = b(1);XYZgm_ul(i,3) = b(2);
    b = Rcw_ll*[XYZgm_ll(i,1);XYZgm_ll(i,3)];
    XYZgm_ll(i,1) = b(1);XYZgm_ll(i,3) = b(2);
end
% Rotate back from j=0
Rccw = [cos(thetaGM) -sin(thetaGM);sin(thetaGM) cos(thetaGM)];
for i=1:numel(XYZgm(:,1))
    b = Rccw*[XYZgm(i,1);XYZgm(i,2)];
    XYZgm(i,1) = b(1);XYZgm(i,2) = b(2);
    b = Rccw*[XYZgm_ul(i,1);XYZgm_ul(i,2)];
    XYZgm_ul(i,1) = b(1);XYZgm_ul(i,2) = b(2);
    b = Rccw*[XYZgm_ll(i,1);XYZgm_ll(i,2)];
    XYZgm_ll(i,1) = b(1);XYZgm_ll(i,2) = b(2);
end

% translate back from origin
XYZgm(:,1) = XYZgm(:,1) + xs;
XYZgm_ul(:,1) = XYZgm_ul(:,1) + xs;
XYZgm_ll(:,1) = XYZgm_ll(:,1) + xs;

% plot fault with upper/lower residual geometries
figure
plot3(XYZgm(:,1),XYZgm(:,2),XYZgm(:,3),'ko');
hold on
plot3(XYZgm_ul(:,1),XYZgm_ul(:,2),XYZgm_ul(:,3),'ko');
plot3(XYZgm_ll(:,1),XYZgm_ll(:,2),XYZgm_ll(:,3),'ko');
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
axis equal;grid on
xlim([0 2000])
ylim([0 6500])
zlim([-3600 0])
title('Goat Mountain Fault','fontsize',12)
for i=1:numel(CONgm(:,1))
    plot3(XYZgm([CONgm_ul(i,:) CONgm(i,1)],1),XYZgm([CONgm(i,:) CONgm(i,1)],2),XYZgm([CONgm(i,:) CONgm(i,1)],3),'k-')
    plot3(XYZgm_ul([CONgm_ul(i,:) CONgm_ul(i,1)],1),XYZgm_ul([CONgm_ul(i,:) CONgm_ul(i,1)],2),XYZgm_ul([CONgm_ul(i,:) CONgm_ul(i,1)],3),'r-')
    plot3(XYZgm_ll([CONgm_ll(i,:) CONgm_ll(i,1)],1),XYZgm_ll([CONgm_ll(i,:) CONgm_ll(i,1)],2),XYZgm_ll([CONgm_ll(i,:) CONgm_ll(i,1)],3),'r-')
end

% correct polarity of triangulation
convention = 'z-';
[tngm,nvecgm] = PlotTriNormals(CONgm,XYZgm,convention);
[tngm_ul,nvecgm_ul] = PlotTriNormals(CONgm_ul,XYZgm_ul,convention);
[tngm_ll,nvecgm_ll] = PlotTriNormals(CONgm_ll,XYZgm_ll,convention);


%% Translate Faults back to UTM

XYZgm(:,1) = XYZgm(:,1) + xoGM;
XYZgm(:,2) = XYZgm(:,2) + yoGM;

XYZgm_ul(:,1) = XYZgm_ul(:,1) + xoGM;
XYZgm_ul(:,2) = XYZgm_ul(:,2) + yoGM;

XYZgm_ll(:,1) = XYZgm_ll(:,1) + xoGM;
XYZgm_ll(:,2) = XYZgm_ll(:,2) + yoGM;