%% MSH_Plot_EQ_3D_northfault_uncertainty.m
%
% AUTHOR:           Michael W. Swyer
% DATE CREATED:     2015/03/26
% DATE MODIFIED:    2015/07/27
% VERSION:          v.2
%
% PURPOSE:  To construct 3D fault geometry around Mount Saint Helens for
% permeability potential modeling for play fairway analysis
%
% PURPOSE OF MODIFICATION: To model fault geometry uncertainty on north fault
% which is the upper/lower residuals of the LOWESS regression
% 
% INPUT: Seismic catalogs selected from Pacific Northwest Seismic Network:
%           1) pnsn_relocations_faults_north.csv
%
% OUTPUT: Position of fault nodes in UTM Zone 10N meters (XYZ-) and for upper,
% (XYZ-_ul) and lower (XYZ-_ll) residuals, triangulation of fault nodes (tn-),
% and for upper (tn-_ul) and lower (tn-_ll) residuals, and unit normal vectors
% of fault element triangles (nvec-), and upper (nvec-_ul) and lower (nvec-_ll)
% residuals.
%
% NOTE:
%
% KEY functions written specifically for this modeling:
%
% Other related functions:
%       deg2utm.m
%       fitNormal.m
%       lowess.m
%       smooth.m
%       R3RSR.m
%       R3R.m
%       SR.m
%       H.m
%       Delaunay2_5D.m
%       PlotTriNormals.m

clear all;close all

show_graph = 0;% Show graph of fit normal (0=no 1=yes)

%% Import Data

%%% north fault %%%
data = importdata('pnsn_relocations_faults_north.csv',',');
for i=2:numel(data.textdata(:,1))
    datetime = strsplit(data.textdata{i,3},' ');
    date = strsplit(datetime{1},'/');
    Date = [str2num(date{1}) str2num(date{2}) str2num(date{3})];
    time = strsplit(datetime{2},':');
    Time = [str2num(time{1}) str2num(time{2}) str2num(time{3})];
    dnum(i-1,1) = datenum(Date(1),Date(2),Date(3),Time(1),Time(2),Time(3));
    lat(i-1,1) = str2num(data.textdata{i,7});
    long(i-1,1) = str2num(data.textdata{i,8});
    z(i-1,1) = (-str2num(data.textdata{i,5})*1000);%-(rand*(10^-7));
    m(i-1,1) = str2num(data.textdata{i,6});
end
S = 3.^m;
[x,y,utmzone] = deg2utm(lat,long);
% UTM origin
xo = 555500;
yo = 5123500;
x = x - xo;
y = y - yo;


%% filter data
% by time
% north fault
J = find(dnum>datenum(1980,1,1)&dnum<datenum(1990,1,1));% 1980s
K = find(dnum>datenum(1990,1,1));% 1990s & 2000s


%% Plot 3D EQs
% north fault
figure
h1 = scatter3(x(J),y(J),z(J),S(J),'k');
hold on
h2 = scatter3(x(K),y(K),z(K),S(K),'r');
legend([h1 h2],'1980s','1990s & 2000s');
axis equal;grid on;
xlim([0 8000])
ylim([0 18000])
zlim([-14000 0])
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title('North Fault','fontsize',14)

%% cross sections

% remove events below 14km depth
Q = find(z>-14000);
x = x(Q);y = y(Q);z = z(Q);

% depth slices for node locations
depths = [-13900:300:-1000];depths = depths';
% northing coordinates for cross-sections/node locations
norths = [900:300:13800];% 300m spacing

q = 2;% moving window, # of cells on each side

for i=1:numel(norths)
    if norths(i)<4500
        I = find(y>(norths(i)-3500)&y<=(norths(i)+2500));
        J = find(dnum(I)>datenum(1980,1,1)&dnum(I)<datenum(1990,1,1));% 1980s
        K = find(dnum(I)>datenum(1990,1,1));% 1990s & 2000s
        figure
        scatter(x(I(J)),z(I(J)),S(I(J)),'k');
        hold on
        scatter(x(I(K)),z(I(K)),S(I(K)),'r');
        axis equal;grid on;
        xlim([0 8000])
        ylim([-14000 0])
        xlabel('E-W(m)','fontsize',12)
        ylabel('depth(m)','fontsize',12)
        title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
        X{i}.pre1990 = x(I(J));X{i}.post1990 = x(I(K));X{i}.north = norths(i);
        Y{i}.pre1990 = y(I(J));Y{i}.post1990 = y(I(K));Y{i}.north = norths(i);
        Z{i}.pre1990 = z(I(J));Z{i}.post1990 = z(I(K));Z{i}.north = norths(i);
    else
        if norths(i)<8100
            q=5;
            I = find(y>(norths(i)-3500)&y<=(norths(i)+2500));
            J = find(dnum(I)>datenum(1980,1,1)&dnum(I)<datenum(1990,1,1));% 1980s
            K = find(dnum(I)>datenum(1990,1,1));% 1990s & 2000s
            figure
            scatter(x(I(J)),z(I(J)),S(I(J)),'k');
            hold on
            scatter(x(I(K)),z(I(K)),S(I(K)),'r');
            axis equal;grid on;
            xlim([0 8000])
            ylim([-14000 0])
            xlabel('E-W(m)','fontsize',12)
            ylabel('depth(m)','fontsize',12)
            title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
            X{i}.pre1990 = x(I(J));X{i}.post1990 = x(I(K));X{i}.north = norths(i);
            Y{i}.pre1990 = y(I(J));Y{i}.post1990 = y(I(K));Y{i}.north = norths(i);
            Z{i}.pre1990 = z(I(J));Z{i}.post1990 = z(I(K));Z{i}.north = norths(i);
        elseif norths(i)>=8100&&norths(i)<=11100
            q=7;
            Is = find(y>(norths(i)-(q*300+300))&y<=(norths(i)+q*300)&z>-6000);% shallow fault
            Id = find(y>(norths(i)-(q*300+300))&y<=(norths(i)+q*300)&z<-6000);% deep fault
            Js = find(dnum(Is)>datenum(1980,1,1)&dnum(Is)<datenum(1990,1,1));% 1980s
            Jd = find(dnum(Id)>datenum(1980,1,1)&dnum(Id)<datenum(1990,1,1));% 1980s
            Ks = find(dnum(Is)>datenum(1990,1,1));% 1990s & 2000s
            Kd = find(dnum(Id)>datenum(1990,1,1));% 1990s & 2000s
            figure
            scatter(x(Is(Js)),z(Is(Js)),S(Is(Js)),'k');
            hold on
            scatter(x(Id(Jd)),z(Id(Jd)),S(Id(Jd)),'k');
            scatter(x(Is(Ks)),z(Is(Ks)),S(Is(Ks)),'r');
            scatter(x(Id(Kd)),z(Id(Kd)),S(Id(Kd)),'r');
            axis equal;grid on;
            xlim([0 8000])
            ylim([-14000 0])
            xlabel('E-W(m)','fontsize',12)
            ylabel('depth(m)','fontsize',12)
            title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
            X{i}.pre1990s = x(Is(Js));X{i}.post1990s = x(Is(Ks));X{i}.north = norths(i);
            X{i}.pre1990d = x(Id(Jd));X{i}.post1990d = x(Id(Kd));
            Y{i}.pre1990s = y(Is(Js));Y{i}.post1990s = y(Is(Ks));Y{i}.north = norths(i);
            Y{i}.pre1990d = y(Id(Jd));Y{i}.post1990d = y(Id(Kd));
            Z{i}.pre1990s = z(Is(Js));Z{i}.post1990s = z(Is(Ks));Z{i}.north = norths(i);
            Z{i}.pre1990d = z(Id(Jd));Z{i}.post1990d = z(Id(Kd));
        else
            q=5;
            Is = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z>-7000);% shallow fault
            if norths(i)<12600
                Id = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z<-7000);% deep fault
            else
                Id = find(y>(norths(i)-q*300)&y<=(norths(i)+q*300)&z<-7000&z>-12000);% deep fault
            end
            Js = find(dnum(Is)>datenum(1980,1,1)&dnum(Is)<datenum(1990,1,1));% 1980s
            Jd = find(dnum(Id)>datenum(1980,1,1)&dnum(Id)<datenum(1990,1,1));% 1980s
            Ks = find(dnum(Is)>datenum(1990,1,1));% 1990s & 2000s
            Kd = find(dnum(Id)>datenum(1990,1,1));% 1990s & 2000s
            figure
            scatter(x(Is(Js)),z(Is(Js)),S(Is(Js)),'k');
            hold on
            scatter(x(Id(Jd)),z(Id(Jd)),S(Id(Jd)),'k');
            scatter(x(Is(Ks)),z(Is(Ks)),S(Is(Ks)),'r');
            scatter(x(Id(Kd)),z(Id(Kd)),S(Id(Kd)),'r');
            axis equal;grid on;
            xlim([0 8000])
            ylim([-14000 0])
            xlabel('E-W(m)','fontsize',12)
            ylabel('depth(m)','fontsize',12)
            title(strcat(num2str(norths(i)),'m north'),'fontsize',12)
            X{i}.pre1990d = x(Id(Jd));X{i}.post1990d = x(Id(Kd));X{i}.north = norths(i);
            Y{i}.pre1990d = y(Id(Jd));Y{i}.post1990d = y(Id(Kd));Y{i}.north = norths(i);
            Z{i}.pre1990d = z(Id(Jd));Z{i}.post1990d = z(Id(Kd));Z{i}.north = norths(i);
        end
    end
end
    
%% Regression on cross sections
f = 1;% smoothing factor (0 to 1)
wantplot = 1;% zero = no plot, one = plot w/ residuals
imagefile = 'lowess_plot\';% directory where plot is stored

Xc = [];Yc = [];Zc = [];% nodes south of 8100m north (bifercation point)
Xs = [];Ys = [];Zs = [];% nodes of shallow fault
Xd = [];Yd = [];Zd = [];% nodes of deep fault
% Upper Limit
Xc_ul = [];Yc_ul = [];Zc_ul = [];% nodes south of 8100m north (bifercation point)
Xs_ul = [];Ys_ul = [];Zs_ul = [];% nodes of shallow fault
Xd_ul = [];Yd_ul = [];Zd_ul = [];% nodes of deep fault
% Lower Limit
Xc_ll = [];Yc_ll = [];Zc_ll = [];% nodes south of 8100m north (bifercation point)
Xs_ll = [];Ys_ll = [];Zs_ll = [];% nodes of shallow fault
Xd_ll = [];Yd_ll = [];Zd_ll = [];% nodes of deep fault

for i=1:numel(X)
    clear datain
    clear datains
    clear dataind
    clear xy
    clear xys
    clear xyd
    clear xdata
    clear lowerLimit
    clear upperLimit
    clear UL
    clear LL
    clear lowerLimits
    clear upperLimits
    clear ULs
    clear LLs
    clear lowerLimitd
    clear upperLimitd
    clear ULd
    clear LLd
    if X{1,i}.north<8100
        % pre/post 1990 events
        datain(:,1) = [Z{1,i}.pre1990;Z{1,i}.post1990];
        datain(:,2) = [X{1,i}.pre1990;X{1,i}.post1990];
        xdata = depths;
        
        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);
        
        % vertical extent of fault 
        if X{1,i}.north==900
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);
        elseif X{1,i}.north==1200
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);%&xdata<=-3100);
            K = find(xdata==-11800);
            I = [K;I];
            I = unique(I);
        elseif X{1,i}.north==1500
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);%&xdata<=-2500);
            K = find(xdata<=-11800&xdata>=-12100);
            I = [K;I];
            I = unique(I);
        elseif X{1,i}.north==1800
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);%&xdata<=-2200);
            K = find(xdata<=-11800&xdata>=-12400);
            I = [K;I];
            I = unique(I);
        elseif X{1,i}.north==2100
            I = find(xdata>=min(datain(:,1))&xdata<=-1900);%&xdata<=-2200);
            K = find(xdata<=-11800&xdata>=-12700);
            I = [K;I];
            I = unique(I);
        elseif X{1,i}.north==2400
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            K = find(xdata<=-11800&xdata>=-13000);
            I = [K;I];
            I = unique(I);
        elseif X{1,i}.north==2700
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            K = find(xdata<=-11800&xdata>=-13000);
            I = [K;I];
            I = unique(I);
        elseif X{1,i}.north==3000
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            K = find(xdata<=-11800&xdata>=-13000);
            I = [K;I];
            I = unique(I);
        elseif X{1,i}.north==4500
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            J = find(xdata>=-3100&xdata<=-1600);
            I = [I;J];
            I = unique(I);
        elseif X{1,i}.north==4800
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            J = find(xdata>=-1900&xdata<=-1600);
            I = [I;J];
            I = unique(I);
        elseif X{1,i}.north==5100
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
            J = find(xdata>=-1900&xdata<=-1600);
            I = [I;J];
            I = unique(I);
        else
            I = find(xdata>=min(datain(:,1))&xdata<=max(datain(:,1)));
        end
        
        % plot regression on cross sections
        figure(i+1)
        h1 = plot(xy(I,2),xy(I,1),'k-');
        h2 = plot(xy(I,3),xy(I,1),'r-');
        plot(xy(I,4),xy(I,1),'r-');
        legend([h1 h2],'Fault Model','Upper/Lower Residuals');
        
        % store xyz points for fault nodels
        Xc(numel(Xc)+1:numel(Xc)+numel(I),1) = xy(I,2);
        Yc(numel(Yc)+1:numel(Yc)+numel(I),1) = Y{i}.north;
        Zc(numel(Zc)+1:numel(Zc)+numel(I),1) = xy(I,1);

        Xc_ul(numel(Xc_ul)+1:numel(Xc_ul)+numel(I),1) = xy(I,3);
        Yc_ul(numel(Yc_ul)+1:numel(Yc_ul)+numel(I),1) = Y{i}.north;
        Zc_ul(numel(Zc_ul)+1:numel(Zc_ul)+numel(I),1) = xy(I,1);
        
        Xc_ll(numel(Xc_ll)+1:numel(Xc_ll)+numel(I),1) = xy(I,4);
        Yc_ll(numel(Yc_ll)+1:numel(Yc_ll)+numel(I),1) = Y{i}.north;
        Zc_ll(numel(Zc_ll)+1:numel(Zc_ll)+numel(I),1) = xy(I,1);
    else
        % bifercation point
        if X{1,i}.north>=8100&&X{1,i}.north<=11100
                % pre/post 1990 events
                datains(:,1) = [Z{1,i}.pre1990s;Z{1,i}.post1990s];
                datains(:,2) = [X{1,i}.pre1990s;X{1,i}.post1990s];
                dataind(:,1) = [Z{1,i}.pre1990d;Z{1,i}.post1990d];
                dataind(:,2) = [X{1,i}.pre1990d;X{1,i}.post1990d];
                
            xdata = depths;
            
            % regression on cross section using Lowess function
            [dataout lowerLimits upperLimits xys] = lowess(datains,f,wantplot,imagefile,xdata);
            [dataout lowerLimitd upperLimitd xyd] = lowess(dataind,f,wantplot,imagefile,xdata);
                     
            % vertical extent of fault
            if X{1,i}.north==9300
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=max(Z{1,i}.pre1990s));
                Js = find(xdata>=-1900&xdata<=-1300);
                Is = [Is;Js];
            elseif X{1,i}.north==9600
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=max(Z{1,i}.pre1990s));
                Js = find(xdata>=-1900&xdata<=-1600);
                Ks = find(xdata==-5500);
                Is = [Ks;Is;Js];
            elseif X{1,i}.north==10200
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=max(Z{1,i}.pre1990s));
                Js = find(xdata==-2500);
                Is = [Is;Js];
            else
                Is = find(xdata>=min(Z{1,i}.pre1990s)&xdata<=max(Z{1,i}.pre1990s));
            end           
            Is = find(xdata>=min(datains(:,1))&xdata<=max(datains(:,1)));
            Id = find(xdata>=min(dataind(:,1))&xdata<=max(dataind(:,1)));
            
            % plot regression on cross sections
            figure(i+1)
            plot(xys(Is,2),xys(Is,1),'k-');
            plot(xys(Is,3),xys(Is,1),'r-');
            plot(xys(Is,4),xys(Is,1),'r-');
            plot(xyd(Id,2),xyd(Id,1),'k-');
            plot(xyd(Id,3),xyd(Id,1),'r-');
            plot(xyd(Id,4),xyd(Id,1),'r-');
            
            % store xyz points for fault nodels
            Xs(numel(Xs)+1:numel(Xs)+numel(Is),1) = xys(Is,2);
            Ys(numel(Ys)+1:numel(Ys)+numel(Is),1) = Y{i}.north;
            Zs(numel(Zs)+1:numel(Zs)+numel(Is),1) = xys(Is,1);
            
            Xs_ul(numel(Xs_ul)+1:numel(Xs_ul)+numel(Is),1) = xys(Is,3);
            Ys_ul(numel(Ys_ul)+1:numel(Ys_ul)+numel(Is),1) = Y{i}.north;
            Zs_ul(numel(Zs_ul)+1:numel(Zs_ul)+numel(Is),1) = xys(Is,1);
            
            Xs_ll(numel(Xs_ll)+1:numel(Xs_ll)+numel(Is),1) = xys(Is,4);
            Ys_ll(numel(Ys_ll)+1:numel(Ys_ll)+numel(Is),1) = Y{i}.north;
            Zs_ll(numel(Zs_ll)+1:numel(Zs_ll)+numel(Is),1) = xys(Is,1);
            
            Xd(numel(Xd)+1:numel(Xd)+numel(Id),1) = xyd(Id,2);
            Yd(numel(Yd)+1:numel(Yd)+numel(Id),1) = Y{i}.north;
            Zd(numel(Zd)+1:numel(Zd)+numel(Id),1) = xyd(Id,1);
            
            Xd_ul(numel(Xd_ul)+1:numel(Xd_ul)+numel(Id),1) = xyd(Id,3);
            Yd_ul(numel(Yd_ul)+1:numel(Yd_ul)+numel(Id),1) = Y{i}.north;
            Zd_ul(numel(Zd_ul)+1:numel(Zd_ul)+numel(Id),1) = xyd(Id,1);
            
            Xd_ll(numel(Xd_ll)+1:numel(Xd_ll)+numel(Id),1) = xyd(Id,4);
            Yd_ll(numel(Yd_ll)+1:numel(Yd_ll)+numel(Id),1) = Y{i}.north;
            Zd_ll(numel(Zd_ll)+1:numel(Zd_ll)+numel(Id),1) = xyd(Id,1);
        else% shallow fault termination
            
            % pre/post 1990 events
            dataind(:,1) = [Z{1,i}.pre1990d;Z{1,i}.post1990d];
            dataind(:,2) = [X{1,i}.pre1990d;X{1,i}.post1990d];
            xdata = depths;
            
            % regression on cross section using Lowess function
            [dataout lowerLimitd upperLimitd xyd] = lowess(dataind,f,wantplot,imagefile,xdata);
            
            Id = find(xdata>=min(dataind(:,1))&xdata<=max(dataind(:,1)));
            
            figure(i+1)
            plot(xyd(Id,2),xyd(Id,1),'k-');
            plot(xyd(Id,3),xyd(Id,1),'r-');
            plot(xyd(Id,4),xyd(Id,1),'r-');
                        
            % store xyz points for fault nodels
            Xd(numel(Xd)+1:numel(Xd)+numel(Id),1) = xyd(Id,2);
            Yd(numel(Yd)+1:numel(Yd)+numel(Id),1) = Y{i}.north;
            Zd(numel(Zd)+1:numel(Zd)+numel(Id),1) = xyd(Id,1);
            
            Xd_ul(numel(Xd_ul)+1:numel(Xd_ul)+numel(Id),1) = xyd(Id,3);
            Yd_ul(numel(Yd_ul)+1:numel(Yd_ul)+numel(Id),1) = Y{i}.north;
            Zd_ul(numel(Zd_ul)+1:numel(Zd_ul)+numel(Id),1) = xyd(Id,1);
            
            Xd_ll(numel(Xd_ll)+1:numel(Xd_ll)+numel(Id),1) = xyd(Id,4);
            Yd_ll(numel(Yd_ll)+1:numel(Yd_ll)+numel(Id),1) = Y{i}.north;
            Zd_ll(numel(Zd_ll)+1:numel(Zd_ll)+numel(Id),1) = xyd(Id,1);
        end
    end
end

% smooth data in y-direction
Cc = unique(Zc);
Cs = unique(Zs);
Cd = unique(Zd);

Cc_ul = unique(Zc_ul);
Cs_ul = unique(Zs_ul);
Cd_ul = unique(Zd_ul);

Cc_ll = unique(Zc_ll);
Cs_ll = unique(Zs_ll);
Cd_ll = unique(Zd_ll);

for i=1:numel(Cc)
    I = find(Zc==Cc(i));
    [Xc(I),r]=smooth(Xc(I),1);
end
for i=1:numel(Cs)
    I = find(Zs==Cs(i));
    [Xs(I),r]=smooth(Xs(I),1);
end
for i=1:numel(Cd)
    I = find(Zd==Cd(i));
    [Xd(I),r]=smooth(Xd(I),1);
end

for i=1:numel(Cc_ul)
    I = find(Zc_ul==Cc_ul(i));
    [Xc_ul(I),r]=smooth(Xc_ul(I),1);
end
for i=1:numel(Cs_ul)
    I = find(Zs_ul==Cs_ul(i));
    [Xs_ul(I),r]=smooth(Xs_ul(I),1);
end
for i=1:numel(Cd_ul)
    I = find(Zd_ul==Cd_ul(i));
    [Xd_ul(I),r]=smooth(Xd_ul(I),1);
end

for i=1:numel(Cc_ll)
    I = find(Zc_ll==Cc_ll(i));
    [Xc_ll(I),r]=smooth(Xc_ll(I),1);
end
for i=1:numel(Cs_ll)
    I = find(Zs_ll==Cs_ll(i));
    [Xs_ll(I),r]=smooth(Xs_ll(I),1);
end
for i=1:numel(Cd_ll)
    I = find(Zd_ll==Cd_ll(i));
    [Xd_ll(I),r]=smooth(Xd_ll(I),1);
end

% put nodes in 3 column matrix
XYZc(:,1) = Xc;XYZc(:,2) = Yc;XYZc(:,3) = Zc;
XYZs(:,1) = Xs;XYZs(:,2) = Ys;XYZs(:,3) = Zs;
XYZd(:,1) = Xd;XYZd(:,2) = Yd;XYZd(:,3) = Zd;

XYZc_ul(:,1) = Xc_ul;XYZc_ul(:,2) = Yc_ul;XYZc_ul(:,3) = Zc_ul;
XYZs_ul(:,1) = Xs_ul;XYZs_ul(:,2) = Ys_ul;XYZs_ul(:,3) = Zs_ul;
XYZd_ul(:,1) = Xd_ul;XYZd_ul(:,2) = Yd_ul;XYZd_ul(:,3) = Zd_ul;

XYZc_ll(:,1) = Xc_ll;XYZc_ll(:,2) = Yc_ll;XYZc_ll(:,3) = Zc_ll;
XYZs_ll(:,1) = Xs_ll;XYZs_ll(:,2) = Ys_ll;XYZs_ll(:,3) = Zs_ll;
XYZd_ll(:,1) = Xd_ll;XYZd_ll(:,2) = Yd_ll;XYZd_ll(:,3) = Zd_ll;

% connect faults sections
Is = find(XYZc(:,2)==7800&XYZc(:,3)>=-6100);
Id = find(XYZc(:,2)==7800&XYZc(:,3)<=-6100);
XYZs = [XYZc(Is,:);XYZs];
XYZd = [XYZc(Id,:);XYZd];

Is = find(XYZc_ul(:,2)==7800&XYZc_ul(:,3)>=-6100);
Id = find(XYZc_ul(:,2)==7800&XYZc_ul(:,3)<=-6100);
XYZs_ul = [XYZc_ul(Is,:);XYZs_ul];
XYZd_ul = [XYZc_ul(Id,:);XYZd_ul];

Is = find(XYZc_ll(:,2)==7800&XYZc_ll(:,3)>=-6100);
Id = find(XYZc_ll(:,2)==7800&XYZc_ll(:,3)<=-6100);
XYZs_ll = [XYZc_ll(Is,:);XYZs_ll];
XYZd_ll = [XYZc_ll(Id,:);XYZd_ll];

% triangulate fault nodes
CONc=Delaunay2_5D(XYZc);
CONs=Delaunay2_5D(XYZs);
CONd=Delaunay2_5D(XYZd);

CONc_ul=Delaunay2_5D(XYZc_ul);
CONs_ul=Delaunay2_5D(XYZs_ul);
CONd_ul=Delaunay2_5D(XYZd_ul);

CONc_ll=Delaunay2_5D(XYZc_ll);
CONs_ll=Delaunay2_5D(XYZs_ll);
CONd_ll=Delaunay2_5D(XYZd_ll);

% plot fault plane with upper/lower residual fault geometries
figure
for i=1:numel(CONc(:,1))
    plot3(XYZc([CONc(i,:) CONc(i,1)],1),XYZc([CONc(i,:) CONc(i,1)],2),XYZc([CONc(i,:) CONc(i,1)],3),'k-')
    hold on
end
for i=1:numel(CONs(:,1))
    plot3(XYZs([CONs(i,:) CONs(i,1)],1),XYZs([CONs(i,:) CONs(i,1)],2),XYZs([CONs(i,:) CONs(i,1)],3),'k-')
end
for i=1:numel(CONd(:,1))
    plot3(XYZd([CONd(i,:) CONd(i,1)],1),XYZd([CONd(i,:) CONd(i,1)],2),XYZd([CONd(i,:) CONd(i,1)],3),'k-')
end

for i=1:numel(CONc_ul(:,1))
    plot3(XYZc_ul([CONc_ul(i,:) CONc_ul(i,1)],1),XYZc_ul([CONc_ul(i,:) CONc_ul(i,1)],2),XYZc_ul([CONc_ul(i,:) CONc_ul(i,1)],3),'r-')
end
for i=1:numel(CONs_ul(:,1))
    plot3(XYZs_ul([CONs_ul(i,:) CONs_ul(i,1)],1),XYZs_ul([CONs_ul(i,:) CONs_ul(i,1)],2),XYZs_ul([CONs_ul(i,:) CONs_ul(i,1)],3),'r-')
end
for i=1:numel(CONd_ul(:,1))
    plot3(XYZd_ul([CONd_ul(i,:) CONd_ul(i,1)],1),XYZd_ul([CONd_ul(i,:) CONd_ul(i,1)],2),XYZd_ul([CONd_ul(i,:) CONd_ul(i,1)],3),'r-')
end

for i=1:numel(CONc_ll(:,1))
    plot3(XYZc_ll([CONc_ll(i,:) CONc_ll(i,1)],1),XYZc_ll([CONc_ll(i,:) CONc_ll(i,1)],2),XYZc_ll([CONc_ll(i,:) CONc_ll(i,1)],3),'r-')
end
for i=1:numel(CONs_ll(:,1))
    plot3(XYZs_ll([CONs_ll(i,:) CONs_ll(i,1)],1),XYZs_ll([CONs_ll(i,:) CONs_ll(i,1)],2),XYZs_ll([CONs_ll(i,:) CONs_ll(i,1)],3),'r-')
end
for i=1:numel(CONd_ll(:,1))
    plot3(XYZd_ll([CONd_ll(i,:) CONd_ll(i,1)],1),XYZd_ll([CONd_ll(i,:) CONd_ll(i,1)],2),XYZd_ll([CONd_ll(i,:) CONd_ll(i,1)],3),'r-')
end
axis equal;grid on
xlim([0 8000])
ylim([0 18000])
zlim([-14000 0])
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title('North Fault','fontsize',14)

% correct polarity of triangulation
convention = 'z-';
[tnc,nvecc] = PlotTriNormals(CONc,XYZc,convention);
[tns,nvecs] = PlotTriNormals(CONs,XYZs,convention);
[tnd,nvecd] = PlotTriNormals(CONd,XYZd,convention);
[tnc_ul,nvecc_ul] = PlotTriNormals(CONc_ul,XYZc_ul,convention);
[tns_ul,nvecs_ul] = PlotTriNormals(CONs_ul,XYZs_ul,convention);
[tnd_ul,nvecd_ul] = PlotTriNormals(CONd_ul,XYZd_ul,convention);
[tnc_ll,nvecc_ll] = PlotTriNormals(CONc_ll,XYZc_ll,convention);
[tns_ll,nvecs_ll] = PlotTriNormals(CONs_ll,XYZs_ll,convention);
[tnd_ll,nvecd_ll] = PlotTriNormals(CONd_ll,XYZd_ll,convention);

% Correct polarity of upper residual fault
I = find(nvecc_ul(:,1)<0);
for i=1:numel(I)
    a = tnc_ul(I(i),1);
    c = tnc_ul(I(i),3);
    tnc_ul(I(i),1) = c;
    tnc_ul(I(i),3) = a;
    nvecc_ul(I(i),:) = -nvecc_ul(I(i),:);
end

I = find(nvecs_ul(:,1)<0);
for i=1:numel(I)
    a = tns_ul(I(i),1);
    c = tns_ul(I(i),3);
    tns_ul(I(i),1) = c;
    tns_ul(I(i),3) = a;
    nvecs_ul(I(i),:) = -nvecs_ul(I(i),:);
end

I = find(nvecd_ul(:,1)<0);
for i=1:numel(I)
    a = tnd_ul(I(i),1);
    c = tnd_ul(I(i),3);
    tnd_ul(I(i),1) = c;
    tnd_ul(I(i),3) = a;
    nvecd_ul(I(i),:) = -nvecd_ul(I(i),:);
end

% Correct polarity of lower residual fault
I = find(nvecc_ll(:,1)<0);
for i=1:numel(I)
    a = tnc_ll(I(i),1);
    c = tnc_ll(I(i),3);
    tnc_ll(I(i),1) = c;
    tnc_ll(I(i),3) = a;
    nvecc_ll(I(i),:) = -nvecc_ll(I(i),:);
end

I = find(nvecs_ll(:,1)<0);
for i=1:numel(I)
    a = tns_ll(I(i),1);
    c = tns_ll(I(i),3);
    tns_ll(I(i),1) = c;
    tns_ll(I(i),3) = a;
    nvecs_ll(I(i),:) = -nvecs_ll(I(i),:);
end

I = find(nvecd_ll(:,1)<0);
for i=1:numel(I)
    a = tnd_ll(I(i),1);
    c = tnd_ll(I(i),3);
    tnd_ll(I(i),1) = c;
    tnd_ll(I(i),3) = a;
    nvecd_ll(I(i),:) = -nvecd_ll(I(i),:);
end

%% Translate Faults back to UTM

% north fault
XYZc(:,1) = XYZc(:,1) + xo;
XYZc(:,2) = XYZc(:,2) + yo;
XYZs(:,1) = XYZs(:,1) + xo;
XYZs(:,2) = XYZs(:,2) + yo;
XYZd(:,1) = XYZd(:,1) + xo;
XYZd(:,2) = XYZd(:,2) + yo;

XYZc_ul(:,1) = XYZc_ul(:,1) + xo;
XYZc_ul(:,2) = XYZc_ul(:,2) + yo;
XYZs_ul(:,1) = XYZs_ul(:,1) + xo;
XYZs_ul(:,2) = XYZs_ul(:,2) + yo;
XYZd_ul(:,1) = XYZd_ul(:,1) + xo;
XYZd_ul(:,2) = XYZd_ul(:,2) + yo;

XYZc_ll(:,1) = XYZc_ll(:,1) + xo;
XYZc_ll(:,2) = XYZc_ll(:,2) + yo;
XYZs_ll(:,1) = XYZs_ll(:,1) + xo;
XYZs_ll(:,2) = XYZs_ll(:,2) + yo;
XYZd_ll(:,1) = XYZd_ll(:,1) + xo;
XYZd_ll(:,2) = XYZd_ll(:,2) + yo;