%% MSH_Plot_EQ_3D_southfault_uncertainty.m
%
% AUTHOR:           Michael W. Swyer
% DATE CREATED:     2015/03/26
% DATE MODIFIED:    2015/08/25
% VERSION:          v.2
%
% PURPOSE:  To construct 3D fault geometry around Mount Saint Helens for
% permeability potential modeling for play fairway analysis
%
% PURPOSE OF MODIFICATION: To model fault geometry uncertainty on south fault
% which is the upper/lower residuals of the LOWESS regression
% 
% INPUT: Seismic catalogs selected from Pacific Northwest Seismic Network:
%           1) FocalMech_south.csv
%
% OUTPUT: Position of fault nodes in UTM Zone 10N meters (XYZ-) and for upper,
% (XYZ-_ul) and lower (XYZ-_ll) residuals, triangulation of fault nodes (tn-),
% and for upper (tn-_ul) and lower (tn-_ll) residuals, and unit normal vectors
% of fault element triangles (nvec-), and upper (nvec-_ul) and lower (nvec-_ll)
% residuals.
%
% NOTE:
%
% KEY functions written specifically for this modeling:
%
% Other related functions:
%       deg2utm.m
%       fitNormal.m
%       lowess.m
%       smooth.m
%       R3RSR.m
%       R3R.m
%       SR.m
%       H.m
%       Delaunay2_5D.m
%       PlotTriNormals.m

clear all;close all

show_graph = 0;% Show graph of fit normal (0=no 1=yes)

%% Import Data

%%% south fault %%%
dataSF = importdata('FocalMech_south.csv',',');
for i=2:numel(dataSF)
    d = strsplit(dataSF{i},',');
    datetime = strsplit(d{5});
    date = strsplit(datetime{1},'/');
    Date = [str2num(date{3}) str2num(date{1}) str2num(date{2})];
    dnumSF(i-1,1) = datenum(Date(1),Date(2),Date(3));
    latSF(i-1,1) = str2num(d{6}) + (rand*(10^-7));
    longSF(i-1,1) = str2num(d{7}) + (rand*(10^-7));
    mSF(i-1,1) = str2num(d{8});
    if mSF(i-1,1)<1
        mSF(i-1,1)=1;
    end
    zSF(i-1,1) = -str2num(d{9})*1000 - rand;
end
SSF = 3.^mSF;
[xSF,ySF,utmzone] = deg2utm(latSF,longSF);
% UTM origin
xoSF = 564000;
yoSF = 5097500;
xSF = xSF - xoSF;
ySF = ySF - yoSF;
% fit normal vector
if show_graph==1
    figure
end
clear c;clear n
c(:,1) = xSF;c(:,2) = ySF;c(:,3) = zSF;
n = fitNormal(c, show_graph);
thetaSF = atan(n(2)/n(1));
% rotate to j=0
Rcw = [cos(thetaSF) sin(thetaSF);-sin(thetaSF) cos(thetaSF)];
for i=1:numel(xSF)
    b = Rcw*[xSF(i);ySF(i)];
    xSF(i) = b(1)-4000;ySF(i) = b(2)+3000;
end
% rotate about y-axis
zSF = zSF + 8000;xSF = xSF - 1750;
Rcw = [cos(17*(pi/180)) sin(17*(pi/180));-sin(17*(pi/180)) cos(17*(pi/180))];
for i=1:numel(xSF)
    b = Rcw*[xSF(i);zSF(i)];
    xSF(i) = b(1);zSF(i) = b(2);
end
zSF = zSF - 8000;xSF = xSF + 1750;

%% filter data
% by time

% south fault
JSF = find(dnumSF<datenum(1990,1,1));% 1980s
KSF = find(dnumSF>datenum(1990,1,1));% 1990s & 2000s

%% Plot 3D EQs

% south fault
figure
h1 = scatter3(xSF(JSF),ySF(JSF),zSF(JSF),SSF(JSF),'k');
hold on
h2 = scatter3(xSF(KSF),ySF(KSF),zSF(KSF),SSF(KSF),'r');
legend([h1 h2],'1980s','1990s & 2000s');
axis equal;grid on;
xlim([0 5000])
ylim([0 17500])
zlim([-13000 -3000])
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title(strcat('South Fault 1 ',' Strike=',num2str(round(180-thetaSF*(180/pi)))),'fontsize',14)

%% South Fault

% depth slices for node locations
depthsSF = [-13000:300:-3000];depthsSF = depthsSF';
% northing coordinates for cross-sections/node locations
northsSF = [520:300:16420];% 300m spacing

% cross sections
for i=1:numel(northsSF)
        I = find(ySF>(northsSF(i)-11000)&ySF<=(northsSF(i)+11000));
        J = find(dnumSF(I)<datenum(1990,1,1));% 1980s
        K = find(dnumSF(I)>datenum(1990,1,1));% 1990s & 2000s
        figure
        scatter(xSF(I(J)),zSF(I(J)),SSF(I(J)),'k');
        hold on
        scatter(xSF(I(K)),zSF(I(K)),SSF(I(K)),'r');
        axis equal;grid on;
        xlim([0 5000])
        ylim([-13000 -3000])
        xlabel('E-W(m)','fontsize',12)
        ylabel('depth(m)','fontsize',12)
        title(strcat(num2str(northsSF(i)),'m north'),'fontsize',12)
        XSF{i}.pre1990 = xSF(I(J));XSF{i}.post1990 = xSF(I(K));XSF{i}.north = northsSF(i);%XEF1{i}.extent = xEF1(Ie);
        YSF{i}.pre1990 = ySF(I(J));YSF{i}.post1990 = ySF(I(K));YSF{i}.north = northsSF(i);%YEF1{i}.extent = yEF1(Ie);
        ZSF{i}.pre1990 = zSF(I(J));ZSF{i}.post1990 = zSF(I(K));ZSF{i}.north = northsSF(i);%ZEF1{i}.extent = zEF1(Ie);
end

% regression
f = 1;% smoothing factor (0 to 1)
wantplot = 1;% zero = no plot
imagefile = 'lowess_plot\';% directory where plot is stored

% nodes for south fault
Xsf=[];Xsf_ul=[];Xsf_ll=[];
Ysf=[];Ysf_ul=[];Ysf_ll=[];
Zsf=[];Zsf_ul=[];Zsf_ll=[];

for i=1:numel(XSF)
    clear datain
    clear xy
    clear xdata
    clear lowerLimit
    clear upperLimit
    
    % pre/post 1990 events
    if northsSF(i)==16420
        datain(:,1) = [ZSF{1,i-1}.pre1990;ZSF{1,i-1}.post1990];
        datain(:,2) = [XSF{1,i-1}.pre1990;XSF{1,i-1}.post1990];
    else
        datain(:,1) = [ZSF{1,i}.pre1990;ZSF{1,i}.post1990];
        datain(:,2) = [XSF{1,i}.pre1990;XSF{1,i}.post1990];
    end
        xdata = depthsSF;
    
        % regression on cross section using Lowess function
        [dataout lowerLimit upperLimit xy] = lowess(datain,f,wantplot,imagefile,xdata);
    
        % vertical extent of fault 
        if XSF{1,i}.north==520
                I = find(xdata>=min(datain(:,1))&xdata<=-5800&xdata>=-9700);
        elseif XSF{1,i}.north==820
                I = find(xdata>=min(datain(:,1))&xdata<=-4900&xdata>=-10600);
        elseif XSF{1,i}.north==1120
                I = find(xdata>=min(datain(:,1))&xdata<=-4300&xdata>=-11200);
        elseif XSF{1,i}.north==1420
                I = find(xdata>=min(datain(:,1))&xdata<=-4000&xdata>=-11500);
        elseif XSF{1,i}.north==1720
                I = find(xdata>=min(datain(:,1))&xdata<=-3700&xdata>=-11800);
        elseif XSF{1,i}.north==2020
                I = find(xdata>=min(datain(:,1))&xdata<=-3700&xdata>=-11800);
        elseif XSF{1,i}.north==2320
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-12100);
        elseif XSF{1,i}.north==2620
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-12100);
        elseif XSF{1,i}.north==2920
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-12100);
        elseif XSF{1,i}.north==14020
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-11500);
        elseif XSF{1,i}.north==14320
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-11500);
        elseif XSF{1,i}.north==14620
                I = find(xdata>=min(datain(:,1))&xdata<=-3400&xdata>=-11500);
        elseif XSF{1,i}.north==14920
                I = find(xdata>=min(datain(:,1))&xdata<=-3700&xdata>=-11200);
        elseif XSF{1,i}.north==15220
                I = find(xdata>=min(datain(:,1))&xdata<=-3700&xdata>=-11200);
        elseif XSF{1,i}.north==15520
                I = find(xdata>=min(datain(:,1))&xdata<=-4000&xdata>=-10900);
        elseif XSF{1,i}.north==15820
                I = find(xdata>=min(datain(:,1))&xdata<=-4300&xdata>=-10600);
        elseif XSF{1,i}.north==16120
                I = find(xdata>=min(datain(:,1))&xdata<=-4900&xdata>=-10000);
        elseif XSF{1,i}.north==16420
                I = find(xdata>=min(datain(:,1))&xdata<=-5800&xdata>=-9400);
        else
                I = find(xdata>=min(datain(:,1)));%&xdata<=max(datain(:,1)));
        end

        % plot regression on cross sections
        figure(i+1)
        h1 = plot(xy(I,2),xy(I,1),'k-');
        h2 = plot(xy(I,3),xy(I,1),'r-');
        plot(xy(I,4),xy(I,1),'r-');
        legend([h1 h2],'Fault Model','Upper/Lower Residuals');

        % store xyz points for fault nodels
        Xsf(numel(Xsf)+1:numel(Xsf)+numel(I),1) = xy(I,2);
        Ysf(numel(Ysf)+1:numel(Ysf)+numel(I),1) = YSF{i}.north;
        Zsf(numel(Zsf)+1:numel(Zsf)+numel(I),1) = xy(I,1);
        
        Xsf_ul(numel(Xsf_ul)+1:numel(Xsf_ul)+numel(I),1) = xy(I,3);
        Ysf_ul(numel(Ysf_ul)+1:numel(Ysf_ul)+numel(I),1) = YSF{i}.north;
        Zsf_ul(numel(Zsf_ul)+1:numel(Zsf_ul)+numel(I),1) = xy(I,1);
        
        Xsf_ll(numel(Xsf_ll)+1:numel(Xsf_ll)+numel(I),1) = xy(I,4);
        Ysf_ll(numel(Ysf_ll)+1:numel(Ysf_ll)+numel(I),1) = YSF{i}.north;
        Zsf_ll(numel(Zsf_ll)+1:numel(Zsf_ll)+numel(I),1) = xy(I,1);
end

% smooth data in y-direction
Csf = unique(Zsf);
for i=1:numel(Csf)
    I = find(Zsf==Csf(i));
    [Xsf(I),r]=smooth(Xsf(I),1);
    I = find(Zsf_ul==Csf(i));
    [Xsf_ul(I),r]=smooth(Xsf_ul(I),1);
    I = find(Zsf_ll==Csf(i));
    [Xsf_ll(I),r]=smooth(Xsf_ll(I),1);
end

% put nodes in 3 column matrix
XYZsf(:,1) = Xsf;XYZsf(:,2) = Ysf;XYZsf(:,3) = Zsf;
XYZsf_ul(:,1) = Xsf_ul;XYZsf_ul(:,2) = Ysf_ul;XYZsf_ul(:,3) = Zsf_ul;
XYZsf_ll(:,1) = Xsf_ll;XYZsf_ll(:,2) = Ysf_ll;XYZsf_ll(:,3) = Zsf_ll;

% triangulate fault nodes
CONsf=Delaunay2_5D(XYZsf);
CONsf_ul=Delaunay2_5D(XYZsf_ul);
CONsf_ll=Delaunay2_5D(XYZsf_ll);

% plot fault plane with upper/lower residual fault geometries
figure
for i=1:numel(CONsf(:,1))
    plot3(XYZsf([CONsf(i,:) CONsf(i,1)],1),XYZsf([CONsf(i,:) CONsf(i,1)],2),XYZsf([CONsf(i,:) CONsf(i,1)],3),'k-')
    hold on
end
for i=1:numel(CONsf_ul(:,1))
    plot3(XYZsf_ul([CONsf_ul(i,:) CONsf_ul(i,1)],1),XYZsf_ul([CONsf_ul(i,:) CONsf_ul(i,1)],2),XYZsf_ul([CONsf_ul(i,:) CONsf_ul(i,1)],3),'r-')
end
for i=1:numel(CONsf_ll(:,1))
    plot3(XYZsf_ll([CONsf_ll(i,:) CONsf_ll(i,1)],1),XYZsf_ll([CONsf_ll(i,:) CONsf_ll(i,1)],2),XYZsf_ll([CONsf_ll(i,:) CONsf_ll(i,1)],3),'r-')
end
axis equal;grid on;
xlim([0 5000])
ylim([0 17500])
zlim([-13000 -3000])
xlabel('E-W(m)','fontsize',12)
ylabel('N-S(m)','fontsize',12)
zlabel('depth(m)','fontsize',12)
title(strcat('South Fault 1 ',' Strike=',num2str(round(180-thetaSF*(180/pi)))),'fontsize',14)

% Rotate back
Rccw = [cos(thetaSF) -sin(thetaSF);sin(thetaSF) cos(thetaSF)];
for i=1:numel(Xsf)
    b = Rccw*[(XYZsf(i,1)+4000);XYZsf(i,2)-3000];
    XYZsf(i,1) = b(1);XYZsf(i,2) = b(2);
    b = Rccw*[(XYZsf_ul(i,1)+4000);XYZsf_ul(i,2)-3000];
    XYZsf_ul(i,1) = b(1);XYZsf_ul(i,2) = b(2);
    b = Rccw*[(XYZsf_ll(i,1)+4000);XYZsf_ll(i,2)-3000];
    XYZsf_ll(i,1) = b(1);XYZsf_ll(i,2) = b(2);
end

% correct polarity of triangulation
convention = 'z-';
[tnsf,nvecsf] = PlotTriNormals(CONsf,XYZsf,convention);
[tnsf_ul,nvecsf_ul] = PlotTriNormals(CONsf_ul,XYZsf_ul,convention);
[tnsf_ll,nvecsf_ll] = PlotTriNormals(CONsf_ll,XYZsf_ll,convention);

% Correct polarity of upper residual fault
I = find(nvecsf_ul(:,1)<0);
for i=1:numel(I)
    a = tnsf_ul(I(i),1);
    c = tnsf_ul(I(i),3);
    tnsf_ul(I(i),1) = c;
    tnsf_ul(I(i),3) = a;
    nvecsf_ul(I(i),:) = -nvecsf_ul(I(i),:);
end

% Correct polarity of lower residual fault
I = find(nvecsf_ll(:,1)<0);
for i=1:numel(I)
    a = tnsf_ll(I(i),1);
    c = tnsf_ll(I(i),3);
    tnsf_ll(I(i),1) = c;
    tnsf_ll(I(i),3) = a;
    nvecsf_ll(I(i),:) = -nvecsf_ll(I(i),:);
end

%% Translate Faults back to UTM

% south fault
XYZsf(:,1) = XYZsf(:,1) + xoSF;
XYZsf(:,2) = XYZsf(:,2) + yoSF;

XYZsf_ul(:,1) = XYZsf_ul(:,1) + xoSF;
XYZsf_ul(:,2) = XYZsf_ul(:,2) + yoSF;

XYZsf_ll(:,1) = XYZsf_ll(:,1) + xoSF;
XYZsf_ll(:,2) = XYZsf_ll(:,2) + yoSF;