% MASTER_p3d_MSH_v2.m
%
% AUTHOR:           Michael Swyer
% DATE CREATED:     2/27/2015
% DATE MODIFIED:    6/22/2015
% VERSION:          v.2
%
% PURPOSE:
%   Script to run poly3d, version from 1999, in c++ is slaved to matlab
%   for:
%   - Modeling the faults at Mount Saint Helens
%
% OBSERVATION GRIDS:
%      Areal grid: 2D
%           -Extent of WADGER thermal model for MSH with 2000 ft obs point
%           spacing converted to meters UTM Zone 10N.
%           -Extent of AltaRock lease.
%          
% PURPOSE OF MODIFICATION: To model stress boundary using rotational strain
% component and strains removed from near block boundary. Also, run 200m
% and 3km models back to back.
%
% INPUT: Fault nodes and connectivity in global coordinates NAD_1983_UTM_ZONE_10N
%       , Stress tensors for 200m and 3000m depths:
%           -MSH_NorthFault.xls
%           -MSH_EastFaults.xls
%           -MSH_SouthFault.xls
%           -MSH_WestFaults.xls
%           -StressTensorMSH200.csv
%           -StressTensorMSH3km.csv
%           -FocalMech_obspoints.csv
% 
% OUTPUT: Poly 3D Input and Output files named:
%           -MtStHelens200m.in
%           -MtStHelens200m.out
%           -MtStHelens3km.in
%           -MtStHelens3km.out
%
% NOTE: 
% - Based on original files drafted by J. Ole Kaven
% 
% - Note specific paths to the folder containg p3d and to the folder
% containing this mfile are specified for running p3d and saving .in and
% .out files respectively
% 
% KEY functions written specifically for this modeling
%
% Other related functions
% - p3din_FAULT_func_v1_200m
% - p3din_FAULT_func_v1_3km
% - EllipsoidTri.m
%

clear all, close all

tic

%% 1. Define OBSERVATION GRID RANGES
% Generate observation grids in global coordinates.

% AREAL GRIDS: 2D

% Extent of WADGER thermal model
% User Coordinate System
x1o = 537614.064424;
x2o = 5100613.11251;
x3o = 0;
rot1 = 0;
rot2 = 0;
rot3 = 1.8128;
CS_WADGER = [x1o, x2o, x3o, rot1, rot2, rot3, 123];% matrix used in p3din_FAULT_func_v1
% min/max values 2000ft spacing at 200m depth
minx = 0;  % min value of x
maxx = 40221.072401;  % max value of x
miny = 0;  % min value of y
maxy = 37173.820308;  % max value of y
minz = -200;  % min value of z
maxz = -200;  % max value of z    
npx  = 67; % number of points corresponding to inc in each dimension
npy  = 62; % see also function: linspace
npz  = 1;
OBSgridWADGER200 = [minx miny minz maxx maxy maxz npx npy npz];% matrix used in p3din_FAULT_func_v1
% min/max values 2000ft spacing at 3000m depth
minx = 0;  % min value of x
maxx = 40221.072401;  % max value of x
miny = 0;  % min value of y
maxy = 37173.820308;  % max value of y
minz = -3000;  % min value of z
maxz = -3000;  % max value of z
npx  = 67; % number of points corresponding to inc in each dimension
npy  = 62; % see also function: linspace
npz  = 1;
OBSgridWADGER3km = [minx miny minz maxx maxy maxz npx npy npz];% matrix used in p3din_FAULT_func_v1

%% 2. Define Boundary Conditions to run

% depth to water table(meters)
dw = 20;

% import stress tensor(s)
% 200m
Sigma200 = importdata('StressTensorMSH200.csv',',');% Stress Tensor
d = .2;% depth km
Pp200 = (9.733*(d-(dw/1000)));
Pf200 = Pp200*1000000;% fluid pressure used in poly3D
% 3000m
Sigma3km = importdata('StressTensorMSH3km.csv',',');% Stress Tensor
d = 3;% depth km
Pp3km = (9.733*(d-(dw/1000)));
Pf3km = Pp3km*1000000;% fluid pressure used in poly3D

% Elastic Properties
nu = 0.25; % poisson's ratio
E = 30000000000; % Young's Modulus in Pa
       
%% 3. IMPORT FAULTS
    
    % North Fault
    XYZc = xlsread('MSH_NorthFault.xls','XYZc','A2:C914');
    tnc = xlsread('MSH_NorthFault.xls','tnc','A1:C1706');
    XYZs = xlsread('MSH_NorthFault.xls','XYZs','A2:C161');
    tns = xlsread('MSH_NorthFault.xls','tns','A1:C271');
    XYZd = xlsread('MSH_NorthFault.xls','XYZd','A2:C439');
    tnd = xlsread('MSH_NorthFault.xls','tnd','A1:C796');

    % East Fault 1
    XYZef1 = xlsread('MSH_EastFaults.xls','XYZef1','A2:C32');
    tnef1 = xlsread('MSH_EastFaults.xls','tnef1','A1:C43');
    % East Fault 2
    XYZef2 = xlsread('MSH_EastFaults.xls','XYZef2','A2:C45');
    tnef2 = xlsread('MSH_EastFaults.xls','tnef2','A1:C63');

    % South Fault
    XYZsf = xlsread('MSH_SouthFault.xls','XYZsf','A2:C1104');
    tnsf = xlsread('MSH_SouthFault.xls','tnsf','A1:C2092');
    
    % West Fault 1
    XYZwf1 = xlsread('MSH_WestFaults.xls','XYZwf1','A2:C269');
    tnwf1 = xlsread('MSH_WestFaults.xls','tnwf1','A1:C481');
    % West Fault 2
    XYZwf2 = xlsread('MSH_WestFaults.xls','XYZwf2','A2:C114');
    tnwf2 = xlsread('MSH_WestFaults.xls','tnwf2','A1:C188');
    % Goat Mountain Fault
    XYZgm = xlsread('MSH_WestFaults.xls','XYZgm','A2:C205');
    tngm = xlsread('MSH_WestFaults.xls','tngm','A1:C360');
    
%% 4. Create Magma Chamber Ellipsoid
    
% distances given in meters
% position of ellipsoid
LATmc = 46.204;LONmc = -122.187;
[Xmc,Ymc,utmzone] = deg2utm(LATmc,LONmc);
xc = Xmc; yc = Ymc; zc = -9000;
% trend (azimuth) and plunge (positive down) of ellipsoid; gamma is the
% ortation about the semi-major axis to allow rotation of the 
% semi-intermediate and semi-minor axes out of horizontal and vertical
% respectively
trend = 0; plunge = 90; gamma = 0;
% Define anisotropy through the semi-axial lengths
xr = 3000; yr = 1000; zr = 1000;
% number of vertices definine the ellipsoid will be (n+1)^2
n = 15;

% Generate Ellipoid (default outward pointing normals)
[tri,v,nvec,tc,A] = EllipsoidTri(xc,yc,zc,xr,yr,zr,n,trend,plunge,gamma);
XYZmc = v;tnmc = tri;

%% 5. Magma Chamber Boundary Conditions
%  Proposed Poly3D BC on the elements are: 
%   Td = Tx = 0; Ts = Ty = 0; Tn = Tz = Pm
%  where Pm is an equivalent pressure in the magma chamber. and dPm is the
%  difference from the ambient stress field such that positive dPM is 
%  pressure in excess of ambient and negative values are a reudction
%  (suction)
%  and dPm represents a traction everywhere on the chamber surface 
%  defined by the ellipsoid.

%   dPm   derivation:
%       DERIVATION 1:
%       dPm = (rho_r - rho_m)zg; 
%       rho_r = country rock density = 2700 kg.m^-3 (Williams et al, 1987)
%       rho_m = magma density = 2460 km.m^-3 (Olhoeft et al, 1981)

%   Use Pm = rho_m*z*g for normal tractions
Pm200 = -(2460*200*9.81);
Pm3km = -(2460*3000*9.81);

%% 6. RUN p3din_FAULT_v1.m to generate P3D input file and run P3D

    filename = 'MtStHelens200m';
    path_in = strcat('InputOutputFiles/');
    filename_in  = [path_in,filename,'.in'];
    
        % p3din_FAULT_v1
        p3din_FAULT_func_v1_MSH(E,nu,Sigma200,Pf200,filename_in,XYZc,XYZs,XYZd,XYZef1,XYZef2,XYZsf,XYZwf1,XYZwf2,XYZgm,XYZmc,tnc,tns,tnd,tnef1,tnef2,tnsf,tnwf1,tnwf2,tngm,tnmc,Pm200,CS_WADGER,OBSgridWADGER200)
    
        %%% Un-comment these lines to run Poly3D %%%
%         filename_out  = [path_in,filename,'.out'];
%         pcdloc = '/P3D/P3DinC1999/newpoly3d';
%         system([pcdloc,' -i ',filename_in,' -o ',filename_out])

    filename = 'MtStHelens3km';
    path_in = strcat('InputOutputFiles/');
    filename_in  = [path_in,filename,'.in'];
    
        % p3din_FAULT_v1
        p3din_FAULT_func_v1_MSH(E,nu,Sigma3km,Pf3km,filename_in,XYZc,XYZs,XYZd,XYZef1,XYZef2,XYZsf,XYZwf1,XYZwf2,XYZgm,XYZmc,tnc,tns,tnd,tnef1,tnef2,tnsf,tnwf1,tnwf2,tngm,tnmc,Pm3km,CS_WADGER,OBSgridWADGER3km)
    
        %%% Un-comment these lines to run Poly3D %%%
%         filename_out  = [path_in,filename,'.out'];
%         pcdloc = '/P3D/P3DinC1999/newpoly3d';
%         system([pcdloc,' -i ',filename_in,' -o ',filename_out])

toc