% MSH_PermeabilityModel_200m.m
%
% AUTHOR:           Michael Swyer
% DATE CREATED:     8/24/2015
% DATE MODIFIED:    
% VERSION:          v.1
%
% PURPOSE:
%   Read Poly3D .out file and process data for permeability potential
%         
% PURPOSE OF MODIFICATION:
%
% INPUT: Poly3D output file
%           MtStHelens200m.out
% 
% OUTPUT:
%   fault/stress model outputs:
%           -FAULT_GRID: x, y, z, slip tendency, dilation tendency, displacement(m), displacement gradient(m/m)
%           -STRESS_GRID: x, y, z, maximum Coulomb shear stress(Pa), sigma3(Pa)
%   Permeability potential layers:
%           -Fault.sliptendency: x, y, z, slip tendency
%           -Fault.dilationtendency: x, y, z, dilation tendency
%           -Fault.displacement: x, y, z, normalized displacement
%           -Fault.displacementgradient: x, y, z, normalized displacement gradient
%           -Stress.MaximumCoulombShearStress: x, y, z, normalized maximum Coulomb shear stress
%           -Stress.Sigma3: x, y, z, normalized sigma 3
%
% NOTE: 
%
% KEY functions written specifically for this modeling
%
% Other related functions
% - p3doutputreader_v05
% - redblue

clear all;close all

filename = 'InputOutputFiles\MtStHelens200m.out';

out = p3doutputreader_v05(filename);
% Generates structured array including
%   FAULT OBJECTS
%   out.fault.xyz   = array of vertex locations
% 
%   out.fault.vert  = array of vertices
%                     [VERTNUM X1 X2 X3]
% 
%   out.fault.disp  = cell array contain displacements for each fault obj
%                     [ELT X1C X2C X3C B1 U1(+) U1(-) B2 U2(+) U2(-) B3 U3(+)]
%                     ELT = element number, for use in tri
%                     X1C, X2C, X3C = element centroid coordinates
%                     B1, B2, B3 are the burgers vectos components as
%                       (strike,dip,normal)
%                     Ui(+), Ui(-) = relative displacment at centroid
%
%   out.fault.tract = cell array contain tractions for each fault obj
%                     [ELT X1C X2C X3C T1 T2 T3]
%                     T1,T2,T3 are the traction components
%                       (strike,dip,normal)
%   out.fault.name  = cell array contain displacements for each fault obj
% 
%
%   OBSERVATION GRID
%   out.obs.xyz     = array containing the obs grid pts [X1,X2,X3]
%
%   out.obs.disp    = array containing displacements at obs pts
%                     [X1 X2 X3 U1 U2 U3]
%                     U1,U2,U3 = components of the displacement vector
%                     
%   out.obs.pstress = array containing principal stress and principal
%                     stress direcitons at obs pts
%                     [X1 X2 X3 N1 N2 N3 SIG1 N1 N2 N3 SIG2 N1 N2 N3 SIG3]
%                     SIG1,2,3 = principal stress components (tensoral
%                     convention) at each observation point
%                     N1,N2,N3 = principal stress direction vectors for the
%                       following principal stress in the array)
%
%   out.obs.stress  = array contain 
%                     [X1 X2 X3 SIG11 SIG22 SIG33 SIG12 SIG23 SIG13]
%                     SIG11,22,33,12,23,13 = stress tensor component at
%                     each observation point

Sigma200 = importdata('StressTensorMSH200.csv',',');
[V,PS200] = eig(Sigma200);% PS200 = eigenvalues as principal stresses

%% Slip/Dilation Tendency/Displacement/Displacement Gradient

%%% 200m %%%
% Goat Mountain Fault
nvecgm = xlsread('MSH_WestFaults.xls','nvecgm','A2:C361');
I = find(out.fault.disp{1,9}(:,4)<-90&out.fault.disp{1,9}(:,4)>-210);
% Tractions
for i=1:length(I)
    XYZ(i,:) = out.fault.disp{1,9}(I(i),2:4);% XYZ points
    T(i,:) = Sigma200*nvecgm(I(i),:)';% Tractions
    Tn(i,1) = -T(i,:)*nvecgm(I(i),:)';% Normal Tractions
    Tmag(i,1) = (T(i,1)^2 + T(i,2)^2 + T(i,3)^2)^(1/2);% Traction Magnitudes
    DC(i,1) = Tn(i,1)/Tmag(i,1);% Direction Cosines
    theta(i,1) = acos(DC(i,1));% Traction angle from normal vector
    Ts(i,1) = Tmag(i,1)*sin(theta(i,1));% Shear Tractions
    STgm(i,1) = Ts(i,1)/Tn(i,1);% Slip Tendencies
    DTgm(i,1) = ((-PS200(1,1))-(Tn(i,1)))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    Dispgm(i,1) = (out.fault.disp{1,9}(I(i),5)^2 + out.fault.disp{1,9}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZ(:,2));
XYZ = XYZ(I,:);
STgm = STgm(I,1);
DTgm = DTgm(I,1);
Dispgm = Dispgm(I,1);
% Displacement Gradients
for i=1:length(I)
    if i==1
        dg(i,1) = Dispgm(i)/100;
    else
        dg(i,1) = abs(Dispgm(i)-Dispgm(i-1))/(((XYZ(i,1)-XYZ(i-1,1))^2 + (XYZ(i,2)-XYZ(i-1,2))^2 + (XYZ(i,3)-XYZ(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispgm(i)/100;
for i=1:length(I)
    DGgm(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
for i=1:length(I)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XYZ(i,1)-out.obs.pstress{1}(j,1))^2 + (XYZ(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lgm(:,1) = unique(J(:,1));
for i=1:length(Lgm)
    I = find(J(:,1)==Lgm(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZ(J(I(j),2),1)-out.obs.pstress{1}(Lgm(i),1))^2 + (XYZ(J(I(j),2),2)-out.obs.pstress{1}(Lgm(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lgm(i,2) = J(I(K),2);
end

% 24k Faults
data1 = importdata('24k_faults\24k_Fault1.csv',',');
data2 = importdata('24k_faults\24k_Fault2.csv',',');
data3 = importdata('24k_faults\24k_Fault3.csv',',');
data4 = importdata('24k_faults\24k_Fault4.csv',',');
data5 = importdata('24k_faults\24k_Fault5.csv',',');
data6 = importdata('24k_faults\24k_Fault6.csv',',');
data7 = importdata('24k_faults\24k_Fault7.csv',',');
data8 = importdata('24k_faults\24k_Fault8.csv',',');
data9 = importdata('24k_faults\24k_Fault9.csv',',');
data10 = importdata('24k_faults\24k_Fault10.csv',',');
data11 = importdata('24k_faults\24k_Fault11.csv',',');


% Fault1
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data1.data)
    x1 = data1.data(i-1,1);x2 = data1.data(i,1);
    y1 = data1.data(i-1,2);y2 = data1.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST1(i-1,1) = Ts/Tn;% Slip Tendencies
    DT1(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY1(i-1,1) = x1;XY1(i-1,2) = y1;% XY points
end
Disp1 = zeros(length(XY1),1);
DG1 = zeros(length(XY1),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY1)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XY1(i,1)-out.obs.pstress{1}(j,1))^2 + (XY1(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L1(:,1) = unique(J(:,1));
for i=1:length(L1)
    I = find(J(:,1)==L1(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY1(J(I(j),2),1)-out.obs.pstress{1}(L1(i),1))^2 + (XY1(J(I(j),2),2)-out.obs.pstress{1}(L1(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L1(i,2) = J(I(K),2);
end


% Fault2
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data2.data)
    x1 = data2.data(i-1,1);x2 = data2.data(i,1);
    y1 = data2.data(i-1,2);y2 = data2.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST2(i-1,1) = Ts/Tn;% Slip Tendencies
    DT2(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY2(i-1,1) = x1;XY2(i-1,2) = y1;% XY points
end
Disp2 = zeros(length(XY2),1);
DG2 = zeros(length(XY2),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY2)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XY2(i,1)-out.obs.pstress{1}(j,1))^2 + (XY2(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L2(:,1) = unique(J(:,1));
for i=1:length(L2)
    I = find(J(:,1)==L2(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY2(J(I(j),2),1)-out.obs.pstress{1}(L2(i),1))^2 + (XY2(J(I(j),2),2)-out.obs.pstress{1}(L2(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L2(i,2) = J(I(K),2);
end


% Fault3
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data3.data)
    x1 = data3.data(i-1,1);x2 = data3.data(i,1);
    y1 = data3.data(i-1,2);y2 = data3.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST3(i-1,1) = Ts/Tn;% Slip Tendencies
    DT3(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY3(i-1,1) = x1;XY3(i-1,2) = y1;% XY points
end
Disp3 = zeros(length(XY3),1);
DG3 = zeros(length(XY3),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY3)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XY3(i,1)-out.obs.pstress{1}(j,1))^2 + (XY3(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L3(:,1) = unique(J(:,1));
for i=1:length(L3)
    I = find(J(:,1)==L3(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY3(J(I(j),2),1)-out.obs.pstress{1}(L3(i),1))^2 + (XY3(J(I(j),2),2)-out.obs.pstress{1}(L3(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L3(i,2) = J(I(K),2);
end


% Fault4
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data4.data)
    x1 = data4.data(i-1,1);x2 = data4.data(i,1);
    y1 = data4.data(i-1,2);y2 = data4.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn(i-1) = -T'*nvec;% Normal Tractions
    Tmag(i-1) = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC(i-1) = Tn(i-1)/Tmag(i-1);% Direction Cosines
    theta(i-1) = acos(DC(i-1));% Traction angle from normal vector
    Ts(i-1) = Tmag(i-1)*sin(theta(i-1));% Shear Tractions
    ST4(i-1,1) = Ts(i-1)/Tn(i-1);% Slip Tendencies
    DT4(i-1,1) = ((-PS200(1,1))-(Tn(i-1)))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY4(i-1,1) = x1;XY4(i-1,2) = y1;% XY points
end
Disp4 = zeros(length(XY4),1);
DG4 = zeros(length(XY4),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY4)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XY4(i,1)-out.obs.pstress{1}(j,1))^2 + (XY4(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L4(:,1) = unique(J(:,1));
for i=1:length(L4)
    I = find(J(:,1)==L4(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY4(J(I(j),2),1)-out.obs.pstress{1}(L4(i),1))^2 + (XY4(J(I(j),2),2)-out.obs.pstress{1}(L4(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L4(i,2) = J(I(K),2);
end


% Fault5
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data5.data)
    x1 = data5.data(i-1,1);x2 = data5.data(i,1);
    y1 = data5.data(i-1,2);y2 = data5.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST5(i-1,1) = Ts/Tn;% Slip Tendencies
    DT5(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY5(i-1,1) = x1;XY5(i-1,2) = y1;% XY points
end
Disp5 = zeros(length(XY5),1);
DG5 = zeros(length(XY5),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY5)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XY5(i,1)-out.obs.pstress{1}(j,1))^2 + (XY5(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L5(:,1) = unique(J(:,1));
for i=1:length(L5)
    I = find(J(:,1)==L5(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY5(J(I(j),2),1)-out.obs.pstress{1}(L5(i),1))^2 + (XY5(J(I(j),2),2)-out.obs.pstress{1}(L5(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L5(i,2) = J(I(K),2);
end


% Fault6
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data6.data)
    x1 = data6.data(i-1,1);x2 = data6.data(i,1);
    y1 = data6.data(i-1,2);y2 = data6.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST6(i-1,1) = Ts/Tn;% Slip Tendencies
    DT6(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY6(i-1,1) = x1;XY6(i-1,2) = y1;% XY points
end
Disp6 = zeros(length(XY6),1);
DG6 = zeros(length(XY6),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY6)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XY6(i,1)-out.obs.pstress{1}(j,1))^2 + (XY6(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L6(:,1) = unique(J(:,1));
for i=1:length(L6)
    I = find(J(:,1)==L6(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY6(J(I(j),2),1)-out.obs.pstress{1}(L6(i),1))^2 + (XY6(J(I(j),2),2)-out.obs.pstress{1}(L6(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L6(i,2) = J(I(K),2);
end


% Fault7
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data7.data)
    x1 = data7.data(i-1,1);x2 = data7.data(i,1);
    y1 = data7.data(i-1,2);y2 = data7.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST7(i-1,1) = Ts/Tn;% Slip Tendencies
    DT7(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY7(i-1,1) = x1;XY7(i-1,2) = y1;% XY points
end
Disp7 = zeros(length(XY7),1);
DG7 = zeros(length(XY7),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY7)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XY7(i,1)-out.obs.pstress{1}(j,1))^2 + (XY7(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L7(:,1) = unique(J(:,1));
for i=1:length(L7)
    I = find(J(:,1)==L7(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY7(J(I(j),2),1)-out.obs.pstress{1}(L7(i),1))^2 + (XY7(J(I(j),2),2)-out.obs.pstress{1}(L7(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L7(i,2) = J(I(K),2);
end


% Fault8
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data8.data)
    x1 = data8.data(i-1,1);x2 = data8.data(i,1);
    y1 = data8.data(i-1,2);y2 = data8.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST8(i-1,1) = Ts/Tn;% Slip Tendencies
    DT8(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY8(i-1,1) = x1;XY8(i-1,2) = y1;% XY points
end
Disp8 = zeros(length(XY8),1);
DG8 = zeros(length(XY8),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY8)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XY8(i,1)-out.obs.pstress{1}(j,1))^2 + (XY8(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L8(:,1) = unique(J(:,1));
for i=1:length(L8)
    I = find(J(:,1)==L8(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY8(J(I(j),2),1)-out.obs.pstress{1}(L8(i),1))^2 + (XY8(J(I(j),2),2)-out.obs.pstress{1}(L8(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L8(i,2) = J(I(K),2);
end


% Fault9
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data9.data)
    x1 = data9.data(i-1,1);x2 = data9.data(i,1);
    y1 = data9.data(i-1,2);y2 = data9.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST9(i-1,1) = Ts/Tn;% Slip Tendencies
    DT9(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY9(i-1,1) = x1;XY9(i-1,2) = y1;% XY points
end
Disp9 = zeros(length(XY9),1);
DG9 = zeros(length(XY9),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY9(1))
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XY9(i,1)-out.obs.pstress{1}(j,1))^2 + (XY9(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L9(:,1) = unique(J(:,1));
for i=1:length(L9)
    I = find(J(:,1)==L9(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY9(J(I(j),2),1)-out.obs.pstress{1}(L9(i),1))^2 + (XY9(J(I(j),2),2)-out.obs.pstress{1}(L9(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L9(i,2) = J(I(K),2);
end


% Fault10
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data10.data)
    x1 = data10.data(i-1,1);x2 = data10.data(i,1);
    y1 = data10.data(i-1,2);y2 = data10.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST10(i-1,1) = Ts/Tn;% Slip Tendencies
    DT10(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY10(i-1,1) = x1;XY10(i-1,2) = y1;% XY points
end
Disp10 = zeros(length(XY10),1);
DG10 = zeros(length(XY10),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY10)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XY10(i,1)-out.obs.pstress{1}(j,1))^2 + (XY10(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L10(:,1) = unique(J(:,1));
for i=1:length(L10)
    I = find(J(:,1)==L10(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY10(J(I(j),2),1)-out.obs.pstress{1}(L10(i),1))^2 + (XY10(J(I(j),2),2)-out.obs.pstress{1}(L10(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L10(i,2) = J(I(K),2);
end


% Fault11
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data11.data)
    x1 = data11.data(i-1,1);x2 = data11.data(i,1);
    y1 = data11.data(i-1,2);y2 = data11.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST11(i-1,1) = Ts/Tn;% Slip Tendencies
    DT11(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY11(i-1,1) = x1;XY11(i-1,2) = y1;% XY points
end
Disp11 = zeros(length(XY11),1);
DG11 = zeros(length(XY11),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY11)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XY11(i,1)-out.obs.pstress{1}(j,1))^2 + (XY11(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L11(:,1) = unique(J(:,1));
for i=1:length(L11)
    I = find(J(:,1)==L11(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY11(J(I(j),2),1)-out.obs.pstress{1}(L11(i),1))^2 + (XY11(J(I(j),2),2)-out.obs.pstress{1}(L11(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L11(i,2) = J(I(K),2);
end

%% combine fault layers
L = [Lgm;L1;L2;L3;L4;L5;L6;L7;L8;L9;L10;L11];
STall = [STgm(Lgm(:,2));ST1(L1(:,2));ST2(L2(:,2));ST3(L3(:,2));ST4(L4(:,2));ST5(L5(:,2));ST6(L6(:,2));ST7(L7(:,2));ST8(L8(:,2));ST9(L9(:,2));ST10(L10(:,2));ST11(L11(:,2))];
DTall = [DTgm(Lgm(:,2));DT1(L1(:,2));DT2(L2(:,2));DT3(L3(:,2));DT4(L4(:,2));DT5(L5(:,2));DT6(L6(:,2));DT7(L7(:,2));DT8(L8(:,2));DT9(L9(:,2));DT10(L10(:,2));DT11(L11(:,2))];
Dispall = [Dispgm(Lgm(:,2));Disp1(L1(:,2));Disp2(L2(:,2));Disp3(L3(:,2));Disp4(L4(:,2));Disp5(L5(:,2));Disp6(L6(:,2));Disp7(L7(:,2));Disp8(L8(:,2));Disp9(L9(:,2));Disp10(L10(:,2));Disp11(L11(:,2))];
DGall = [DGgm(Lgm(:,2));DG1(L1(:,2));DG2(L2(:,2));DG3(L3(:,2));DG4(L4(:,2));DG5(L5(:,2));DG6(L6(:,2));DG7(L7(:,2));DG8(L8(:,2));DG9(L9(:,2));DG10(L10(:,2));DG11(L11(:,2))];

% put fault layers on map coordinates
k=1;
for i=1:length(out.obs.pstress{1})
    FAULT_GRID(i,1:3) = out.obs.pstress{1}(i,1:3);
    if find(L(:,1)==i)
        if STall(k)>1
            ST(i,1) = 1;
        elseif STall(k)<0
            ST(i,1) = 0;
        else
            ST(i,1) = STall(k);
        end
        DT(i,1) = DTall(k);
        Disp(i,1) = Dispall(k);
        DG(i,1) = DGall(k);
        k=k+1;
    else
        ST(i,1) = 0;
        DT(i,1) = 0;
        Disp(i,1) = 0;
        DG(i,1) = 0;
    end
end

% FAULT_GRID --> X,Y,Z,slip tendency,dilation tendency,displacement,displacement gradient
FAULT_GRID(:,4) = ST;
FAULT_GRID(:,5) = DT;
FAULT_GRID(:,6) = Disp;
FAULT_GRID(:,7) = DG;

%% Combine Stress layers

% calculate Sc and S3
for i=1:length(out.obs.pstress{1})
    Sc(i,1) = (((-out.obs.pstress{1}(i,15))-(-out.obs.pstress{1}(i,7)))/2)*(1+.6^2)^.5 - .6*(((-out.obs.pstress{1}(i,15))+(-out.obs.pstress{1}(i,7)))/2);
    S3(i,1) = (-out.obs.pstress{1}(i,7));
    if find(L(:,1)==i)
        Sc(i,1) = 0;
        S3(i,1) = 0;
    end
end

% extract stress values to compute means and standard deviations
k=1;
for i=1:length(Sc)
    if isnan(Sc(i))
    else
        ScReal(k,1) = Sc(i);
        S3Real(k,1) = S3(i);
        k=k+1;
    end
end

% Mean and Standard Deviation
% Sc
Scmean = mean(ScReal);
SigSc = std(ScReal);
% S3
S3mean = mean(S3Real);
SigS3 = std(S3Real);

% STRESS_GRID --> X,Y,Z,maximum Coulomb shear stress,sigma3
STRESS_GRID(:,1:3) = out.obs.pstress{1}(:,1:3);
STRESS_GRID(:,4) = Sc;
STRESS_GRID(:,5) = S3;

%% Permeability Model Layer Normalization

% Fault Layers
% no normalization needed for slip/dilation tendency (0-1)
Fault.sliptendency(:,1:3) = FAULT_GRID(:,1:3);
Fault.sliptendency(:,4) = FAULT_GRID(:,4);
Fault.dilationtendency(:,1:3) = FAULT_GRID(:,1:3);
Fault.dilationtendency(:,4) = FAULT_GRID(:,5);
% displacement
Fault.displacement(:,1:3) = FAULT_GRID(:,1:3);
Fault.displacement(:,4) = FAULT_GRID(:,6)/max(FAULT_GRID(:,6));
% displacement gradient
Fault.displacementgradient(:,1:3) = FAULT_GRID(:,1:3);
Fault.displacementgradient(:,4) = FAULT_GRID(:,7)/max(FAULT_GRID(:,7));

% normalization of stress between +/- X standard deviations
nsd = 3;% # of standard deviations

% Stress Layers
Stress.MaximumCoulombShearStress(:,1:3) = STRESS_GRID(:,1:3);
Stress.MaximumCoulombShearStress(:,4) = (STRESS_GRID(:,4)-(Scmean-nsd*SigSc))/(2*nsd*SigSc);
I = find(Stress.MaximumCoulombShearStress(:,4)<0);
Stress.MaximumCoulombShearStress(I,4) = 0;
I = find(Stress.MaximumCoulombShearStress(:,4)>1);
Stress.MaximumCoulombShearStress(I,4) = 1;
for i=1:length(L)
    Stress.MaximumCoulombShearStress(L(i,1),4) = 0;
end

% Inverse Shmin values by adding 2*mean to negative values
Stress.Sigma3(:,1:3) = STRESS_GRID(:,1:3);
Stress.Sigma3(:,4) = -STRESS_GRID(:,5);
Stress.Sigma3(:,4) = (Stress.Sigma3(:,4)-(-S3mean-nsd*SigS3))/(2*nsd*SigS3);
I = find(Stress.Sigma3(:,4)<0);
Stress.Sigma3(I,4) = 0;
I = find(Stress.Sigma3(:,4)>1);
Stress.Sigma3(I,4) = 1;
for i=1:length(L)
    Stress.Sigma3(L(i,1),4) = 0;
end
%% FIGURES

figure
plot([1:numel(Sc)],Sc,'k-');ylabel('Sc')
hold on
plot([0 4500],[Scmean+nsd*SigSc Scmean+nsd*SigSc])
plot([0 4500],[Scmean-nsd*SigSc Scmean-nsd*SigSc])
figure
plot([1:numel(S3)],S3,'k-');ylabel('S3')
hold on
plot([0 4500],[S3mean+nsd*SigS3 S3mean+nsd*SigS3])
plot([0 4500],[S3mean-nsd*SigS3 S3mean-nsd*SigS3])

figure
plot([1:numel(Stress.MaximumCoulombShearStress(:,4))],Stress.MaximumCoulombShearStress(:,4),'k.');ylabel('Sc favorability')
figure
plot([1:numel(Stress.Sigma3(:,4))],Stress.Sigma3(:,4),'k.');ylabel('S3 favorability')


% grid data for contour plots
for j=0:61
for i=0:66
    X(1+i,1+j) = out.obs.pstress{1}((1+i)+67*j,1);
    Y(1+i,1+j) = out.obs.pstress{1}((1+i)+67*j,2);
    ScXY(1+i,1+j) = ((((-out.obs.pstress{1}((1+i)+67*j,15))-(-out.obs.pstress{1}((1+i)+67*j,7)))/2)*(1+.6^2)^.5 - .6*(((-out.obs.pstress{1}((1+i)+67*j,15))+(-out.obs.pstress{1}((1+i)+67*j,7)))/2))/1000000;
    S3XY(1+i,1+j) = (-out.obs.pstress{1}((1+i)+67*j,7))/1000000;
    if find(L(:,1)==((1+i)+67*j))
        ScXY(1+i,1+j) = NaN;
        S3XY(1+i,1+j) = NaN;
    end
end
end

% Sc contour intervals
Vm = [(Scmean-nsd*SigSc)/1000000:.1:(Scmean+nsd*SigSc)/1000000];
figure
contourf(X,Y,ScXY,Vm)
hold on
contour(X,Y,ScXY,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sc(MPa)','Fontsize',14);
caxis([(Scmean-nsd*SigSc)/1000000 (Scmean+nsd*SigSc)/1000000]);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% S3 contour intervals
Vm = [(S3mean-nsd*SigS3)/1000000:.1:(S3mean+nsd*SigS3)/1000000];
figure
contourf(X,Y,S3XY,Vm)
hold on
contour(X,Y,S3XY,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sigma3(MPa)','Fontsize',14);
caxis([(S3mean-nsd*SigS3)/1000000 (S3mean+nsd*SigS3)/1000000]);
colormap(redblue);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)
