% MSH_PermeabilityModel_3km.m
%
% AUTHOR:           Michael Swyer
% DATE CREATED:     8/24/2015
% DATE MODIFIED:    
% VERSION:          v.1
%
% PURPOSE:
%   Read Poly3D .out file and process data for permeability potential
%         
% PURPOSE OF MODIFICATION:
%
% INPUT: Poly3D output file
%           MtStHelens3km.out
% 
% OUTPUT:
%   fault/stress model outputs:
%           -FAULT_GRID: x, y, z, slip tendency, dilation tendency, displacement(m), displacement gradient(m/m)
%           -STRESS_GRID: x, y, z, maximum Coulomb shear stress(Pa), sigma3(Pa)
%   Permeability potential layers:
%           -Fault.sliptendency: x, y, z, slip tendency
%           -Fault.dilationtendency: x, y, z, dilation tendency
%           -Fault.displacement: x, y, z, normalized displacement
%           -Fault.displacementgradient: x, y, z, normalized displacement gradient
%           -Stress.MaximumCoulombShearStress: x, y, z, normalized maximum Coulomb shear stress
%           -Stress.Sigma3: x, y, z, normalized sigma 3
%
% NOTE: 
%
% KEY functions written specifically for this modeling
%
% Other related functions
% - p3doutputreader_v05
% - redblue

clear all;close all

filename = 'InputOutputFiles\MtStHelens3km.out';

out = p3doutputreader_v05(filename);
% Generates structured array including
%   FAULT OBJECTS
%   out.fault.xyz   = array of vertex locations
% 
%   out.fault.vert  = array of vertices
%                     [VERTNUM X1 X2 X3]
% 
%   out.fault.disp  = cell array contain displacements for each fault obj
%                     [ELT X1C X2C X3C B1 U1(+) U1(-) B2 U2(+) U2(-) B3 U3(+)]
%                     ELT = element number, for use in tri
%                     X1C, X2C, X3C = element centroid coordinates
%                     B1, B2, B3 are the burgers vectos components as
%                       (strike,dip,normal)
%                     Ui(+), Ui(-) = relative displacment at centroid
%
%   out.fault.tract = cell array contain tractions for each fault obj
%                     [ELT X1C X2C X3C T1 T2 T3]
%                     T1,T2,T3 are the traction components
%                       (strike,dip,normal)
%   out.fault.name  = cell array contain displacements for each fault obj
% 
%
%   OBSERVATION GRID
%   out.obs.xyz     = array containing the obs grid pts [X1,X2,X3]
%
%   out.obs.disp    = array containing displacements at obs pts
%                     [X1 X2 X3 U1 U2 U3]
%                     U1,U2,U3 = components of the displacement vector
%                     
%   out.obs.pstress = array containing principal stress and principal
%                     stress direcitons at obs pts
%                     [X1 X2 X3 N1 N2 N3 SIG1 N1 N2 N3 SIG2 N1 N2 N3 SIG3]
%                     SIG1,2,3 = principal stress components (tensoral
%                     convention) at each observation point
%                     N1,N2,N3 = principal stress direction vectors for the
%                       following principal stress in the array)
%
%   out.obs.stress  = array contain 
%                     [X1 X2 X3 SIG11 SIG22 SIG33 SIG12 SIG23 SIG13]
%                     SIG11,22,33,12,23,13 = stress tensor component at
%                     each observation point

Sigma3km = importdata('StressTensorMSH3km.csv',',');
[V,PS3km] = eig(Sigma3km);% PS3km = eigenvalues as principal stresses

%% Slip/Dilation Tendency/Displacement/Displacement Gradient

%%% 3000m %%%

% Goat Mountain Fault
nvecgm = xlsread('MSH_WestFaults.xls','nvecgm','A2:C361');
I = find(out.fault.disp{1,9}(:,4)<-2600&out.fault.disp{1,9}(:,4)>-3000);
% Tractions
for i=1:length(I)
    XYZgm(i,:) = out.fault.disp{1,9}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecgm(I(i),:)';% Tractions
    Tn = -T'*nvecgm(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STgm(i,1) = Ts/Tn;% Slip Tendencies
    DTgm(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Dispgm(i,1) = (out.fault.disp{1,9}(I(i),5)^2 + out.fault.disp{1,9}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZgm(:,2));
XYZgm = XYZgm(I,:);
STgm = STgm(I,1);
DTgm = DTgm(I,1);
Dispgm = Dispgm(I,1);
% Displacement Gradients
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Dispgm(i)/100;
    else
        dg(i,1) = abs(Dispgm(i)-Dispgm(i-1))/(((XYZgm(i,1)-XYZgm(i-1,1))^2 + (XYZgm(i,2)-XYZgm(i-1,2))^2 + (XYZgm(i,3)-XYZgm(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispgm(i)/100;
for i=1:length(I)
    DGgm(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XYZgm(i,1)-out.obs.pstress{1}(j,1))^2 + (XYZgm(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lgm(:,1) = unique(J(:,1));
for i=1:length(Lgm)
    I = find(J(:,1)==Lgm(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZgm(J(I(j),2),1)-out.obs.pstress{1}(Lgm(i),1))^2 + (XYZgm(J(I(j),2),2)-out.obs.pstress{1}(Lgm(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lgm(i,2) = J(I(K),2);
end


% North Fault
nvecc = xlsread('MSH_NorthFault.xls','nvecc','A2:C1733');
I = find(out.fault.disp{1,1}(:,4)<-2800&out.fault.disp{1,1}(:,4)>-3100);
% Tractions
for i=1:length(I)
    XYZc(i,:) = out.fault.disp{1,1}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecc(I(i),:)';% Tractions
    Tn = -T'*nvecc(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STc(i,1) = Ts/Tn;% Slip Tendencies
    DTc(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Dispc(i,1) = (out.fault.disp{1,1}(I(i),5)^2 + out.fault.disp{1,1}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZc(:,2));
XYZc = XYZc(I,:);
STc = STc(I,1);
DTc = DTc(I,1);
Dispc = Dispc(I,1);
% Displacement Gradientws
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Dispc(i)/100;
    else
        dg(i,1) = abs(Dispc(i)-Dispc(i-1))/(((XYZc(i,1)-XYZc(i-1,1))^2 + (XYZc(i,2)-XYZc(i-1,2))^2 + (XYZc(i,3)-XYZc(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispc(i)/100;
for i=1:length(I)
    DGc(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XYZc(i,1)-out.obs.pstress{1}(j,1))^2 + (XYZc(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lc(:,1) = unique(J(:,1));
for i=1:length(Lc)
    I = find(J(:,1)==Lc(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZc(J(I(j),2),1)-out.obs.pstress{1}(Lc(i),1))^2 + (XYZc(J(I(j),2),2)-out.obs.pstress{1}(Lc(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lc(i,2) = J(I(K),2);
end


% North Fault Shallow
nvecs = xlsread('MSH_NorthFault.xls','nvecs','A2:C272');
I = find(out.fault.disp{1,2}(:,4)<-2800&out.fault.disp{1,2}(:,4)>-3100);
% Tractions
for i=1:length(I)
    XYZs(i,:) = out.fault.disp{1,2}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecs(I(i),:)';% Tractions
    Tn = -T'*nvecs(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STs(i,1) = Ts/Tn;% Slip Tendencies
    DTs(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Disps(i,1) = (out.fault.disp{1,2}(I(i),5)^2 + out.fault.disp{1,2}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZs(:,2));
XYZs = XYZs(I,:);
STs = STs(I,1);
DTs = DTs(I,1);
Disps = Disps(I,1);
% Displacement Gradientws
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Disps(i)/100;
    else
        dg(i,1) = abs(Disps(i)-Disps(i-1))/(((XYZs(i,1)-XYZs(i-1,1))^2 + (XYZs(i,2)-XYZs(i-1,2))^2 + (XYZs(i,3)-XYZs(i-1,3))^2)^.5);
    end
end
dg(i+1) = Disps(i)/100;
for i=1:length(I)
    DGs(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XYZs(i,1)-out.obs.pstress{1}(j,1))^2 + (XYZs(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Ls(:,1) = unique(J(:,1));
for i=1:length(Ls)
    I = find(J(:,1)==Ls(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZs(J(I(j),2),1)-out.obs.pstress{1}(Ls(i),2))^2 + (XYZs(J(I(j),2),2)-out.obs.pstress{1}(Ls(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Ls(i,2) = J(I(K),2);
end


% South Fault
nvecsf = xlsread('MSH_SouthFault.xls','nvecsf','A2:C3106');
I = find(out.fault.disp{1,6}(:,4)<-3100&out.fault.disp{1,6}(:,4)>-3400);
% Tractions
for i=1:length(I)
    XYZsf(i,:) = out.fault.disp{1,6}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecsf(I(i),:)';% Tractions
    Tn = -T'*nvecsf(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STsf(i,1) = Ts/Tn;% Slip Tendencies
    DTsf(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Dispsf(i,1) = (out.fault.disp{1,6}(I(i),5)^2 + out.fault.disp{1,6}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZsf(:,2));
XYZsf = XYZsf(I,:);
STsf = STsf(I,1);
DTsf = DTsf(I,1);
Dispsf = Dispsf(I,1);
% Displacement Gradientws
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Dispsf(i)/100;
    else
        dg(i,1) = abs(Dispsf(i)-Dispsf(i-1))/(((XYZsf(i,1)-XYZsf(i-1,1))^2 + (XYZsf(i,2)-XYZsf(i-1,2))^2 + (XYZsf(i,3)-XYZsf(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispsf(i)/100;
for i=1:length(I)
    DGsf(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out.obs.pstress{1})
        Dobs(j) = ((XYZsf(i,1)-out.obs.pstress{1}(j,1))^2 + (XYZsf(i,2)-out.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lsf(:,1) = unique(J(:,1));
for i=1:length(Lsf)
    I = find(J(:,1)==Lsf(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZsf(J(I(j),2),1)-out.obs.pstress{1}(Lsf(i),2))^2 + (XYZsf(J(I(j),2),2)-out.obs.pstress{1}(Lsf(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lsf(i,2) = J(I(K),2);
end

%% combine fault layers
L = [Lgm;Lc;Ls;Lsf];
STall = [STgm(Lgm(:,2));STc(Lc(:,2));STs(Ls(:,2));STsf(Lsf(:,2))];
DTall = [DTgm(Lgm(:,2));DTc(Lc(:,2));DTs(Ls(:,2));DTsf(Lsf(:,2))];
Dispall = [Dispgm(Lgm(:,2));Dispc(Lc(:,2));Disps(Ls(:,2));Dispsf(Lsf(:,2))];
DGall = [DGgm(Lgm(:,2));DGc(Lc(:,2));DGs(Ls(:,2));DGsf(Lsf(:,2))];

% put fault layers on map coordinates
k=1;
for i=1:length(out.obs.pstress{1})
    FAULT_GRID(i,1:3) = out.obs.pstress{1}(i,1:3);
    if find(L(:,1)==i)
        if STall(k)>1
            ST(i,1) = 1;
        elseif STall(k)<0
            ST(i,1) = 0;
        else
            ST(i,1) = STall(k);
        end
        DT(i,1) = DTall(k);
        Disp(i,1) = Dispall(k);
        DG(i,1) = DGall(k);
        k=k+1;
    else
        ST(i,1) = 0;
        DT(i,1) = 0;
        Disp(i,1) = 0;
        DG(i,1) = 0;
    end
end

% FAULT_GRID --> X,Y,Z,slip tendency,dilation tendency,displacement,displacement gradient
FAULT_GRID(:,4) = ST;
FAULT_GRID(:,5) = DT;
FAULT_GRID(:,6) = Disp;
FAULT_GRID(:,7) = DG;

%% Combine Stress layers

% calculate Sc and S3
for i=1:length(out.obs.pstress{1})
    Sc(i,1) = (((-out.obs.pstress{1}(i,15))-(-out.obs.pstress{1}(i,7)))/2)*(1+.6^2)^.5 - .6*(((-out.obs.pstress{1}(i,15))+(-out.obs.pstress{1}(i,7)))/2);
    S3(i,1) = (-out.obs.pstress{1}(i,7));
    if find(L(:,1)==i)
        Sc(i,1) = NaN;
        S3(i,1) = NaN;
    end
end

% extract stress values to compute means and standard deviations
k=1;
for i=1:length(Sc)
    if isnan(Sc(i,1))
        Sc(i,1) = 0;
        S3(i,1) = 0;
    else
        ScReal(k,1) = Sc(i);
        S3Real(k,1) = S3(i);
        k=k+1;
    end
end

% Mean and Standard Deviation
% Sc
Scmean = mean(ScReal);
SigSc = std(ScReal);
% Shmin
S3mean = mean(S3Real);
SigS3 = std(S3Real);

nsd = .05;% # of standard deviations to filter out anomolies

% Filter out nomolies
k=1;
for i=1:length(ScReal)
    if ScReal(i)<(Scmean+nsd*SigSc)&&ScReal(i)>(Scmean-nsd*SigSc)
        Screal(k) = ScReal(i);
        S3real(k) = S3Real(i);
        k=k+1;
    end
end

nsd = 3;% # of standard deviations for favorability

% Mean and Standard Deviation
% Sc
Scmean = mean(Screal);
SigSc = std(Screal);
% Shmin
S3mean = mean(S3real);
SigS3 = std(S3real);

% STRESS_GRID --> X,Y,Z,maximum Coulomb shear stress,sigma3
STRESS_GRID(:,1:3) = out.obs.pstress{1}(:,1:3);
STRESS_GRID(:,4) = Sc;
STRESS_GRID(:,5) = S3;

%% Permeability Model Layer Normalization

% Fault Layers
% no normalization needed for slip/dilation tendency (0-1)
Fault.sliptendency(:,1:3) = FAULT_GRID(:,1:3);
Fault.sliptendency(:,4) = FAULT_GRID(:,4);
Fault.dilationtendency(:,1:3) = FAULT_GRID(:,1:3);
Fault.dilationtendency(:,4) = FAULT_GRID(:,5);
% displacement gradient
Fault.displacement(:,1:3) = FAULT_GRID(:,1:3);
Fault.displacement(:,4) = FAULT_GRID(:,6)/max(FAULT_GRID(:,6));
% displacement gradient
Fault.displacementgradient(:,1:3) = FAULT_GRID(:,1:3);
Fault.displacementgradient(:,4) = FAULT_GRID(:,7)/max(FAULT_GRID(:,7));

% Stress Layers
% normalization between +/- X standard deviations
Stress.MaximumCoulombShearStress(:,1:3) = STRESS_GRID(:,1:3);
Stress.MaximumCoulombShearStress(:,4) = (STRESS_GRID(:,4)-(Scmean-nsd*SigSc))/(2*nsd*SigSc);
I = find(Stress.MaximumCoulombShearStress(:,4)<0);
Stress.MaximumCoulombShearStress(I,4) = 0;
I = find(Stress.MaximumCoulombShearStress(:,4)>1);
Stress.MaximumCoulombShearStress(I,4) = 1;
for i=1:length(L)
    Stress.MaximumCoulombShearStress(L(i,1),4) = 0;
end

% Inverse Shmin values by adding 2*mean to negative values
Stress.Sigma3(:,1:3) = STRESS_GRID(:,1:3);
Stress.Sigma3(:,4) = -STRESS_GRID(:,5);
Stress.Sigma3(:,4) = (Stress.Sigma3(:,4)-(-S3mean-nsd*SigS3))/(2*nsd*SigS3);
I = find(Stress.Sigma3(:,4)<0);
Stress.Sigma3(I,4) = 0;
I = find(Stress.Sigma3(:,4)>1);
Stress.Sigma3(I,4) = 1;
for i=1:length(L)
    Stress.Sigma3(L(i,1),4) = 0;
end

%% FIGURES

figure
plot([1:numel(ScReal)],ScReal,'k-');ylabel('Sc')
hold on
plot([0 4500],[Scmean+nsd*SigSc Scmean+nsd*SigSc])
plot([0 4500],[Scmean-nsd*SigSc Scmean-nsd*SigSc])
ylim([Scmean-nsd*SigSc Scmean+nsd*SigSc])
figure
plot([1:numel(S3Real)],S3Real,'k-');ylabel('S3')
hold on
plot([0 4500],[S3mean+nsd*SigS3 S3mean+nsd*SigS3])
plot([0 4500],[S3mean-nsd*SigS3 S3mean-nsd*SigS3])
ylim([S3mean-nsd*SigS3 S3mean+nsd*SigS3])

figure
plot([1:numel(Stress.MaximumCoulombShearStress(:,4))],Stress.MaximumCoulombShearStress(:,4),'k.');ylabel('Sc favorability')
figure
plot([1:numel(Stress.Sigma3(:,4))],Stress.Sigma3(:,4),'k.');ylabel('S3 favorability')


% grid data for contour plots
for j=0:61
for i=0:66
    X(1+i,1+j) = out.obs.pstress{1}((1+i)+67*j,1);
    Y(1+i,1+j) = out.obs.pstress{1}((1+i)+67*j,2);
    ScXY(1+i,1+j) = ((((-out.obs.pstress{1}((1+i)+67*j,15))-(-out.obs.pstress{1}((1+i)+67*j,7)))/2)*(1+.6^2)^.5 - .6*(((-out.obs.pstress{1}((1+i)+67*j,15))+(-out.obs.pstress{1}((1+i)+67*j,7)))/2))/1000000;
    S3XY(1+i,1+j) = (-out.obs.pstress{1}((1+i)+67*j,7))/1000000;
    if find(L(:,1)==((1+i)+67*j))
        ScXY(1+i,1+j) = NaN;
        S3XY(1+i,1+j) = NaN;
    end
end
end
% Sc contour intervals
Vm = [(Scmean-nsd*SigSc)/1000000:1:(Scmean+nsd*SigSc)/1000000];
figure
contourf(X,Y,ScXY,Vm)
hold on
contour(X,Y,ScXY,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sc(MPa)','Fontsize',14);
caxis([(Scmean-nsd*SigSc)/1000000 (Scmean+nsd*SigSc)/1000000]);
colormap(redblue)
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% S3 contour intervals
Vm = [(S3mean-nsd*SigS3)/1000000:1:(S3mean+nsd*SigS3)/1000000];
figure
contourf(X,Y,S3XY,Vm)
hold on
contour(X,Y,S3XY,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sigma3(MPa)','Fontsize',14);
caxis([(S3mean-nsd*SigS3)/1000000 (S3mean+nsd*SigS3)/1000000]);
colormap(redblue)
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)