% MSH_Sensitivity_200m.m
%
% AUTHOR:           Michael Swyer
% DATE CREATED:     9/4/2015
% DATE MODIFIED:    
% VERSION:          v.1
%
% PURPOSE:
%   Read Poly3D .out file and process data for permeability potential
%   sensitivity
%         
% PURPOSE OF MODIFICATION:
%
% INPUT: Poly3D output files
%           -MtStHelens200m_ul.out
%           -MtStHelens200m_ll.out
% 
% OUTPUT:
%   fault/stress model sensitivity outputs:
%           -FAULT_GRID: x, y, z, slip tendency, dilation tendency, displacement(m), displacement gradient(m/m)
%           -STRESS_GRID: x, y, z, maximum Coulomb shear stress(Pa), sigma3(Pa)
%   Permeability potential sensitivity layers:
%           -Fault.sliptendency: x, y, z, slip tendency
%           -Fault.dilationtendency: x, y, z, dilation tendency
%           -Fault.displacement: x, y, z, normalized displacement
%           -Fault.displacementgradient: x, y, z, normalized displacement gradient
%           -Stress.MaximumCoulombShearStress: x, y, z, normalized maximum Coulomb shear stress
%           -Stress.Sigma3: x, y, z, normalized sigma 3
%
% NOTE: 
%
% KEY functions written specifically for this modeling
%
% Other related functions
% - p3doutputreader_v05

clear all;close all

filename = 'InputOutputFiles\MtStHelens200m_ul.out';
out1 = p3doutputreader_v05(filename);
filename = 'InputOutputFiles\MtStHelens200m_ll.out';
out2 = p3doutputreader_v05(filename);
% Generates structured array including
%   FAULT OBJECTS
%   out.fault.xyz   = array of vertex locations
% 
%   out.fault.vert  = array of vertices
%                     [VERTNUM X1 X2 X3]
% 
%   out.fault.disp  = cell array contain displacements for each fault obj
%                     [ELT X1C X2C X3C B1 U1(+) U1(-) B2 U2(+) U2(-) B3 U3(+)]
%                     ELT = element number, for use in tri
%                     X1C, X2C, X3C = element centroid coordinates
%                     B1, B2, B3 are the burgers vectos components as
%                       (strike,dip,normal)
%                     Ui(+), Ui(-) = relative displacment at centroid
%
%   out.fault.tract = cell array contain tractions for each fault obj
%                     [ELT X1C X2C X3C T1 T2 T3]
%                     T1,T2,T3 are the traction components
%                       (strike,dip,normal)
%   out.fault.name  = cell array contain displacements for each fault obj
% 
%
%   OBSERVATION GRID
%   out.obs.xyz     = array containing the obs grid pts [X1,X2,X3]
%
%   out.obs.disp    = array containing displacements at obs pts
%                     [X1 X2 X3 U1 U2 U3]
%                     U1,U2,U3 = components of the displacement vector
%                     
%   out.obs.pstress = array containing principal stress and principal
%                     stress direcitons at obs pts
%                     [X1 X2 X3 N1 N2 N3 SIG1 N1 N2 N3 SIG2 N1 N2 N3 SIG3]
%                     SIG1,2,3 = principal stress components (tensoral
%                     convention) at each observation point
%                     N1,N2,N3 = principal stress direction vectors for the
%                       following principal stress in the array)
%
%   out.obs.stress  = array contain 
%                     [X1 X2 X3 SIG11 SIG22 SIG33 SIG12 SIG23 SIG13]
%                     SIG11,22,33,12,23,13 = stress tensor component at
%                     each observation point

Sigma200 = importdata('StressTensorMSH200.csv',',');
[V,PS200] = eig(Sigma200);% PS200 = eigenvalues as principal stresses

%% Slip/Dilation Tendency

%%% 200m %%%
% Goat Mountain Fault
% upper residual
nvecgm_ul = xlsread('MSH_GM_ul.xls','nvecgm_ul','A2:C500');
I = find(out1.fault.disp{1,9}(:,4)<-90&out1.fault.disp{1,9}(:,4)>-210);
% Tractions
for i=1:length(I)
    XYZ(i,:) = out1.fault.disp{1,9}(I(i),2:4);% XYZ points
    T(i,:) = Sigma200*nvecgm_ul(I(i),:)';% Tractions
    Tn(i,1) = -T(i,:)*nvecgm_ul(I(i),:)';% Normal Tractions
    Tmag(i,1) = (T(i,1)^2 + T(i,2)^2 + T(i,3)^2)^(1/2);% Traction Magnitudes
    DC(i,1) = Tn(i,1)/Tmag(i,1);% Direction Cosines
    theta(i,1) = acos(DC(i,1));% Traction angle from normal vector
    Ts(i,1) = Tmag(i,1)*sin(theta(i,1));% Shear Tractions
    STgm_ul(i,1) = Ts(i,1)/Tn(i,1);% Slip Tendencies
    DTgm_ul(i,1) = ((-PS200(1,1))-(Tn(i,1)))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    Dispgm_ul(i,1) = (out1.fault.disp{1,9}(I(i),5)^2 + out1.fault.disp{1,9}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZ(:,2));
XYZ = XYZ(I,:);
STgm_ul = STgm_ul(I,1);
DTgm_ul = DTgm_ul(I,1);
Dispgm_ul = Dispgm_ul(I,1);
% Displacement Gradients
for i=1:length(I)
    if i==1
        dg(i,1) = Dispgm_ul(i)/100;
    else
        dg(i,1) = abs(Dispgm_ul(i)-Dispgm_ul(i-1))/(((XYZ(i,1)-XYZ(i-1,1))^2 + (XYZ(i,2)-XYZ(i-1,2))^2 + (XYZ(i,3)-XYZ(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispgm_ul(i)/100;
for i=1:length(I)
    DGgm_ul(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
for i=1:length(I)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XYZ(i,1)-out1.obs.pstress{1}(j,1))^2 + (XYZ(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lgm_ul(:,1) = unique(J(:,1));
for i=1:length(Lgm_ul)
    I = find(J(:,1)==Lgm_ul(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZ(J(I(j),2),1)-out1.obs.pstress{1}(Lgm_ul(i),1))^2 + (XYZ(J(I(j),2),2)-out1.obs.pstress{1}(Lgm_ul(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lgm_ul(i,2) = J(I(K),2);
end
clear XYZ
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
clear dg
% lower residual
nvecgm_ll = xlsread('MSH_GM_ll.xls','nvecgm_ll','A2:C500');
I = find(out2.fault.disp{1,9}(:,4)<-90&out2.fault.disp{1,9}(:,4)>-210);
% Tractions
for i=1:length(I)
    XYZ(i,:) = out2.fault.disp{1,9}(I(i),2:4);% XYZ points
    T(i,:) = Sigma200*nvecgm_ll(I(i),:)';% Tractions
    Tn(i,1) = -T(i,:)*nvecgm_ll(I(i),:)';% Normal Tractions
    Tmag(i,1) = (T(i,1)^2 + T(i,2)^2 + T(i,3)^2)^(1/2);% Traction Magnitudes
    DC(i,1) = Tn(i,1)/Tmag(i,1);% Direction Cosines
    theta(i,1) = acos(DC(i,1));% Traction angle from normal vector
    Ts(i,1) = Tmag(i,1)*sin(theta(i,1));% Shear Tractions
    STgm_ll(i,1) = Ts(i,1)/Tn(i,1);% Slip Tendencies
    DTgm_ll(i,1) = ((-PS200(1,1))-(Tn(i,1)))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    Dispgm_ll(i,1) = (out2.fault.disp{1,9}(I(i),5)^2 + out2.fault.disp{1,9}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZ(:,2));
XYZ = XYZ(I,:);
STgm_ll = STgm_ll(I,1);
DTgm_ll = DTgm_ll(I,1);
Dispgm_ll = Dispgm_ll(I,1);
% Displacement Gradients
for i=1:length(I)
    if i==1
        dg(i,1) = Dispgm_ll(i)/100;
    else
        dg(i,1) = abs(Dispgm_ll(i)-Dispgm_ll(i-1))/(((XYZ(i,1)-XYZ(i-1,1))^2 + (XYZ(i,2)-XYZ(i-1,2))^2 + (XYZ(i,3)-XYZ(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispgm_ll(i)/100;
for i=1:length(I)
    DGgm_ll(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
for i=1:length(I)
    for j=1:length(out2.obs.pstress{1})
        Dobs(j) = ((XYZ(i,1)-out2.obs.pstress{1}(j,1))^2 + (XYZ(i,2)-out2.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lgm_ll(:,1) = unique(J(:,1));
for i=1:length(Lgm_ll)
    I = find(J(:,1)==Lgm_ll(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZ(J(I(j),2),1)-out2.obs.pstress{1}(Lgm_ll(i),1))^2 + (XYZ(J(I(j),2),2)-out2.obs.pstress{1}(Lgm_ll(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lgm_ll(i,2) = J(I(K),2);
end


% 24k Faults
data1 = importdata('24k_faults\F1.csv',',');
data2 = importdata('24k_faults\F2.csv',',');
data3 = importdata('24k_faults\F3.csv',',');
data4 = importdata('24k_faults\F4.csv',',');
data5 = importdata('24k_faults\F5.csv',',');
data6 = importdata('24k_faults\F6.csv',',');
data7 = importdata('24k_faults\F7.csv',',');
data8 = importdata('24k_faults\F8.csv',',');
data9 = importdata('24k_faults\F9.csv',',');
data10 = importdata('24k_faults\F10.csv',',');
data11 = importdata('24k_faults\F11.csv',',');


% Fault1
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data1.data)
    x1 = data1.data(i-1,1);x2 = data1.data(i,1);
    y1 = data1.data(i-1,2);y2 = data1.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = ((y2-y1)/l);
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST1_ul(i-1,1) = Ts/Tn;% Slip Tendencies
    DT1_ul(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY1(i-1,1) = x1;XY1(i-1,2) = y1;% XY points
    % lower residual w/ 20 degree dip
    nvec(1,1) = ((y2-y1)/l)*cos(20*(pi/180));
    nvec(2,1) = (-(x2-x1)/l)*cos(20*(pi/180));
    nvec(3,1) = ((((y2-y1)/l)^2+(-(x2-x1)/l)^2)^.5)*sin(20*(pi/180));
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST1_ll(i-1,1) = Ts/Tn;% Slip Tendencies
    DT1_ll(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
end
Disp1 = zeros(length(XY1),1);
DG1 = zeros(length(XY1),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY1)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XY1(i,1)-out1.obs.pstress{1}(j,1))^2 + (XY1(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L1(:,1) = unique(J(:,1));
for i=1:length(L1)
    I = find(J(:,1)==L1(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY1(J(I(j),2),1)-out1.obs.pstress{1}(L1(i),1))^2 + (XY1(J(I(j),2),2)-out1.obs.pstress{1}(L1(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L1(i,2) = J(I(K),2);
end


% Fault2
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data2.data)
    x1 = data2.data(i-1,1);x2 = data2.data(i,1);
    y1 = data2.data(i-1,2);y2 = data2.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST2_ul(i-1,1) = Ts/Tn;% Slip Tendencies
    DT2_ul(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY2(i-1,1) = x1;XY2(i-1,2) = y1;% XY points
    % lower residual w/ 20 degree dip
    nvec(1,1) = ((y2-y1)/l)*cos(20*(pi/180));
    nvec(2,1) = (-(x2-x1)/l)*cos(20*(pi/180));
    nvec(3,1) = ((((y2-y1)/l)^2+(-(x2-x1)/l)^2)^.5)*sin(20*(pi/180));
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST2_ll(i-1,1) = Ts/Tn;% Slip Tendencies
    DT2_ll(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
end
Disp2 = zeros(length(XY2),1);
DG2 = zeros(length(XY2),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY2)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XY2(i,1)-out1.obs.pstress{1}(j,1))^2 + (XY2(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L2(:,1) = unique(J(:,1));
for i=1:length(L2)
    I = find(J(:,1)==L2(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY2(J(I(j),2),1)-out1.obs.pstress{1}(L2(i),1))^2 + (XY2(J(I(j),2),2)-out1.obs.pstress{1}(L2(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L2(i,2) = J(I(K),2);
end


% Fault3
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data3.data)
    x1 = data3.data(i-1,1);x2 = data3.data(i,1);
    y1 = data3.data(i-1,2);y2 = data3.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST3_ul(i-1,1) = Ts/Tn;% Slip Tendencies
    DT3_ul(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY3(i-1,1) = x1;XY3(i-1,2) = y1;% XY points
    % lower residual w/ 20 degree dip
    nvec(1,1) = ((y2-y1)/l)*cos(20*(pi/180));
    nvec(2,1) = (-(x2-x1)/l)*cos(20*(pi/180));
    nvec(3,1) = ((((y2-y1)/l)^2+(-(x2-x1)/l)^2)^.5)*sin(20*(pi/180));
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST3_ll(i-1,1) = Ts/Tn;% Slip Tendencies
    DT3_ll(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
end
Disp3 = zeros(length(XY3),1);
DG3 = zeros(length(XY3),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY3)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XY3(i,1)-out1.obs.pstress{1}(j,1))^2 + (XY3(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L3(:,1) = unique(J(:,1));
for i=1:length(L3)
    I = find(J(:,1)==L3(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY3(J(I(j),2),1)-out1.obs.pstress{1}(L3(i),1))^2 + (XY3(J(I(j),2),2)-out1.obs.pstress{1}(L3(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L3(i,2) = J(I(K),2);
end


% Fault4
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data4.data)
    x1 = data4.data(i-1,1);x2 = data4.data(i,1);
    y1 = data4.data(i-1,2);y2 = data4.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn(i-1) = -T'*nvec;% Normal Tractions
    Tmag(i-1) = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC(i-1) = Tn(i-1)/Tmag(i-1);% Direction Cosines
    theta(i-1) = acos(DC(i-1));% Traction angle from normal vector
    Ts(i-1) = Tmag(i-1)*sin(theta(i-1));% Shear Tractions
    ST4_ul(i-1,1) = Ts(i-1)/Tn(i-1);% Slip Tendencies
    DT4_ul(i-1,1) = ((-PS200(1,1))-(Tn(i-1)))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY4(i-1,1) = x1;XY4(i-1,2) = y1;% XY points
    % lower residual w/ 20 degree dip
    nvec(1,1) = ((y2-y1)/l)*cos(20*(pi/180));
    nvec(2,1) = (-(x2-x1)/l)*cos(20*(pi/180));
    nvec(3,1) = ((((y2-y1)/l)^2+(-(x2-x1)/l)^2)^.5)*sin(20*(pi/180));
    T = Sigma200*nvec;% Tractions
    Tn(i-1) = -T'*nvec;% Normal Tractions
    Tmag(i-1) = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC(i-1) = Tn(i-1)/Tmag(i-1);% Direction Cosines
    theta(i-1) = acos(DC(i-1));% Traction angle from normal vector
    Ts(i-1) = Tmag(i-1)*sin(theta(i-1));% Shear Tractions
    ST4_ll(i-1,1) = Ts(i-1)/Tn(i-1);% Slip Tendencies
    DT4_ll(i-1,1) = ((-PS200(1,1))-(Tn(i-1)))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
end
Disp4 = zeros(length(XY4),1);
DG4 = zeros(length(XY4),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY4)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XY4(i,1)-out1.obs.pstress{1}(j,1))^2 + (XY4(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L4(:,1) = unique(J(:,1));
for i=1:length(L4)
    I = find(J(:,1)==L4(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY4(J(I(j),2),1)-out1.obs.pstress{1}(L4(i),1))^2 + (XY4(J(I(j),2),2)-out1.obs.pstress{1}(L4(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L4(i,2) = J(I(K),2);
end


% Fault5
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data5.data)
    x1 = data5.data(i-1,1);x2 = data5.data(i,1);
    y1 = data5.data(i-1,2);y2 = data5.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST5_ul(i-1,1) = Ts/Tn;% Slip Tendencies
    DT5_ul(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY5(i-1,1) = x1;XY5(i-1,2) = y1;% XY points
    % lower residual w/ 20 degree dip
    nvec(1,1) = ((y2-y1)/l)*cos(20*(pi/180));
    nvec(2,1) = (-(x2-x1)/l)*cos(20*(pi/180));
    nvec(3,1) = ((((y2-y1)/l)^2+(-(x2-x1)/l)^2)^.5)*sin(20*(pi/180));
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST5_ll(i-1,1) = Ts/Tn;% Slip Tendencies
    DT5_ll(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
end
Disp5 = zeros(length(XY5),1);
DG5 = zeros(length(XY5),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY5)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XY5(i,1)-out1.obs.pstress{1}(j,1))^2 + (XY5(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L5(:,1) = unique(J(:,1));
for i=1:length(L5)
    I = find(J(:,1)==L5(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY5(J(I(j),2),1)-out1.obs.pstress{1}(L5(i),1))^2 + (XY5(J(I(j),2),2)-out1.obs.pstress{1}(L5(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L5(i,2) = J(I(K),2);
end


% Fault6
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data6.data)
    x1 = data6.data(i-1,1);x2 = data6.data(i,1);
    y1 = data6.data(i-1,2);y2 = data6.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST6_ul(i-1,1) = Ts/Tn;% Slip Tendencies
    DT6_ul(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY6(i-1,1) = x1;XY6(i-1,2) = y1;% XY points
    % lower residual w/ 20 degree dip
    nvec(1,1) = ((y2-y1)/l)*cos(20*(pi/180));
    nvec(2,1) = (-(x2-x1)/l)*cos(20*(pi/180));
    nvec(3,1) = ((((y2-y1)/l)^2+(-(x2-x1)/l)^2)^.5)*sin(20*(pi/180));
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST6_ll(i-1,1) = Ts/Tn;% Slip Tendencies
    DT6_ll(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
end
Disp6 = zeros(length(XY6),1);
DG6 = zeros(length(XY6),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY6)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XY6(i,1)-out1.obs.pstress{1}(j,1))^2 + (XY6(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L6(:,1) = unique(J(:,1));
for i=1:length(L6)
    I = find(J(:,1)==L6(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY6(J(I(j),2),1)-out1.obs.pstress{1}(L6(i),1))^2 + (XY6(J(I(j),2),2)-out1.obs.pstress{1}(L6(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L6(i,2) = J(I(K),2);
end


% Fault7
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data7.data)
    x1 = data7.data(i-1,1);x2 = data7.data(i,1);
    y1 = data7.data(i-1,2);y2 = data7.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST7_ul(i-1,1) = Ts/Tn;% Slip Tendencies
    DT7_ul(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY7(i-1,1) = x1;XY7(i-1,2) = y1;% XY points
    % lower residual w/ 20 degree dip
    nvec(1,1) = ((y2-y1)/l)*cos(20*(pi/180));
    nvec(2,1) = (-(x2-x1)/l)*cos(20*(pi/180));
    nvec(3,1) = ((((y2-y1)/l)^2+(-(x2-x1)/l)^2)^.5)*sin(20*(pi/180));
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST7_ll(i-1,1) = Ts/Tn;% Slip Tendencies
    DT7_ll(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
end
Disp7 = zeros(length(XY7),1);
DG7 = zeros(length(XY7),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY7)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XY7(i,1)-out1.obs.pstress{1}(j,1))^2 + (XY7(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L7(:,1) = unique(J(:,1));
for i=1:length(L7)
    I = find(J(:,1)==L7(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY7(J(I(j),2),1)-out1.obs.pstress{1}(L7(i),1))^2 + (XY7(J(I(j),2),2)-out1.obs.pstress{1}(L7(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L7(i,2) = J(I(K),2);
end


% Fault8
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data8.data)
    x1 = data8.data(i-1,1);x2 = data8.data(i,1);
    y1 = data8.data(i-1,2);y2 = data8.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST8_ul(i-1,1) = Ts/Tn;% Slip Tendencies
    DT8_ul(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY8(i-1,1) = x1;XY8(i-1,2) = y1;% XY points
    % lower residual w/ 20 degree dip
    nvec(1,1) = ((y2-y1)/l)*cos(20*(pi/180));
    nvec(2,1) = (-(x2-x1)/l)*cos(20*(pi/180));
    nvec(3,1) = ((((y2-y1)/l)^2+(-(x2-x1)/l)^2)^.5)*sin(20*(pi/180));
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST8_ll(i-1,1) = Ts/Tn;% Slip Tendencies
    DT8_ll(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
end
Disp8 = zeros(length(XY8),1);
DG8 = zeros(length(XY8),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY8)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XY8(i,1)-out1.obs.pstress{1}(j,1))^2 + (XY8(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L8(:,1) = unique(J(:,1));
for i=1:length(L8)
    I = find(J(:,1)==L8(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY8(J(I(j),2),1)-out1.obs.pstress{1}(L8(i),1))^2 + (XY8(J(I(j),2),2)-out1.obs.pstress{1}(L8(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L8(i,2) = J(I(K),2);
end


% Fault9
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data9.data)
    x1 = data9.data(i-1,1);x2 = data9.data(i,1);
    y1 = data9.data(i-1,2);y2 = data9.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST9_ul(i-1,1) = Ts/Tn;% Slip Tendencies
    DT9_ul(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY9(i-1,1) = x1;XY9(i-1,2) = y1;% XY points
    % lower residual w/ 20 degree dip
    nvec(1,1) = ((y2-y1)/l)*cos(20*(pi/180));
    nvec(2,1) = (-(x2-x1)/l)*cos(20*(pi/180));
    nvec(3,1) = ((((y2-y1)/l)^2+(-(x2-x1)/l)^2)^.5)*sin(20*(pi/180));
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST9_ll(i-1,1) = Ts/Tn;% Slip Tendencies
    DT9_ll(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
end
Disp9 = zeros(length(XY9),1);
DG9 = zeros(length(XY9),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY9(1))
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XY9(i,1)-out1.obs.pstress{1}(j,1))^2 + (XY9(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L9(:,1) = unique(J(:,1));
for i=1:length(L9)
    I = find(J(:,1)==L9(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY9(J(I(j),2),1)-out1.obs.pstress{1}(L9(i),1))^2 + (XY9(J(I(j),2),2)-out1.obs.pstress{1}(L9(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L9(i,2) = J(I(K),2);
end


% Fault10
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data10.data)
    x1 = data10.data(i-1,1);x2 = data10.data(i,1);
    y1 = data10.data(i-1,2);y2 = data10.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST10_ul(i-1,1) = Ts/Tn;% Slip Tendencies
    DT10_ul(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY10(i-1,1) = x1;XY10(i-1,2) = y1;% XY points
    % lower residual w/ 20 degree dip
    nvec(1,1) = ((y2-y1)/l)*cos(20*(pi/180));
    nvec(2,1) = (-(x2-x1)/l)*cos(20*(pi/180));
    nvec(3,1) = ((((y2-y1)/l)^2+(-(x2-x1)/l)^2)^.5)*sin(20*(pi/180));
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST10_ll(i-1,1) = Ts/Tn;% Slip Tendencies
    DT10_ll(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
end
Disp10 = zeros(length(XY10),1);
DG10 = zeros(length(XY10),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY10)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XY10(i,1)-out1.obs.pstress{1}(j,1))^2 + (XY10(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L10(:,1) = unique(J(:,1));
for i=1:length(L10)
    I = find(J(:,1)==L10(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY10(J(I(j),2),1)-out1.obs.pstress{1}(L10(i),1))^2 + (XY10(J(I(j),2),2)-out1.obs.pstress{1}(L10(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L10(i,2) = J(I(K),2);
end


% Fault11
clear nvec
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% Normal Vectors --> Slip/Dilation Tendencies
for i=2:length(data11.data)
    x1 = data11.data(i-1,1);x2 = data11.data(i,1);
    y1 = data11.data(i-1,2);y2 = data11.data(i,2);
    l = ((x2-x1)^2+(y2-y1)^2)^.5;
    nvec(1,1) = (y2-y1)/l;
    nvec(2,1) = -(x2-x1)/l;
    nvec(3,1) = 0;
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST11_ul(i-1,1) = Ts/Tn;% Slip Tendencies
    DT11_ul(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
    XY11(i-1,1) = x1;XY11(i-1,2) = y1;% XY points
    % lower residual w/ 20 degree dip
    nvec(1,1) = ((y2-y1)/l)*cos(20*(pi/180));
    nvec(2,1) = (-(x2-x1)/l)*cos(20*(pi/180));
    nvec(3,1) = ((((y2-y1)/l)^2+(-(x2-x1)/l)^2)^.5)*sin(20*(pi/180));
    T = Sigma200*nvec;% Tractions
    Tn = -T'*nvec;% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    ST11_ll(i-1,1) = Ts/Tn;% Slip Tendencies
    DT11_ll(i-1,1) = ((-PS200(1,1))-(Tn))/((-PS200(1,1))-(-PS200(3,3)));% Dilation Tendencies
end
Disp11 = zeros(length(XY11),1);
DG11 = zeros(length(XY11),1);
% Find observation points closest to fault
clear Dobs;clear J
for i=1:length(XY11)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XY11(i,1)-out1.obs.pstress{1}(j,1))^2 + (XY11(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
L11(:,1) = unique(J(:,1));
for i=1:length(L11)
    I = find(J(:,1)==L11(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XY11(J(I(j),2),1)-out1.obs.pstress{1}(L11(i),1))^2 + (XY11(J(I(j),2),2)-out1.obs.pstress{1}(L11(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    L11(i,2) = J(I(K),2);
end

%% combine fault layers
L_ul = [Lgm_ul;L1;L2;L3;L4;L5;L6;L7;L8;L9;L10;L11];
STall_ul = [STgm_ul(Lgm_ul(:,2));ST1_ul(L1(:,2));ST2_ul(L2(:,2));ST3_ul(L3(:,2));ST4_ul(L4(:,2));ST5_ul(L5(:,2));ST6_ul(L6(:,2));ST7_ul(L7(:,2));ST8_ul(L8(:,2));ST9_ul(L9(:,2));ST10_ul(L10(:,2));ST11_ul(L11(:,2))];
DTall_ul = [DTgm_ul(Lgm_ul(:,2));DT1_ul(L1(:,2));DT2_ul(L2(:,2));DT3_ul(L3(:,2));DT4_ul(L4(:,2));DT5_ul(L5(:,2));DT6_ul(L6(:,2));DT7_ul(L7(:,2));DT8_ul(L8(:,2));DT9_ul(L9(:,2));DT10_ul(L10(:,2));DT11_ul(L11(:,2))];
Dispall_ul = [Dispgm_ul(Lgm_ul(:,2));Disp1(L1(:,2));Disp2(L2(:,2));Disp3(L3(:,2));Disp4(L4(:,2));Disp5(L5(:,2));Disp6(L6(:,2));Disp7(L7(:,2));Disp8(L8(:,2));Disp9(L9(:,2));Disp10(L10(:,2));Disp11(L11(:,2))];
DGall_ul = [DGgm_ul(Lgm_ul(:,2));DG1(L1(:,2));DG2(L2(:,2));DG3(L3(:,2));DG4(L4(:,2));DG5(L5(:,2));DG6(L6(:,2));DG7(L7(:,2));DG8(L8(:,2));DG9(L9(:,2));DG10(L10(:,2));DG11(L11(:,2))];

L_ll = [Lgm_ll;L1;L2;L3;L4;L5;L6;L7;L8;L9;L10;L11];
STall_ll = [STgm_ll(Lgm_ll(:,2));ST1_ll(L1(:,2));ST2_ll(L2(:,2));ST3_ll(L3(:,2));ST4_ll(L4(:,2));ST5_ll(L5(:,2));ST6_ll(L6(:,2));ST7_ll(L7(:,2));ST8_ll(L8(:,2));ST9_ll(L9(:,2));ST10_ll(L10(:,2));ST11_ll(L11(:,2))];
DTall_ll = [DTgm_ll(Lgm_ll(:,2));DT1_ll(L1(:,2));DT2_ll(L2(:,2));DT3_ll(L3(:,2));DT4_ll(L4(:,2));DT5_ll(L5(:,2));DT6_ll(L6(:,2));DT7_ll(L7(:,2));DT8_ll(L8(:,2));DT9_ll(L9(:,2));DT10_ll(L10(:,2));DT11_ll(L11(:,2))];
Dispall_ll = [Dispgm_ll(Lgm_ll(:,2));Disp1(L1(:,2));Disp2(L2(:,2));Disp3(L3(:,2));Disp4(L4(:,2));Disp5(L5(:,2));Disp6(L6(:,2));Disp7(L7(:,2));Disp8(L8(:,2));Disp9(L9(:,2));Disp10(L10(:,2));Disp11(L11(:,2))];
DGall_ll = [DGgm_ll(Lgm_ll(:,2));DG1(L1(:,2));DG2(L2(:,2));DG3(L3(:,2));DG4(L4(:,2));DG5(L5(:,2));DG6(L6(:,2));DG7(L7(:,2));DG8(L8(:,2));DG9(L9(:,2));DG10(L10(:,2));DG11(L11(:,2))];

% put fault layers in on map coordinates
k=1;
for i=1:length(out1.obs.pstress{1})
    if find(L_ul(:,1)==i)
        if STall_ul(k)>1
            ST_ul(i,1) = 1;
        elseif STall_ul(k)<0
            ST_ul(i,1) = 0;
        else
            ST_ul(i,1) = STall_ul(k);
        end
        DT_ul(i,1) = DTall_ul(k);
        Disp_ul(i,1) = Dispall_ul(k);
        DG_ul(i,1) = DGall_ul(k);
        k=k+1;
    else
        ST_ul(i,1) = 0;
        DT_ul(i,1) = 0;
        Disp_ul(i,1) = 0;
        DG_ul(i,1) = 0;
    end
end

k=1;
for i=1:length(out2.obs.pstress{1})
    if find(L_ll(:,1)==i)
        if STall_ll(k)>1
            ST_ll(i,1) = 1;
        elseif STall_ll(k)<0
            ST_ll(i,1) = 0;
        else
            ST_ll(i,1) = STall_ll(k);
        end
        DT_ll(i,1) = DTall_ll(k);
        Disp_ll(i,1) = Dispall_ll(k);
        DG_ll(i,1) = DGall_ll(k);
        k=k+1;
    else
        ST_ll(i,1) = 0;
        DT_ll(i,1) = 0;
        Disp_ll(i,1) = 0;
        DG_ll(i,1) = 0;
    end
end

for i=1:length(out1.obs.pstress{1})
    FAULT_GRID(i,1:3) = out1.obs.pstress{1}(i,1:3);
end

% FAULT_GRID --> X,Y,Z,slip tendency absolute difference,dilation tendency absolute difference,displacement absolute difference,displacement gradient absolute difference
FAULT_GRID(:,4) = abs(ST_ul-ST_ll);
FAULT_GRID(:,5) = abs(DT_ul-DT_ll);
FAULT_GRID(:,6) = abs(Disp_ul-Disp_ll);
FAULT_GRID(:,7) = abs(DG_ul-DG_ll);

%% Combine Stress layers

% calculate Sc and S3
for i=1:length(out1.obs.pstress{1})
    Sc_ul(i,1) = (((-out1.obs.pstress{1}(i,15))-(-out1.obs.pstress{1}(i,7)))/2)*(1+.6^2)^.5 - .6*(((-out1.obs.pstress{1}(i,15))+(-out1.obs.pstress{1}(i,7)))/2);
    S3_ul(i,1) = (-out1.obs.pstress{1}(i,7));
    if find(L_ul(:,1)==i)
        Sc_ul(i,1) = NaN;
        S3_ul(i,1) = NaN;
    end
end
for i=1:length(out2.obs.pstress{1})
    Sc_ll(i,1) = (((-out2.obs.pstress{1}(i,15))-(-out2.obs.pstress{1}(i,7)))/2)*(1+.6^2)^.5 - .6*(((-out2.obs.pstress{1}(i,15))+(-out1.obs.pstress{1}(i,7)))/2);
    S3_ll(i,1) = (-out2.obs.pstress{1}(i,7));
    if find(L_ll(:,1)==i)
        Sc_ll(i,1) = NaN;
        S3_ll(i,1) = NaN;
    end
end

% extract stress values to compute means and standard deviations
k=1;
for i=1:length(Sc_ul)
    if isnan(Sc_ul(i)-Sc_ll(i))
        Sc(i,1) = 0;
        S3(i,1) = 0;
    else
        Sc(i,1) = abs(Sc_ul(i)-Sc_ll(i));
        S3(i,1) = abs(S3_ul(i)-S3_ll(i));
        if abs(S3_ul(i)-S3_ll(i))>1e7
        else
        ScReal(k,1) = abs(Sc_ul(i)-Sc_ll(i));
        S3Real(k,1) = abs(S3_ul(i)-S3_ll(i));
        k=k+1;
        end
    end
end

% Mean and Standard Deviation
% Sc
Scmean = mean(ScReal);
SigSc = std(ScReal);
% Shmin
S3mean = mean(S3Real);
SigS3 = std(S3Real);

% STRESS_GRID --> X,Y,Z,maximum Coulomb shear stress absolute difference,sigma3 absolute difference
STRESS_GRID(:,1:3) = out1.obs.pstress{1}(:,1:3);
STRESS_GRID(:,4) = Sc;
STRESS_GRID(:,5) = S3;

%% Permeability Model Layer Normalization

% Fault Layers
Fault.sliptendency(:,1:3) = FAULT_GRID(:,1:3);
Fault.sliptendency(:,4) = FAULT_GRID(:,4)/max(FAULT_GRID(:,4));
Fault.dilationtendency(:,1:3) = FAULT_GRID(:,1:3);
Fault.dilationtendency(:,4) = FAULT_GRID(:,5)/max(FAULT_GRID(:,5));
% displacement
Fault.displacement(:,1:3) = FAULT_GRID(:,1:3);
Fault.displacement(:,4) = FAULT_GRID(:,6)/max(FAULT_GRID(:,6));
% displacement gradient
Fault.displacementgradient(:,1:3) = FAULT_GRID(:,1:3);
Fault.displacementgradient(:,4) = FAULT_GRID(:,7)/max(FAULT_GRID(:,7));

% Stress Layers
% normalization between +/- X standard deviations
sdc = 5;% # of standard deviations
Stress.MaximumCoulombShearStress(:,1:3) = STRESS_GRID(:,1:3);
Stress.MaximumCoulombShearStress(:,4) = STRESS_GRID(:,4)/(2*sdc*SigSc);
I = find(Stress.MaximumCoulombShearStress(:,4)<0);
Stress.MaximumCoulombShearStress(I,4) = 0;
I = find(Stress.MaximumCoulombShearStress(:,4)>1);
Stress.MaximumCoulombShearStress(I,4) = 1;
for i=1:length(L_ul)
    Stress.MaximumCoulombShearStress(L_ul(i,1),4) = 0;
end
for i=1:length(L_ll)
    Stress.MaximumCoulombShearStress(L_ll(i,1),4) = 0;
end

% Inverse Shmin values by adding 2*mean to negative values
sdh = sdc;% # of standard deviations
Stress.Sigma3(:,1:3) = STRESS_GRID(:,1:3);
Stress.Sigma3(:,4) = STRESS_GRID(:,5);
Stress.Sigma3(:,4) = Stress.Sigma3(:,4)/(2*sdh*SigS3);
I = find(Stress.Sigma3(:,4)<0);
Stress.Sigma3(I,4) = 0;
I = find(Stress.Sigma3(:,4)>1);
Stress.Sigma3(I,4) = 1;
for i=1:length(L_ul)
    Stress.Sigma3(L_ul(i,1),4) = 0;
end
for i=1:length(L_ll)
    Stress.Sigma3(L_ll(i,1),4) = 0;
end
%% FIGURES

figure
plot([1:numel(Sc)],Sc,'k-');ylabel('Sc')
hold on
plot([0 4500],[Scmean+2*sdc*SigSc Scmean+2*sdc*SigSc])
plot([0 4500],[0 0])
ylim([0 Scmean+2*sdc*SigSc])
figure
plot([1:numel(S3)],S3,'k-');ylabel('S3')
hold on
plot([0 4500],[S3mean+2*sdh*SigS3 S3mean+2*sdh*SigS3])
plot([0 4500],[0 0])
ylim([0 S3mean+2*sdh*SigS3])

figure
plot([1:numel(Stress.MaximumCoulombShearStress(:,4))],Stress.MaximumCoulombShearStress(:,4),'k.');ylabel('Sc sensitivity')
figure
plot([1:numel(Stress.Sigma3(:,4))],Stress.Sigma3(:,4),'k.');ylabel('S3 sensitivity')


% grid data for contour plots
for j=0:61
for i=0:66
    X(1+i,1+j) = out1.obs.pstress{1}((1+i)+67*j,1);
    Y(1+i,1+j) = out1.obs.pstress{1}((1+i)+67*j,2);
    STXY(1+i,1+j) = Fault.sliptendency((1+i)+67*j,4);
    DTXY(1+i,1+j) = Fault.dilationtendency((1+i)+67*j,4);
    DispXY(1+i,1+j) = Fault.displacement((1+i)+67*j,4);
    DGXY(1+i,1+j) = Fault.displacementgradient((1+i)+67*j,4);
    ScXY(1+i,1+j) = Stress.MaximumCoulombShearStress((1+i)+67*j,4);
    ShminXY(1+i,1+j) = Stress.Sigma3((1+i)+67*j,4);
    if find(L_ul(:,1)==((1+i)+67*j))
        ScXY(1+i,1+j) = NaN;
        ShminXY(1+i,1+j) = NaN;
    elseif find(L_ll(:,1)==((1+i)+67*j))
        ScXY(1+i,1+j) = NaN;
        ShminXY(1+i,1+j) = NaN;
    end
end
end

% colormap white-black
for i=1:50
    unccm(i,:) = [(1-i/50) (1-i/50) (1-i/50)];
end

% contour intervals
Vm = [0:.01:1];

% Sc sensitivity map
figure
contourf(X,Y,ScXY,Vm)
hold on
contour(X,Y,ScXY,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sc sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% S3 sensitivity map
figure
contourf(X,Y,ShminXY,Vm)
hold on
contour(X,Y,ShminXY,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sigma3 sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% ST sensitivity map
figure
contourf(X,Y,STXY)
hold on
contour(X,Y,STXY)
c=colorbar;
set(get(c,'ylabel'),'String','Slip Tendency sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% DT sensitivity map
figure
contourf(X,Y,DTXY)
hold on
contour(X,Y,DTXY)
c=colorbar;
set(get(c,'ylabel'),'String','Dilation Tendency sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% Disp sensitivity map
figure
contourf(X,Y,DispXY)
hold on
contour(X,Y,DispXY)
c=colorbar;
set(get(c,'ylabel'),'String','Displacement sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% DG sensitivity map
figure
contourf(X,Y,DGXY)
hold on
contour(X,Y,DGXY)
c=colorbar;
set(get(c,'ylabel'),'String','Displacement Gradient sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)