% MSH_Sensitivity_3km.m
%
% AUTHOR:           Michael Swyer
% DATE CREATED:     9/4/2015
% DATE MODIFIED:    
% VERSION:          v.1
%
% PURPOSE:
%   Read Poly3D .out file and process data for permeability potential
%   sensitivity
%         
% PURPOSE OF MODIFICATION:
%
% INPUT: Poly3D output files
%           -MtStHelens3km_ul.out
%           -MtStHelens3km_ll.out
% 
% OUTPUT:
%   fault/stress model sensitivity outputs:
%           -FAULT_GRID: x, y, z, slip tendency, dilation tendency, displacement(m), displacement gradient(m/m)
%           -STRESS_GRID: x, y, z, maximum Coulomb shear stress(Pa), sigma3(Pa)
%   Permeability potential sensitivity layers:
%           -Fault.sliptendency: x, y, z, slip tendency
%           -Fault.dilationtendency: x, y, z, dilation tendency
%           -Fault.displacement: x, y, z, normalized displacement
%           -Fault.displacementgradient: x, y, z, normalized displacement gradient
%           -Stress.MaximumCoulombShearStress: x, y, z, normalized maximum Coulomb shear stress
%           -Stress.Sigma3: x, y, z, normalized sigma 3
%
% NOTE: 
%
% KEY functions written specifically for this modeling
%
% Other related functions
% - p3doutputreader_v05

clear all;close all

filename = 'InputOutputFiles\MtStHelens3km_ul.out';
out1 = p3doutputreader_v05(filename);
filename = 'InputOutputFiles\MtStHelens3km_ll.out';
out2 = p3doutputreader_v05(filename);
% Generates structured array including
%   FAULT OBJECTS
%   out.fault.xyz   = array of vertex locations
% 
%   out.fault.vert  = array of vertices
%                     [VERTNUM X1 X2 X3]
% 
%   out.fault.disp  = cell array contain displacements for each fault obj
%                     [ELT X1C X2C X3C B1 U1(+) U1(-) B2 U2(+) U2(-) B3 U3(+)]
%                     ELT = element number, for use in tri
%                     X1C, X2C, X3C = element centroid coordinates
%                     B1, B2, B3 are the burgers vectos components as
%                       (strike,dip,normal)
%                     Ui(+), Ui(-) = relative displacment at centroid
%
%   out.fault.tract = cell array contain tractions for each fault obj
%                     [ELT X1C X2C X3C T1 T2 T3]
%                     T1,T2,T3 are the traction components
%                       (strike,dip,normal)
%   out.fault.name  = cell array contain displacements for each fault obj
% 
%
%   OBSERVATION GRID
%   out.obs.xyz     = array containing the obs grid pts [X1,X2,X3]
%
%   out.obs.disp    = array containing displacements at obs pts
%                     [X1 X2 X3 U1 U2 U3]
%                     U1,U2,U3 = components of the displacement vector
%                     
%   out.obs.pstress = array containing principal stress and principal
%                     stress direcitons at obs pts
%                     [X1 X2 X3 N1 N2 N3 SIG1 N1 N2 N3 SIG2 N1 N2 N3 SIG3]
%                     SIG1,2,3 = principal stress components (tensoral
%                     convention) at each observation point
%                     N1,N2,N3 = principal stress direction vectors for the
%                       following principal stress in the array)
%
%   out.obs.stress  = array contain 
%                     [X1 X2 X3 SIG11 SIG22 SIG33 SIG12 SIG23 SIG13]
%                     SIG11,22,33,12,23,13 = stress tensor component at
%                     each observation point

Sigma3km = importdata('StressTensorMSH3km.csv',',');
[V,PS3km] = eig(Sigma3km);% PS3km = eigenvalues as principal stresses

%% Slip/Dilation Tendency

%%% 3000m %%%

% Goat Mountain Fault
% upper residuals
nvecgm_ul = xlsread('MSH_GM_ul.xls','nvecgm_ul','A2:C500');
I = find(out1.fault.disp{1,9}(:,4)<-2600&out1.fault.disp{1,9}(:,4)>-3000);
% Tractions
for i=1:length(I)
    XYZgm_ul(i,:) = out1.fault.disp{1,9}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecgm_ul(I(i),:)';% Tractions
    Tn = -T'*nvecgm_ul(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STgm_ul(i,1) = Ts/Tn;% Slip Tendencies
    DTgm_ul(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Dispgm_ul(i,1) = (out1.fault.disp{1,9}(I(i),5)^2 + out1.fault.disp{1,9}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZgm_ul(:,2));
XYZgm_ul = XYZgm_ul(I,:);
STgm_ul = STgm_ul(I,1);
DTgm_ul = DTgm_ul(I,1);
Dispgm_ul = Dispgm_ul(I,1);
% Displacement Gradients
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Dispgm_ul(i)/100;
    else
        dg(i,1) = abs(Dispgm_ul(i)-Dispgm_ul(i-1))/(((XYZgm_ul(i,1)-XYZgm_ul(i-1,1))^2 + (XYZgm_ul(i,2)-XYZgm_ul(i-1,2))^2 + (XYZgm_ul(i,3)-XYZgm_ul(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispgm_ul(i)/100;
for i=1:length(I)
    DGgm_ul(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XYZgm_ul(i,1)-out1.obs.pstress{1}(j,1))^2 + (XYZgm_ul(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lgm_ul(:,1) = unique(J(:,1));
for i=1:length(Lgm_ul)
    I = find(J(:,1)==Lgm_ul(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZgm_ul(J(I(j),2),1)-out1.obs.pstress{1}(Lgm_ul(i),1))^2 + (XYZgm_ul(J(I(j),2),2)-out1.obs.pstress{1}(Lgm_ul(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lgm_ul(i,2) = J(I(K),2);
end
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% lower residuals
nvecgm_ll = xlsread('MSH_GM_ll.xls','nvecgm_ll','A2:C500');
I = find(out2.fault.disp{1,9}(:,4)<-2600&out2.fault.disp{1,9}(:,4)>-3000);
% Tractions
for i=1:length(I)
    XYZgm_ll(i,:) = out2.fault.disp{1,9}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecgm_ll(I(i),:)';% Tractions
    Tn = -T'*nvecgm_ll(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STgm_ll(i,1) = Ts/Tn;% Slip Tendencies
    DTgm_ll(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Dispgm_ll(i,1) = (out2.fault.disp{1,9}(I(i),5)^2 + out2.fault.disp{1,9}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZgm_ll(:,2));
XYZgm_ll = XYZgm_ll(I,:);
STgm_ll = STgm_ll(I,1);
DTgm_ll = DTgm_ll(I,1);
Dispgm_ll = Dispgm_ll(I,1);
% Displacement Gradients
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Dispgm_ll(i)/100;
    else
        dg(i,1) = abs(Dispgm_ll(i)-Dispgm_ll(i-1))/(((XYZgm_ll(i,1)-XYZgm_ll(i-1,1))^2 + (XYZgm_ll(i,2)-XYZgm_ll(i-1,2))^2 + (XYZgm_ll(i,3)-XYZgm_ll(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispgm_ll(i)/100;
for i=1:length(I)
    DGgm_ll(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out2.obs.pstress{1})
        Dobs(j) = ((XYZgm_ll(i,1)-out2.obs.pstress{1}(j,1))^2 + (XYZgm_ll(i,2)-out2.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lgm_ll(:,1) = unique(J(:,1));
for i=1:length(Lgm_ll)
    I = find(J(:,1)==Lgm_ll(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZgm_ll(J(I(j),2),1)-out2.obs.pstress{1}(Lgm_ll(i),1))^2 + (XYZgm_ll(J(I(j),2),2)-out2.obs.pstress{1}(Lgm_ll(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lgm_ll(i,2) = J(I(K),2);
end

clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% North Fault
% upper residuals
nvecc_ul = xlsread('MSH_NorthFault_ul.xls','nvecc_ul','A2:C5000');
I = find(out1.fault.disp{1,1}(:,4)<-2800&out1.fault.disp{1,1}(:,4)>-3100);
% Tractions
for i=1:length(I)
    XYZc_ul(i,:) = out1.fault.disp{1,1}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecc_ul(I(i),:)';% Tractions
    Tn = -T'*nvecc_ul(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STc_ul(i,1) = Ts/Tn;% Slip Tendencies
    DTc_ul(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Dispc_ul(i,1) = (out1.fault.disp{1,1}(I(i),5)^2 + out1.fault.disp{1,1}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZc_ul(:,2));
XYZc_ul = XYZc_ul(I,:);
STc_ul = STc_ul(I,1);
DTc_ul = DTc_ul(I,1);
Dispc_ul = Dispc_ul(I,1);
% Displacement Gradientws
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Dispc_ul(i)/100;
    else
        dg(i,1) = abs(Dispc_ul(i)-Dispc_ul(i-1))/(((XYZc_ul(i,1)-XYZc_ul(i-1,1))^2 + (XYZc_ul(i,2)-XYZc_ul(i-1,2))^2 + (XYZc_ul(i,3)-XYZc_ul(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispc_ul(i)/100;
for i=1:length(I)
    DGc_ul(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XYZc_ul(i,1)-out1.obs.pstress{1}(j,1))^2 + (XYZc_ul(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lc_ul(:,1) = unique(J(:,1));
for i=1:length(Lc_ul)
    I = find(J(:,1)==Lc_ul(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZc_ul(J(I(j),2),1)-out1.obs.pstress{1}(Lc_ul(i),1))^2 + (XYZc_ul(J(I(j),2),2)-out1.obs.pstress{1}(Lc_ul(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lc_ul(i,2) = J(I(K),2);
end
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% lower residuals
nvecc_ll = xlsread('MSH_NorthFault_ll.xls','nvecc_ll','A2:C5000');
I = find(out2.fault.disp{1,1}(:,4)<-2800&out2.fault.disp{1,1}(:,4)>-3100);
% Tractions
for i=1:length(I)
    XYZc_ll(i,:) = out2.fault.disp{1,1}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecc_ll(I(i),:)';% Tractions
    Tn = -T'*nvecc_ll(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STc_ll(i,1) = Ts/Tn;% Slip Tendencies
    DTc_ll(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Dispc_ll(i,1) = (out2.fault.disp{1,1}(I(i),5)^2 + out2.fault.disp{1,1}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZc_ll(:,2));
XYZc_ll = XYZc_ll(I,:);
STc_ll = STc_ll(I,1);
DTc_ll = DTc_ll(I,1);
Dispc_ll = Dispc_ll(I,1);
% Displacement Gradientws
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Dispc_ll(i)/100;
    else
        dg(i,1) = abs(Dispc_ll(i)-Dispc_ll(i-1))/(((XYZc_ll(i,1)-XYZc_ll(i-1,1))^2 + (XYZc_ll(i,2)-XYZc_ll(i-1,2))^2 + (XYZc_ll(i,3)-XYZc_ll(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispc_ll(i)/100;
for i=1:length(I)
    DGc_ll(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out2.obs.pstress{1})
        Dobs(j) = ((XYZc_ll(i,1)-out2.obs.pstress{1}(j,1))^2 + (XYZc_ll(i,2)-out2.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lc_ll(:,1) = unique(J(:,1));
for i=1:length(Lc_ll)
    I = find(J(:,1)==Lc_ll(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZc_ll(J(I(j),2),1)-out2.obs.pstress{1}(Lc_ll(i),1))^2 + (XYZc_ll(J(I(j),2),2)-out2.obs.pstress{1}(Lc_ll(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lc_ll(i,2) = J(I(K),2);
end

clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% North Fault Shallow
% upper residuals
nvecs_ul = xlsread('MSH_NorthFault_ul.xls','nvecs_ul','A2:C500');
I = find(out1.fault.disp{1,2}(:,4)<-2800&out1.fault.disp{1,2}(:,4)>-3100);
% Tractions
for i=1:length(I)
    XYZs_ul(i,:) = out1.fault.disp{1,2}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecs_ul(I(i),:)';% Tractions
    Tn = -T'*nvecs_ul(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STs_ul(i,1) = Ts/Tn;% Slip Tendencies
    DTs_ul(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Disps_ul(i,1) = (out1.fault.disp{1,2}(I(i),5)^2 + out1.fault.disp{1,2}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZs_ul(:,2));
XYZs_ul = XYZs_ul(I,:);
STs_ul = STs_ul(I,1);
DTs_ul = DTs_ul(I,1);
Disps_ul = Disps_ul(I,1);
% Displacement Gradientws
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Disps_ul(i)/100;
    else
        dg(i,1) = abs(Disps_ul(i)-Disps_ul(i-1))/(((XYZs_ul(i,1)-XYZs_ul(i-1,1))^2 + (XYZs_ul(i,2)-XYZs_ul(i-1,2))^2 + (XYZs_ul(i,3)-XYZs_ul(i-1,3))^2)^.5);
    end
end
dg(i+1) = Disps_ul(i)/100;
for i=1:length(I)
    DGs_ul(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XYZs_ul(i,1)-out1.obs.pstress{1}(j,1))^2 + (XYZs_ul(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Ls_ul(:,1) = unique(J(:,1));
for i=1:length(Ls_ul)
    I = find(J(:,1)==Ls_ul(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZs_ul(J(I(j),2),1)-out1.obs.pstress{1}(Ls_ul(i),2))^2 + (XYZs_ul(J(I(j),2),2)-out1.obs.pstress{1}(Ls_ul(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Ls_ul(i,2) = J(I(K),2);
end
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% lower residuals
nvecs_ll = xlsread('MSH_NorthFault_ll.xls','nvecs_ll','A2:C500');
I = find(out2.fault.disp{1,2}(:,4)<-2800&out2.fault.disp{1,2}(:,4)>-3100);
% Tractions
for i=1:length(I)
    XYZs_ll(i,:) = out2.fault.disp{1,2}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecs_ll(I(i),:)';% Tractions
    Tn = -T'*nvecs_ll(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STs_ll(i,1) = Ts/Tn;% Slip Tendencies
    DTs_ll(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Disps_ll(i,1) = (out2.fault.disp{1,2}(I(i),5)^2 + out2.fault.disp{1,2}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZs_ll(:,2));
XYZs_ll = XYZs_ll(I,:);
STs_ll = STs_ll(I,1);
DTs_ll = DTs_ll(I,1);
Disps_ll = Disps_ll(I,1);
% Displacement Gradientws
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Disps_ll(i)/100;
    else
        dg(i,1) = abs(Disps_ll(i)-Disps_ll(i-1))/(((XYZs_ll(i,1)-XYZs_ll(i-1,1))^2 + (XYZs_ll(i,2)-XYZs_ll(i-1,2))^2 + (XYZs_ll(i,3)-XYZs_ll(i-1,3))^2)^.5);
    end
end
dg(i+1) = Disps_ll(i)/100;
for i=1:length(I)
    DGs_ll(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out2.obs.pstress{1})
        Dobs(j) = ((XYZs_ll(i,1)-out2.obs.pstress{1}(j,1))^2 + (XYZs_ll(i,2)-out2.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Ls_ll(:,1) = unique(J(:,1));
for i=1:length(Ls_ll)
    I = find(J(:,1)==Ls_ll(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZs_ll(J(I(j),2),1)-out2.obs.pstress{1}(Ls_ll(i),2))^2 + (XYZs_ll(J(I(j),2),2)-out2.obs.pstress{1}(Ls_ll(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Ls_ll(i,2) = J(I(K),2);
end

clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% South Fault
% upper residuals
nvecsf_ul = xlsread('MSH_SouthFault_ul.xls','nvecsf_ul','A2:C5000');
I = find(out1.fault.disp{1,6}(:,4)<-3100&out1.fault.disp{1,6}(:,4)>-3400);
% Tractions
for i=1:length(I)
    XYZsf_ul(i,:) = out1.fault.disp{1,6}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecsf_ul(I(i),:)';% Tractions
    Tn = -T'*nvecsf_ul(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STsf_ul(i,1) = Ts/Tn;% Slip Tendencies
    DTsf_ul(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Dispsf_ul(i,1) = (out1.fault.disp{1,6}(I(i),5)^2 + out1.fault.disp{1,6}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZsf_ul(:,2));
XYZsf_ul = XYZsf_ul(I,:);
STsf_ul = STsf_ul(I,1);
DTsf_ul = DTsf_ul(I,1);
Dispsf_ul = Dispsf_ul(I,1);
% Displacement Gradientws
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Dispsf_ul(i)/100;
    else
        dg(i,1) = abs(Dispsf_ul(i)-Dispsf_ul(i-1))/(((XYZsf_ul(i,1)-XYZsf_ul(i-1,1))^2 + (XYZsf_ul(i,2)-XYZsf_ul(i-1,2))^2 + (XYZsf_ul(i,3)-XYZsf_ul(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispsf_ul(i)/100;
for i=1:length(I)
    DGsf_ul(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out1.obs.pstress{1})
        Dobs(j) = ((XYZsf_ul(i,1)-out1.obs.pstress{1}(j,1))^2 + (XYZsf_ul(i,2)-out1.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lsf_ul(:,1) = unique(J(:,1));
for i=1:length(Lsf_ul)
    I = find(J(:,1)==Lsf_ul(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZsf_ul(J(I(j),2),1)-out1.obs.pstress{1}(Lsf_ul(i),2))^2 + (XYZsf_ul(J(I(j),2),2)-out1.obs.pstress{1}(Lsf_ul(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lsf_ul(i,2) = J(I(K),2);
end
clear T
clear Tn
clear Tmag
clear DC
clear theta
clear Ts
% lower residuals
nvecsf_ll = xlsread('MSH_SouthFault_ll.xls','nvecsf_ll','A2:C5000');
I = find(out2.fault.disp{1,6}(:,4)<-3100&out2.fault.disp{1,6}(:,4)>-3400);
% Tractions
for i=1:length(I)
    XYZsf_ll(i,:) = out2.fault.disp{1,6}(I(i),2:4);% XYZ points
    T = Sigma3km*nvecsf_ll(I(i),:)';% Tractions
    Tn = -T'*nvecsf_ll(I(i),:)';% Normal Tractions
    Tmag = (T(1)^2 + T(2)^2 + T(3)^2)^(1/2);% Traction Magnitudes
    DC = Tn/Tmag;% Direction Cosines
    theta = acos(DC);% Traction angle from normal vector
    Ts = Tmag*sin(theta);% Shear Tractions
    STsf_ll(i,1) = Ts/Tn;% Slip Tendencies
    DTsf_ll(i,1) = ((-PS3km(1,1))-(Tn))/((-PS3km(1,1))-(-PS3km(3,3)));% Dilation Tendencies
    Dispsf_ll(i,1) = (out2.fault.disp{1,6}(I(i),5)^2 + out2.fault.disp{1,6}(I(i),8)^2)^.5;% Displacements
end
% Sort elements from south to north
[ys,I] = sort(XYZsf_ll(:,2));
XYZsf_ll = XYZsf_ll(I,:);
STsf_ll = STsf_ll(I,1);
DTsf_ll = DTsf_ll(I,1);
Dispsf_ll = Dispsf_ll(I,1);
% Displacement Gradientws
clear dg
for i=1:length(I)
    if i==1
        dg(i,1) = Dispsf_ll(i)/100;
    else
        dg(i,1) = abs(Dispsf_ll(i)-Dispsf_ll(i-1))/(((XYZsf_ll(i,1)-XYZsf_ll(i-1,1))^2 + (XYZsf_ll(i,2)-XYZsf_ll(i-1,2))^2 + (XYZsf_ll(i,3)-XYZsf_ll(i-1,3))^2)^.5);
    end
end
dg(i+1) = Dispsf_ll(i)/100;
for i=1:length(I)
    DGsf_ll(i,1) = mean([dg(i) dg(i+1)]);
end
% Find observation points closest to fault
clear Dobs
clear J
for i=1:length(I)
    for j=1:length(out2.obs.pstress{1})
        Dobs(j) = ((XYZsf_ll(i,1)-out2.obs.pstress{1}(j,1))^2 + (XYZsf_ll(i,2)-out2.obs.pstress{1}(j,2))^2)^.5;
    end
    J(i,1) = find(Dobs==min(Dobs));
    J(i,2) = i;
end
% Find unique observation points and closest element center
Lsf_ll(:,1) = unique(J(:,1));
for i=1:length(Lsf_ll)
    I = find(J(:,1)==Lsf_ll(i));
    clear Dobs
    for j=1:length(I)
        Dobs(j) = ((XYZsf_ll(J(I(j),2),1)-out1.obs.pstress{1}(Lsf_ll(i),2))^2 + (XYZsf_ll(J(I(j),2),2)-out1.obs.pstress{1}(Lsf_ll(i),2))^2)^.5;
    end
    K = find(Dobs==min(Dobs));
    Lsf_ll(i,2) = J(I(K),2);
end

%% combine fault layers
L_ul = [Lgm_ul;Lc_ul;Ls_ul;Lsf_ul];
STall_ul = [STgm_ul(Lgm_ul(:,2));STc_ul(Lc_ul(:,2));STs_ul(Ls_ul(:,2));STsf_ul(Lsf_ul(:,2))];
DTall_ul = [DTgm_ul(Lgm_ul(:,2));DTc_ul(Lc_ul(:,2));DTs_ul(Ls_ul(:,2));DTsf_ul(Lsf_ul(:,2))];
Dispall_ul = [Dispgm_ul(Lgm_ul(:,2));Dispc_ul(Lc_ul(:,2));Disps_ul(Ls_ul(:,2));Dispsf_ul(Lsf_ul(:,2))];
DGall_ul = [DGgm_ul(Lgm_ul(:,2));DGc_ul(Lc_ul(:,2));DGs_ul(Ls_ul(:,2));DGsf_ul(Lsf_ul(:,2))];

L_ll = [Lgm_ll;Lc_ll;Ls_ll;Lsf_ll];
STall_ll = [STgm_ll(Lgm_ll(:,2));STc_ll(Lc_ll(:,2));STs_ll(Ls_ll(:,2));STsf_ll(Lsf_ll(:,2))];
DTall_ll = [DTgm_ll(Lgm_ll(:,2));DTc_ll(Lc_ll(:,2));DTs_ll(Ls_ll(:,2));DTsf_ll(Lsf_ll(:,2))];
Dispall_ll = [Dispgm_ll(Lgm_ll(:,2));Dispc_ll(Lc_ll(:,2));Disps_ll(Ls_ll(:,2));Dispsf_ll(Lsf_ll(:,2))];
DGall_ll = [DGgm_ll(Lgm_ll(:,2));DGc_ll(Lc_ll(:,2));DGs_ll(Ls_ll(:,2));DGsf_ll(Lsf_ll(:,2))];

% put fault layers in on map coordinates
k=1;
for i=1:length(out1.obs.pstress{1})
    if find(L_ul(:,1)==i)
        if STall_ul(k)>1
            ST_ul(i,1) = 1;
        elseif STall_ul(k)<0
            ST_ul(i,1) = 0;
        else
            ST_ul(i,1) = STall_ul(k);
        end
        DT_ul(i,1) = DTall_ul(k);
        Disp_ul(i,1) = Dispall_ul(k);
        DG_ul(i,1) = DGall_ul(k);
        k=k+1;
    else
        ST_ul(i,1) = 0;
        DT_ul(i,1) = 0;
        Disp_ul(i,1) = 0;
        DG_ul(i,1) = 0;
    end
end
k=1;
for i=1:length(out2.obs.pstress{1})
    if find(L_ll(:,1)==i)
        if STall_ll(k)>1
            ST_ll(i,1) = 1;
        elseif STall_ll(k)<0
            ST_ll(i,1) = 0;
        else
            ST_ll(i,1) = STall_ll(k);
        end
        DT_ll(i,1) = DTall_ll(k);
        Disp_ll(i,1) = Dispall_ll(k);
        DG_ll(i,1) = DGall_ll(k);
        k=k+1;
    else
        ST_ll(i,1) = 0;
        DT_ll(i,1) = 0;
        Disp_ll(i,1) = 0;
        DG_ll(i,1) = 0;
    end
end

% FAULT_GRID --> X,Y,Z,slip tendency,dilation tendency,displacement gradient
for i=1:length(out1.obs.pstress{1})
    FAULT_GRID(i,1:3) = out1.obs.pstress{1}(i,1:3);
end
FAULT_GRID(:,4) = abs(ST_ul-ST_ll);
FAULT_GRID(:,5) = abs(DT_ul-DT_ll);
FAULT_GRID(:,6) = abs(Disp_ul-Disp_ll);
FAULT_GRID(:,7) = abs(DG_ul-DG_ll);

%% Combine Stress layers

% calculate Sc and S3
for i=1:length(out1.obs.pstress{1})
    Sc_ul(i,1) = (((-out1.obs.pstress{1}(i,15))-(-out1.obs.pstress{1}(i,7)))/2)*(1+.6^2)^.5 - .6*(((-out1.obs.pstress{1}(i,15))+(-out1.obs.pstress{1}(i,7)))/2);
    S3_ul(i,1) = (-out1.obs.pstress{1}(i,7));
    if find(L_ul(:,1)==i)
        Sc_ul(i,1) = NaN;
        S3_ul(i,1) = NaN;
    end
end
for i=1:length(out2.obs.pstress{1})
    Sc_ll(i,1) = (((-out2.obs.pstress{1}(i,15))-(-out2.obs.pstress{1}(i,7)))/2)*(1+.6^2)^.5 - .6*(((-out2.obs.pstress{1}(i,15))+(-out1.obs.pstress{1}(i,7)))/2);
    S3_ll(i,1) = (-out2.obs.pstress{1}(i,7));
    if find(L_ll(:,1)==i)
        Sc_ll(i,1) = NaN;
        S3_ll(i,1) = NaN;
    end
end

% extract stress values to compute means and standard deviations
k=1;
for i=1:length(Sc_ul)
    if isnan(Sc_ul(i)-Sc_ll(i))
        Sc(i,1) = 0;
        S3(i,1) = 0;
    else
        Sc(i,1) = abs(Sc_ul(i)-Sc_ll(i));
        S3(i,1) = abs(S3_ul(i)-S3_ll(i));
        ScReal(k,1) = abs(Sc_ul(i)-Sc_ll(i));
        S3Real(k,1) = abs(S3_ul(i)-S3_ll(i));
        k=k+1;
    end
end

% Mean and Standard Deviation
% Sc
Scmean = mean(ScReal);
SigSc = std(ScReal);
% Shmin
S3mean = mean(S3Real);
SigS3 = std(S3Real);

nsd = .05;% # of standard deviations to filter out anomolies

% Filter out Anomolies
k=1;
for i=1:length(ScReal)
    if ScReal(i)<(Scmean+nsd*SigSc)&&ScReal(i)>(Scmean-nsd*SigSc)
        Screal(k) = ScReal(i);
        Shminreal(k) = S3Real(i);
        k=k+1;
    end
end

nsd = 5;% # of standard deviations for favorability

% Mean and Standard Deviation
% Sc
Scmean = mean(Screal);
SigSc = std(Screal);
% Shmin
S3mean = mean(Shminreal);
SigS3 = std(Shminreal);

% STRESS_GRID --> X,Y,Z,maximum Coulomb shear stress absolute difference,Shmin absolute difference
STRESS_GRID(:,1:3) = out1.obs.pstress{1}(:,1:3);
STRESS_GRID(:,4) = Sc;
STRESS_GRID(:,5) = S3;

%% Permeability Model Layer Normalization

% Fault Layers
Fault.sliptendency(:,1:3) = FAULT_GRID(:,1:3);
Fault.sliptendency(:,4) = FAULT_GRID(:,4)/max(FAULT_GRID(:,4));
Fault.dilationtendency(:,1:3) = FAULT_GRID(:,1:3);
Fault.dilationtendency(:,4) = FAULT_GRID(:,5)/max(FAULT_GRID(:,5));
% displacement gradient
Fault.displacement(:,1:3) = FAULT_GRID(:,1:3);
Fault.displacement(:,4) = FAULT_GRID(:,6)/max(FAULT_GRID(:,6));
% displacement gradient
Fault.displacementgradient(:,1:3) = FAULT_GRID(:,1:3);
Fault.displacementgradient(:,4) = FAULT_GRID(:,7)/max(FAULT_GRID(:,7));

% Stress Layers
% normalization between +/- X standard deviations
Stress.MaximumCoulombShearStress(:,1:3) = STRESS_GRID(:,1:3);
% Stress.MaximumCoulombShearStress(:,4) = (STRESS_GRID(:,4)-(Scmean-sdc*SigSc))/(2*sdc*SigSc);
Stress.MaximumCoulombShearStress(:,4) = STRESS_GRID(:,4)/(2*nsd*SigSc);
I = find(Stress.MaximumCoulombShearStress(:,4)<0);
Stress.MaximumCoulombShearStress(I,4) = 0;
I = find(Stress.MaximumCoulombShearStress(:,4)>1);
Stress.MaximumCoulombShearStress(I,4) = 1;
for i=1:length(L_ul)
    Stress.MaximumCoulombShearStress(L_ul(i,1),4) = 0;
end
for i=1:length(L_ll)
    Stress.MaximumCoulombShearStress(L_ll(i,1),4) = 0;
end

% Inverse Shmin values by adding 2*mean to negative values
Stress.Sigma3(:,1:3) = STRESS_GRID(:,1:3);
Stress.Sigma3(:,4) = STRESS_GRID(:,5);
% Stress.Sigma3(:,4) = (Stress.Sigma3(:,4)-(-Shminmean-sdh*SigShmin))/(2*sdh*SigShmin);
Stress.Sigma3(:,4) = Stress.Sigma3(:,4)/(2*nsd*SigS3);
I = find(Stress.Sigma3(:,4)<0);
Stress.Sigma3(I,4) = 0;
I = find(Stress.Sigma3(:,4)>1);
Stress.Sigma3(I,4) = 1;
for i=1:length(L_ul)
    Stress.Sigma3(L_ul(i,1),4) = 0;
end
for i=1:length(L_ll)
    Stress.Sigma3(L_ll(i,1),4) = 0;
end

%% FIGURES

figure
plot([1:numel(ScReal)],ScReal,'k-');ylabel('Sc')
hold on
plot([0 4500],[Scmean+2*nsd*SigSc Scmean+2*nsd*SigSc])
plot([0 4500],[0 0])
figure
plot([1:numel(S3Real)],S3Real,'k-');ylabel('S3')
hold on
plot([0 4500],[S3mean+2*nsd*SigS3 S3mean+2*nsd*SigS3])
plot([0 4500],[0 0])

figure
plot([1:numel(Stress.MaximumCoulombShearStress(:,4))],Stress.MaximumCoulombShearStress(:,4),'k.');ylabel('Sc sensitivity')
figure
plot([1:numel(Stress.Sigma3(:,4))],Stress.Sigma3(:,4),'k.');ylabel('S3 sensitivity')


% grid data for contour plots
for j=0:61
for i=0:66
    X(1+i,1+j) = out1.obs.pstress{1}((1+i)+67*j,1);
    Y(1+i,1+j) = out1.obs.pstress{1}((1+i)+67*j,2);
    STXY(1+i,1+j) = Fault.sliptendency((1+i)+67*j,4);
    DTXY(1+i,1+j) = Fault.dilationtendency((1+i)+67*j,4);
    DispXY(1+i,1+j) = Fault.displacement((1+i)+67*j,4);
    DGXY(1+i,1+j) = Fault.displacementgradient((1+i)+67*j,4);
    ScXY(1+i,1+j) = Stress.MaximumCoulombShearStress((1+i)+67*j,4);
    ShminXY(1+i,1+j) = Stress.Sigma3((1+i)+67*j,4);
    if find(L_ul(:,1)==((1+i)+67*j))
        ScXY(1+i,1+j) = NaN;
        ShminXY(1+i,1+j) = NaN;
    end
end
end

% clormap white-black
for i=1:50
    unccm(i,:) = [(1-i/50) (1-i/50) (1-i/50)];
end

% contour intervals
Vm = [0:.01:1];

% Sc sensitivity map
figure
contourf(X,Y,ScXY,Vm)
hold on
contour(X,Y,ScXY,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sc sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% S3 sensitivity map
figure
contourf(X,Y,ShminXY,Vm)
hold on
contour(X,Y,ShminXY,Vm)
c=colorbar;
set(get(c,'ylabel'),'String','Sigma3 sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% ST sensitivity map
figure
contourf(X,Y,STXY)
hold on
contour(X,Y,STXY)
c=colorbar;
set(get(c,'ylabel'),'String','Slip Tendency sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% DT sensitivity map
figure
contourf(X,Y,DTXY)
hold on
contour(X,Y,DTXY)
c=colorbar;
set(get(c,'ylabel'),'String','Dilation Tendency sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% Disp sensitivity map
figure
contourf(X,Y,DispXY)
hold on
contour(X,Y,DispXY)
c=colorbar;
set(get(c,'ylabel'),'String','Displacement sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)

% DG sensitivity map
figure
contourf(X,Y,DGXY)
hold on
contour(X,Y,DGXY)
c=colorbar;
set(get(c,'ylabel'),'String','Displacement Gradient sensitivity','Fontsize',14);
caxis([0 1]);
colormap(unccm);
axis equal
xlabel('easting(m)','fontsize',12)
ylabel('northing(m)','fontsize',12)